C     UMAT plane stress Von Mises plasicity model
c     umat.f
c     Copyright (C) 2023  Sergio Lucarini
c
      SUBROUTINE UMAT(STRESS,STATEV,DDSDDE,SSE,SPD,SCD,
     1           RPL,DDSDDT,DRPLDE,DRPLDT,
     2           STRAN,DSTRAN,TIME,DTIME,TEMP,DTEMP,PREDEF,
     2           DPRED,CMNAME,
     3           NDI,NSHR,NTENS,NSTATV,PROPS,NPROPS,COORDS,
     3           DROT,PNEWDT,
     4           CELENT,DFGRD0,DFGRD1,NOEL,NPT,
     5           LAYER,KSPT,KSTEP,KINC)

       INCLUDE 'ABA_PARAM.INC' 

      DIMENSiON STRESS(NTENS),STATEV(NSTATV),
     1     DDSDDE(NTENS,NTENS),DDSDDT(NTENS),DRPLDE(NTENS),
     2     STRAN(NTENS),DSTRAN(NTENS),TIME(2),PREDEF(1),DPRED(1),
     3     PROPS(NPROPS),DFGRD0(3,3),
     4     DFGRD1(3,3)
      
      real*8 stress3(6),dstran3_e(6),stress3_t(6)
      real*8 nflow3(6),eps_e(6),eps_e_t(6)
      real*8 ddsddeinv(3,3),ddsdde3(6,6),ddsdde3inv(6,6)
      real*8 newshear,newbulk,newhard,c33,e33t
      real*8 s_vm,s_hyd,syh,pseq_t,dpseq
      real*8 shear,bulk,EE,nu,sy,tol,tol2,rhs,R2
      integer ii,jj,itern,itern_max,iterps,iterps_max

c     MATERIAL PARAMETERS
      tol=1.0d-6
      tol2=1.0d-3
      itern_max=200
      iterps_max=20
      EE=props(1)
      nu=props(2)
      sy=props(3)
      hh=props(4)
      if(TIME(1).eq.0)then
      statev(1:7)=0.0d0
      endif

c     LOAD STATE
      eps_e_t(1:6)=statev(1:6)
      pseq_t=statev(7)
c     Stress3d
      if(ndi.eq.2)then
      stress3_t(1:2)=stress(1:2)
      stress3_t(4)=stress(3)
      stress3_t(3)=0.0d0
      stress3_t(5:6)=0.0d0
      else
      stress3_t(1:6)=stress(1:6)
      endif
      stress3(1:6)=stress3_t(1:6)
      dpseq=0.0d0

c     Loop for plane stress
      iterps=0
      R2=1.0d10
      itern=0
      do while((R2>tol2).and.(iterps<iterps_max))
      iterps=iterps+1

c     elastic strain accounting plane stress
      if(ndi.eq.2)then
      eps_e(1:2)=eps_e_t(1:2)+DSTRAN(1:2)
      eps_e(4)=eps_e_t(4)+DSTRAN(3)
      eps_e(5:6)=eps_e_t(5:6)
      if(iterps.eq.1)then
        e33t=eps_e_t(3)-nu/(1-nu)*(DSTRAN(1)+DSTRAN(2))
      else
        e33t=e33t-stress3(3)/c33
      endif
      eps_e(3)=e33t
      else
      eps_e(1:6)=eps_e_t(1:6)+DSTRAN(1:6)
      endif

c     Ddsdde3d
      shear=EE/(1.0d0+nu)/2.0d0
      bulk=(EE/(1.0d0-2.0d0*nu)-2.0d0*shear)/3.0d0
      ddsdde3(1:6,1:6)=0.0d0
      do ii=1,3
       do jj=1,3
        ddsdde3(ii,jj)=bulk
       enddo
       ddsdde3(ii,ii)=2.0d0*shear+bulk
      enddo
      do ii=4,6
       ddsdde3(ii,ii)=shear
      enddo
      c33=2.0d0*shear+bulk


c     Stress
      dstran3_e(1:6)=eps_e(1:6)-eps_e_t(1:6)
      stress3(1:6)=stress3_t(1:6)
      do ii=1,6
        do jj=1,6
          stress3(ii)=stress3(ii)+ddsdde3(ii,jj)*dstran3_e(jj)
        enddo
      enddo
     
c     Von mises
      s_vm=(stress3(1)-stress3(2))**2+(stress3(2)-stress3(3))**2
     1 +(stress3(3)-stress3(1))**2
      do ii=4,6
        s_vm=s_vm+6.d0*stress3(ii)**2
      enddo
      s_vm=sqrt(s_vm/2.0d0)



c     Hardening previous
      syh=sy+hh*pseq_t

c     yield
      dpseq=0.0d0
      itern=0
      if(s_vm.gt.(1.0d0+2.0d0*tol)*syh)then

c     Flow
      s_hyd=(stress3(1)+stress3(2)+stress3(3))/3.0d0
      nflow3(1:3)=(stress3(1:3)-s_hyd)/s_vm
      nflow3(4:6)=stress3(4:6)/s_vm
       
c     Newton
      rhs=s_vm-3.0d0*shear*dpseq-syh
      do while((abs(rhs).gt.tol*syh).and.(itern<itern_max))
        itern=itern+1
        dpseq=dpseq+rhs/(3.0d0*shear+hh)
        syh=sy+hh*(pseq_t+dpseq)
        rhs=s_vm-3.0d0*shear*dpseq-syh
      enddo
      if(itern.eq.itern_max)then
        write(*,*)'ERROR no conv plastic'
        pnewdt=0.5
      endif

c     Update in N flow direction
      stress3(1:3)=nflow3(1:3)*syh+s_hyd
      eps_e(1:3)=eps_e(1:3)-3.0d0/2.0d0*nflow3(1:3)*dpseq
      stress3(4:6)=nflow3(4:6)*syh
      eps_e(4:6)=eps_e(4:6)-3.0d0*nflow3(4:6)*dpseq

      
c     Tangent  
      newshear=shear*syh/s_vm
      newbulk=(EE/(1.0d0-2.0d0*nu)-2.0d0*newshear)/3.0d0
      newhard=3.0d0*shear*hh/(3.0d0*shear+hh)-3.0d0*newshear
      do ii=1,3
        do jj=1,3
          ddsdde3(ii,jj)=newbulk
        enddo
        ddsdde3(ii,ii)=2.0d0*newshear+newbulk
      enddo
      do ii=4,6
        ddsdde3(ii,ii)=newshear
      enddo
      do ii=1,6
        do jj=1,6
          ddsdde3(ii,jj)=ddsdde3(ii,jj)+newhard*nflow3(ii)*nflow3(jj)
        enddo
      enddo
      c33=2.d0*newshear+newbulk+newhard*nflow3(3)*nflow3(3)
      endif

c     Check if stress out of plane is zero
      if(ndi.eq.2)then
      R2=abs(stress3(3))
      else
      R2=0.0d0
      endif
      enddo
      if(iterps.eq.iterps_max)then
        write(*,*)'ERROR no conv plane'
        pnewdt=0.5
      endif

c     Tangent in 2d 
      if(ndi.eq.2)then
      call M66INV(ddsdde3,ddsdde3inv)
      ddsddeinv(1,1)=ddsdde3inv(1,1)
      ddsddeinv(1,2)=ddsdde3inv(1,2)
      ddsddeinv(2,1)=ddsdde3inv(2,1)
      ddsddeinv(2,2)=ddsdde3inv(2,2)
      ddsddeinv(1,3)=ddsdde3inv(1,4)
      ddsddeinv(2,3)=ddsdde3inv(2,4)
      ddsddeinv(3,1)=ddsdde3inv(4,1)
      ddsddeinv(3,2)=ddsdde3inv(4,2)
      ddsddeinv(3,3)=ddsdde3inv(4,4)
      call M33INV(ddsddeinv,ddsdde)
      else
      ddsdde(1:6,1:6)=ddsdde3(1:6,1:6)
      endif

c     Stree in 2d 
      if(ndi.eq.2)then
      stress(1:2)=stress3(1:2)
      stress(3)=stress3(4)
      else
      stress(1:6)=stress3(1:6)
      endif

c     State
      statev(1:6)=eps_e(1:6)
      statev(7)=pseq_t+dpseq

      return
      end subroutine umat


      SUBROUTINE M33INV(A,AINV)
      implicit none
      real*8 A(3,3)
      real*8 AINV(3,3)
      real*8 DET
      real*8 COFACTOR(3,3)
      DET =   A(1,1)*A(2,2)*A(3,3)  
     1       - A(1,1)*A(2,3)*A(3,2)  
     1       - A(1,2)*A(2,1)*A(3,3)  
     1       + A(1,2)*A(2,3)*A(3,1)  
     1       + A(1,3)*A(2,1)*A(3,2)  
     1       - A(1,3)*A(2,2)*A(3,1)
      COFACTOR(1,1) = +(A(2,2)*A(3,3)-A(2,3)*A(3,2))
      COFACTOR(1,2) = -(A(2,1)*A(3,3)-A(2,3)*A(3,1))
      COFACTOR(1,3) = +(A(2,1)*A(3,2)-A(2,2)*A(3,1))
      COFACTOR(2,1) = -(A(1,2)*A(3,3)-A(1,3)*A(3,2))
      COFACTOR(2,2) = +(A(1,1)*A(3,3)-A(1,3)*A(3,1))
      COFACTOR(2,3) = -(A(1,1)*A(3,2)-A(1,2)*A(3,1))
      COFACTOR(3,1) = +(A(1,2)*A(2,3)-A(1,3)*A(2,2))
      COFACTOR(3,2) = -(A(1,1)*A(2,3)-A(1,3)*A(2,1))
      COFACTOR(3,3) = +(A(1,1)*A(2,2)-A(1,2)*A(2,1))
      AINV=TRANSPOSE(COFACTOR)/DET
      RETURN
      END SUBROUTINE M33INV


      SUBROUTINE M66INV(A,AINV)
      implicit none
      real*8 A(6,6)
      real*8 AINV(6,6)
      real*8 DET, A11, A12, A13, A14, A15, A16, A21,
     1  A22, A23, A24,
     1    A25, A26, A31, A32, A33, A34, A35, A36,
     1  A41, A42, A43, A44, A45, A46,
     1    A51, A52, A53, A54, A55, A56, A61, A62,
     1  A63, A64, A65, A66
      real*8 COFACTOR(6,6)
      A11=A(1,1);A12=A(1,2);A13=A(1,3);
      A14=A(1,4);A15=A(1,5); A16=A(1,6)
      A21=A(2,1); A22=A(2,2); A23=A(2,3);
      A24=A(2,4);A25=A(2,5); A26=A(2,6)
      A31=A(3,1); A32=A(3,2); A33=A(3,3); 
      A34=A(3,4);A35=A(3,5); A36=A(3,6)
      A41=A(4,1); A42=A(4,2); A43=A(4,3); 
      A44=A(4,4);A45=A(4,5); A46=A(4,6)
      A51=A(5,1); A52=A(5,2); A53=A(5,3); 
      A54=A(5,4);A55=A(5,5); A56=A(5,6)
      A61=A(6,1); A62=A(6,2); A63=A(6,3); 
      A64=A(6,4);A65=A(6,5); A66=A(6,6)
      DET = -(A16*A25*A34*A43*A52-A15*A26*A34*A43*
     1 A52-A16*A24*A35*A43*
     1   A52+A14*A26*A35*A43*A52+A15*A24*A36*A43*
     1 A52-A14*A25*A36*A43*A52-A16*A25*  
     1   A33*A44*A52+A15*A26*A33*A44*A52+A16*A23*
     1 A35*A44*A52-A13*A26*A35*A44*
     1   A52-A15*A23*A36*A44*A52+A13*A25*A36*A44*
     1 A52+A16*A24*A33*A45*A52-A14*A26*  
     1   A33*A45*A52-A16*A23*A34*A45*A52+A13*A26*
     1 A34*A45*A52+A14*A23*A36*A45*
     1   A52-A13*A24*A36*A45*A52-A15*A24*A33*A46*
     1 A52+A14*A25*A33*A46*A52+A15*A23*  
     1   A34*A46*A52-A13*A25*A34*A46*A52-A14*A23*
     1 A35*A46*A52+A13*A24*A35*A46*
     1   A52-A16*A25*A34*A42*A53+A15*A26*A34*A42*
     1 A53+A16*A24*A35*A42*A53-A14*A26*  
     1   A35*A42*A53-A15*A24*A36*A42*A53+A14*A25*
     1 A36*A42*A53+A16*A25*A32*A44*
     1   A53-A15*A26*A32*A44*A53-A16*A22*A35*A44*
     1 A53+A12*A26*A35*A44*A53+A15*A22*  
     1   A36*A44*A53-A12*A25*A36*A44*A53-A16*A24*
     1 A32*A45*A53+A14*A26*A32*A45*
     1   A53+A16*A22*A34*A45*A53-A12*A26*A34*A45*
     1 A53-A14*A22*A36*A45*A53+A12*A24*  
     1   A36*A45*A53+A15*A24*A32*A46*A53-A14*A25*
     1 A32*A46*A53-A15*A22*A34*A46*
     1   A53+A12*A25*A34*A46*A53+A14*A22*A35*A46*
     1 A53-A12*A24*A35*A46*A53+A16*A25*  
     1   A33*A42*A54-A15*A26*A33*A42*A54-A16*A23*
     1 A35*A42*A54+A13*A26*A35*A42*
     1   A54+A15*A23*A36*A42*A54-A13*A25*A36*A42*
     1 A54-A16*A25*A32*A43*A54+A15*A26*  
     1   A32*A43*A54+A16*A22*A35*A43*A54-A12*A26*
     1 A35*A43*A54-A15*A22*A36*A43*
     1   A54+A12*A25*A36*A43*A54+A16*A23*A32*A45*
     1 A54-A13*A26*A32*A45*A54-A16*A22*  
     1   A33*A45*A54+A12*A26*A33*A45*A54+A13*A22*
     1 A36*A45*A54-A12*A23*A36*A45*
     1   A54-A15*A23*A32*A46*A54+A13*A25*A32*A46*
     1 A54+A15*A22*A33*A46*A54-A12*A25*  
     1   A33*A46*A54-A13*A22*A35*A46*A54+A12*A23*
     1 A35*A46*A54-A16*A24*A33*A42*
     1   A55+A14*A26*A33*A42*A55+A16*A23*A34*A42*
     1 A55-A13*A26*A34*A42*A55-A14*A23*  
     1   A36*A42*A55+A13*A24*A36*A42*A55+A16*A24*
     1 A32*A43*A55-A14*A26*A32*A43*
     1   A55-A16*A22*A34*A43*A55+A12*A26*A34*A43*
     1 A55+A14*A22*A36*A43*A55-A12*A24*  
     1   A36*A43*A55-A16*A23*A32*A44*A55+A13*A26*
     1 A32*A44*A55+A16*A22*A33*A44*
     1   A55-A12*A26*A33*A44*A55-A13*A22*A36*A44*
     1 A55+A12*A23*A36*A44*A55+A14*A23*  
     1   A32*A46*A55-A13*A24*A32*A46*A55-A14*A22*
     1 A33*A46*A55+A12*A24*A33*A46*
     1   A55+A13*A22*A34*A46*A55-A12*A23*A34*A46*
     1 A55+A15*A24*A33*A42*A56-A14*A25*  
     1   A33*A42*A56-A15*A23*A34*A42*A56+A13*A25*
     1 A34*A42*A56+A14*A23*A35*A42*
     1   A56-A13*A24*A35*A42*A56-A15*A24*A32*A43*
     1 A56+A14*A25*A32*A43*A56+A15*A22*  
     1   A34*A43*A56-A12*A25*A34*A43*A56-A14*A22*
     1 A35*A43*A56+A12*A24*A35*A43*
     1   A56+A15*A23*A32*A44*A56-A13*A25*A32*A44*
     1 A56-A15*A22*A33*A44*A56+A12*A25*  
     1   A33*A44*A56+A13*A22*A35*A44*A56-A12*A23*
     1 A35*A44*A56-A14*A23*A32*A45*
     1   A56+A13*A24*A32*A45*A56+A14*A22*A33*A45*
     1 A56-A12*A24*A33*A45*A56-A13*A22*  
     1   A34*A45*A56+A12*A23*A34*A45*A56)*A61+(A16*
     1 A25*A34*A43*A51-A15*A26*A34*    
     1   A43*A51-A16*A24*A35*A43*A51+A14*A26*A35*
     1 A43*A51+A15*A24*A36*A43*A51-A14*  
     1   A25*A36*A43*A51-A16*A25*A33*A44*A51+A15*
     1 A26*A33*A44*A51+A16*A23*A35*A44*  
     1   A51-A13*A26*A35*A44*A51-A15*A23*A36*A44*
     1 A51+A13*A25*A36*A44*A51+A16*A24*  
     1   A33*A45*A51-A14*A26*A33*A45*A51-A16*A23*
     1 A34*A45*A51+A13*A26*A34*A45*
     1   A51+A14*A23*A36*A45*A51-A13*A24*A36*A45*
     1 A51-A15*A24*A33*A46*A51+A14*A25*  
     1   A33*A46*A51+A15*A23*A34*A46*A51-A13*A25*
     1 A34*A46*A51-A14*A23*A35*A46*
     1   A51+A13*A24*A35*A46*A51-A16*A25*A34*A41*
     1 A53+A15*A26*A34*A41*A53+A16*A24*  
     1   A35*A41*A53-A14*A26*A35*A41*A53-A15*A24*
     1 A36*A41*A53+A14*A25*A36*A41*
     1   A53+A16*A25*A31*A44*A53-A15*A26*A31*A44*
     1 A53-A16*A21*A35*A44*A53+A11*A26*  
     1   A35*A44*A53+A15*A21*A36*A44*A53-A11*A25*
     1 A36*A44*A53-A16*A24*A31*A45*
     1   A53+A14*A26*A31*A45*A53+A16*A21*A34*A45*
     1 A53-A11*A26*A34*A45*A53-A14*A21*  
     1   A36*A45*A53+A11*A24*A36*A45*A53+A15*A24*
     1 A31*A46*A53-A14*A25*A31*A46*
     1   A53-A15*A21*A34*A46*A53+A11*A25*A34*A46*
     1 A53+A14*A21*A35*A46*A53-A11*A24*  
     1   A35*A46*A53+A16*A25*A33*A41*A54-A15*A26*
     1 A33*A41*A54-A16*A23*A35*A41*
     1   A54+A13*A26*A35*A41*A54+A15*A23*A36*A41*
     1 A54-A13*A25*A36*A41*A54-A16*A25*  
     1   A31*A43*A54+A15*A26*A31*A43*A54+A16*A21*
     1 A35*A43*A54-A11*A26*A35*A43*
     1   A54-A15*A21*A36*A43*A54+A11*A25*A36*A43*
     1 A54+A16*A23*A31*A45*A54-A13*A26*  
     1   A31*A45*A54-A16*A21*A33*A45*A54+A11*A26*
     1 A33*A45*A54+A13*A21*A36*A45*
     1   A54-A11*A23*A36*A45*A54-A15*A23*A31*A46*
     1 A54+A13*A25*A31*A46*A54+A15*A21*  
     1   A33*A46*A54-A11*A25*A33*A46*A54-A13*A21*
     1 A35*A46*A54+A11*A23*A35*A46*
     1   A54-A16*A24*A33*A41*A55+A14*A26*A33*A41*
     1 A55+A16*A23*A34*A41*A55-A13*A26*  
     1   A34*A41*A55-A14*A23*A36*A41*A55+A13*A24*
     1 A36*A41*A55+A16*A24*A31*A43*
     1   A55-A14*A26*A31*A43*A55-A16*A21*A34*A43*
     1 A55+A11*A26*A34*A43*A55+A14*A21*  
     1   A36*A43*A55-A11*A24*A36*A43*A55-A16*A23*
     1 A31*A44*A55+A13*A26*A31*A44*
     1   A55+A16*A21*A33*A44*A55-A11*A26*A33*A44*
     1 A55-A13*A21*A36*A44*A55+A11*A23*  
     1   A36*A44*A55+A14*A23*A31*A46*A55-A13*A24*
     1 A31*A46*A55-A14*A21*A33*A46*
     1   A55+A11*A24*A33*A46*A55+A13*A21*A34*A46*
     1 A55-A11*A23*A34*A46*A55+A15*A24*  
     1   A33*A41*A56-A14*A25*A33*A41*A56-A15*A23*
     1 A34*A41*A56+A13*A25*A34*A41*
     1   A56+A14*A23*A35*A41*A56-A13*A24*A35*A41*
     1 A56-A15*A24*A31*A43*A56+A14*A25*  
     1   A31*A43*A56+A15*A21*A34*A43*A56-A11*A25*
     1 A34*A43*A56-A14*A21*A35*A43*
     1   A56+A11*A24*A35*A43*A56+A15*A23*A31*A44*
     1 A56-A13*A25*A31*A44*A56-A15*A21*  
     1   A33*A44*A56+A11*A25*A33*A44*A56+A13*A21*
     1 A35*A44*A56-A11*A23*A35*A44*
     1   A56-A14*A23*A31*A45*A56+A13*A24*A31*A45*
     1 A56+A14*A21*A33*A45*A56-A11*A24*  
     1   A33*A45*A56-A13*A21*A34*A45*A56+A11*A23*
     1 A34*A45*A56)*A62-(A16*A25*A34*    
     1   A42*A51-A15*A26*A34*A42*A51-A16*A24*A35*
     1 A42*A51+A14*A26*A35*A42*A51+A15*  
     1   A24*A36*A42*A51-A14*A25*A36*A42*A51-A16*
     1 A25*A32*A44*A51+A15*A26*A32*A44*  
     1   A51+A16*A22*A35*A44*A51-A12*A26*A35*A44*
     1 A51-A15*A22*A36*A44*A51+A12*A25*  
     1   A36*A44*A51+A16*A24*A32*A45*A51-A14*A26*
     1 A32*A45*A51-A16*A22*A34*A45*
     1   A51+A12*A26*A34*A45*A51+A14*A22*A36*A45*
     1 A51-A12*A24*A36*A45*A51-A15*A24*  
     1   A32*A46*A51+A14*A25*A32*A46*A51+A15*A22*
     1 A34*A46*A51-A12*A25*A34*A46*
     1   A51-A14*A22*A35*A46*A51+A12*A24*A35*A46*
     1 A51-A16*A25*A34*A41*A52+A15*A26*  
     1   A34*A41*A52+A16*A24*A35*A41*A52-A14*A26*
     1 A35*A41*A52-A15*A24*A36*A41*
     1   A52+A14*A25*A36*A41*A52+A16*A25*A31*A44*
     1 A52-A15*A26*A31*A44*A52-A16*A21*  
     1   A35*A44*A52+A11*A26*A35*A44*A52+A15*A21*
     1 A36*A44*A52-A11*A25*A36*A44*
     1   A52-A16*A24*A31*A45*A52+A14*A26*A31*A45*
     1 A52+A16*A21*A34*A45*A52-A11*A26*  
     1   A34*A45*A52-A14*A21*A36*A45*A52+A11*A24*
     1 A36*A45*A52+A15*A24*A31*A46*
     1   A52-A14*A25*A31*A46*A52-A15*A21*A34*A46*
     1 A52+A11*A25*A34*A46*A52+A14*A21*  
     1   A35*A46*A52-A11*A24*A35*A46*A52+A16*A25*
     1 A32*A41*A54-A15*A26*A32*A41*
     1   A54-A16*A22*A35*A41*A54+A12*A26*A35*A41*
     1 A54+A15*A22*A36*A41*A54-A12*A25*  
     1   A36*A41*A54-A16*A25*A31*A42*A54+A15*A26*
     1 A31*A42*A54+A16*A21*A35*A42*
     1   A54-A11*A26*A35*A42*A54-A15*A21*A36*A42*
     1 A54+A11*A25*A36*A42*A54+A16*A22*  
     1   A31*A45*A54-A12*A26*A31*A45*A54-A16*A21*
     1 A32*A45*A54+A11*A26*A32*A45*
     1   A54+A12*A21*A36*A45*A54-A11*A22*A36*A45*
     1 A54-A15*A22*A31*A46*A54+A12*A25*  
     1   A31*A46*A54+A15*A21*A32*A46*A54-A11*A25*
     1 A32*A46*A54-A12*A21*A35*A46*
     1   A54+A11*A22*A35*A46*A54-A16*A24*A32*A41*
     1 A55+A14*A26*A32*A41*A55+A16*A22*  
     1   A34*A41*A55-A12*A26*A34*A41*A55-A14*A22*
     1 A36*A41*A55+A12*A24*A36*A41*
     1   A55+A16*A24*A31*A42*A55-A14*A26*A31*A42*
     1 A55-A16*A21*A34*A42*A55+A11*A26*  
     1   A34*A42*A55+A14*A21*A36*A42*A55-A11*A24*
     1 A36*A42*A55-A16*A22*A31*A44*
     1   A55+A12*A26*A31*A44*A55+A16*A21*A32*A44*
     1 A55-A11*A26*A32*A44*A55-A12*A21*  
     1   A36*A44*A55+A11*A22*A36*A44*A55+A14*A22*
     1 A31*A46*A55-A12*A24*A31*A46*
     1   A55-A14*A21*A32*A46*A55+A11*A24*A32*A46*
     1 A55+A12*A21*A34*A46*A55-A11*A22*  
     1   A34*A46*A55+A15*A24*A32*A41*A56-A14*A25*
     1 A32*A41*A56-A15*A22*A34*A41*
     1   A56+A12*A25*A34*A41*A56+A14*A22*A35*A41*
     1 A56-A12*A24*A35*A41*A56-A15*A24*  
     1   A31*A42*A56+A14*A25*A31*A42*A56+A15*A21*
     1 A34*A42*A56-A11*A25*A34*A42*
     1   A56-A14*A21*A35*A42*A56+A11*A24*A35*A42*
     1 A56+A15*A22*A31*A44*A56-A12*A25*  
     1   A31*A44*A56-A15*A21*A32*A44*A56+A11*A25*
     1 A32*A44*A56+A12*A21*A35*A44*
     1   A56-A11*A22*A35*A44*A56-A14*A22*A31*A45*
     1 A56+A12*A24*A31*A45*A56+A14*A21*  
     1   A32*A45*A56-A11*A24*A32*A45*A56-A12*A21*
     1 A34*A45*A56+A11*A22*A34*A45*A56)* 
     1   A63+(A16*A25*A33*A42*A51-A15*A26*A33*A42*
     1 A51-A16*A23*A35*A42*A51+A13*A26* 
     1   A35*A42*A51+A15*A23*A36*A42*A51-A13*A25*
     1 A36*A42*A51-A16*A25*A32*A43*
     1   A51+A15*A26*A32*A43*A51+A16*A22*A35*A43*
     1 A51-A12*A26*A35*A43*A51-A15*A22*  
     1   A36*A43*A51+A12*A25*A36*A43*A51+A16*A23*
     1 A32*A45*A51-A13*A26*A32*A45*
     1   A51-A16*A22*A33*A45*A51+A12*A26*A33*A45*
     1 A51+A13*A22*A36*A45*A51-A12*A23*  
     1   A36*A45*A51-A15*A23*A32*A46*A51+A13*A25*
     1 A32*A46*A51+A15*A22*A33*A46*
     1   A51-A12*A25*A33*A46*A51-A13*A22*A35*A46*
     1 A51+A12*A23*A35*A46*A51-A16*A25*  
     1   A33*A41*A52+A15*A26*A33*A41*A52+A16*A23*
     1 A35*A41*A52-A13*A26*A35*A41*
     1   A52-A15*A23*A36*A41*A52+A13*A25*A36*A41*
     1 A52+A16*A25*A31*A43*A52-A15*A26*  
     1   A31*A43*A52-A16*A21*A35*A43*A52+A11*A26*
     1 A35*A43*A52+A15*A21*A36*A43*
     1   A52-A11*A25*A36*A43*A52-A16*A23*A31*A45*
     1 A52+A13*A26*A31*A45*A52+A16*A21*  
     1   A33*A45*A52-A11*A26*A33*A45*A52-A13*A21*
     1 A36*A45*A52+A11*A23*A36*A45*
     1   A52+A15*A23*A31*A46*A52-A13*A25*A31*A46*
     1 A52-A15*A21*A33*A46*A52+A11*A25*  
     1   A33*A46*A52+A13*A21*A35*A46*A52-A11*A23*
     1 A35*A46*A52+A16*A25*A32*A41*
     1   A53-A15*A26*A32*A41*A53-A16*A22*A35*A41*
     1 A53+A12*A26*A35*A41*A53+A15*A22*  
     1   A36*A41*A53-A12*A25*A36*A41*A53-A16*A25*
     1 A31*A42*A53+A15*A26*A31*A42*
     1   A53+A16*A21*A35*A42*A53-A11*A26*A35*A42*
     1 A53-A15*A21*A36*A42*A53+A11*A25*  
     1   A36*A42*A53+A16*A22*A31*A45*A53-A12*A26*
     1 A31*A45*A53-A16*A21*A32*A45*
     1   A53+A11*A26*A32*A45*A53+A12*A21*A36*A45*
     1 A53-A11*A22*A36*A45*A53-A15*A22*  
     1   A31*A46*A53+A12*A25*A31*A46*A53+A15*A21*
     1 A32*A46*A53-A11*A25*A32*A46*
     1   A53-A12*A21*A35*A46*A53+A11*A22*A35*A46*
     1 A53-A16*A23*A32*A41*A55+A13*A26*  
     1   A32*A41*A55+A16*A22*A33*A41*A55-A12*A26*
     1 A33*A41*A55-A13*A22*A36*A41*
     1   A55+A12*A23*A36*A41*A55+A16*A23*A31*A42*
     1 A55-A13*A26*A31*A42*A55-A16*A21*  
     1   A33*A42*A55+A11*A26*A33*A42*A55+A13*A21*
     1 A36*A42*A55-A11*A23*A36*A42*
     1   A55-A16*A22*A31*A43*A55+A12*A26*A31*A43*
     1 A55+A16*A21*A32*A43*A55-A11*A26*  
     1   A32*A43*A55-A12*A21*A36*A43*A55+A11*A22*
     1 A36*A43*A55+A13*A22*A31*A46*
     1   A55-A12*A23*A31*A46*A55-A13*A21*A32*A46*
     1 A55+A11*A23*A32*A46*A55+A12*A21*  
     1   A33*A46*A55-A11*A22*A33*A46*A55+A15*A23*
     1 A32*A41*A56-A13*A25*A32*A41*
     1   A56-A15*A22*A33*A41*A56+A12*A25*A33*A41*
     1 A56+A13*A22*A35*A41*A56-A12*A23*  
     1   A35*A41*A56-A15*A23*A31*A42*A56+A13*A25*
     1 A31*A42*A56+A15*A21*A33*A42*
     1   A56-A11*A25*A33*A42*A56-A13*A21*A35*A42*
     1 A56+A11*A23*A35*A42*A56+A15*A22*  
     1   A31*A43*A56-A12*A25*A31*A43*A56-A15*A21*
     1 A32*A43*A56+A11*A25*A32*A43*
     1   A56+A12*A21*A35*A43*A56-A11*A22*A35*A43*
     1 A56-A13*A22*A31*A45*A56+A12*A23*  
     1   A31*A45*A56+A13*A21*A32*A45*A56-A11*A23*
     1 A32*A45*A56-A12*A21*A33*A45*
     1   A56+A11*A22*A33*A45*A56)*A64-(A16*A24*A33*
     1 A42*A51-A14*A26*A33*A42*  
     1   A51-A16*A23*A34*A42*A51+A13*A26*A34*A42*
     1 A51+A14*A23*A36*A42*A51-A13*A24*  
     1   A36*A42*A51-A16*A24*A32*A43*A51+A14*A26*
     1 A32*A43*A51+A16*A22*A34*A43*
     1   A51-A12*A26*A34*A43*A51-A14*A22*A36*A43*
     1 A51+A12*A24*A36*A43*A51+A16*A23*  
     1   A32*A44*A51-A13*A26*A32*A44*A51-A16*A22*
     1 A33*A44*A51+A12*A26*A33*A44*
     1   A51+A13*A22*A36*A44*A51-A12*A23*A36*A44*
     1 A51-A14*A23*A32*A46*A51+A13*A24*  
     1   A32*A46*A51+A14*A22*A33*A46*A51-A12*A24*
     1 A33*A46*A51-A13*A22*A34*A46*
     1   A51+A12*A23*A34*A46*A51-A16*A24*A33*A41*
     1 A52+A14*A26*A33*A41*A52+A16*A23*  
     1   A34*A41*A52-A13*A26*A34*A41*A52-A14*A23*
     1 A36*A41*A52+A13*A24*A36*A41*
     1   A52+A16*A24*A31*A43*A52-A14*A26*A31*A43*
     1 A52-A16*A21*A34*A43*A52+A11*A26*  
     1   A34*A43*A52+A14*A21*A36*A43*A52-A11*A24*
     1 A36*A43*A52-A16*A23*A31*A44*
     1   A52+A13*A26*A31*A44*A52+A16*A21*A33*A44*
     1 A52-A11*A26*A33*A44*A52-A13*A21*  
     1   A36*A44*A52+A11*A23*A36*A44*A52+A14*A23*
     1 A31*A46*A52-A13*A24*A31*A46*
     1   A52-A14*A21*A33*A46*A52+A11*A24*A33*A46*
     1 A52+A13*A21*A34*A46*A52-A11*A23*  
     1   A34*A46*A52+A16*A24*A32*A41*A53-A14*A26*
     1 A32*A41*A53-A16*A22*A34*A41*
     1   A53+A12*A26*A34*A41*A53+A14*A22*A36*A41*
     1 A53-A12*A24*A36*A41*A53-A16*A24*  
     1   A31*A42*A53+A14*A26*A31*A42*A53+A16*A21*
     1 A34*A42*A53-A11*A26*A34*A42*
     1   A53-A14*A21*A36*A42*A53+A11*A24*A36*A42*
     1 A53+A16*A22*A31*A44*A53-A12*A26*  
     1   A31*A44*A53-A16*A21*A32*A44*A53+A11*A26*
     1 A32*A44*A53+A12*A21*A36*A44*
     1   A53-A11*A22*A36*A44*A53-A14*A22*A31*A46*
     1 A53+A12*A24*A31*A46*A53+A14*A21*  
     1   A32*A46*A53-A11*A24*A32*A46*A53-A12*A21*
     1 A34*A46*A53+A11*A22*A34*A46*
     1   A53-A16*A23*A32*A41*A54+A13*A26*A32*A41*
     1 A54+A16*A22*A33*A41*A54-A12*A26*  
     1   A33*A41*A54-A13*A22*A36*A41*A54+A12*A23*
     1 A36*A41*A54+A16*A23*A31*A42*
     1   A54-A13*A26*A31*A42*A54-A16*A21*A33*A42*
     1 A54+A11*A26*A33*A42*A54+A13*A21*  
     1   A36*A42*A54-A11*A23*A36*A42*A54-A16*A22*
     1 A31*A43*A54+A12*A26*A31*A43*
     1   A54+A16*A21*A32*A43*A54-A11*A26*A32*A43*
     1 A54-A12*A21*A36*A43*A54+A11*A22*  
     1   A36*A43*A54+A13*A22*A31*A46*A54-A12*A23*
     1 A31*A46*A54-A13*A21*A32*A46*
     1   A54+A11*A23*A32*A46*A54+A12*A21*A33*A46*
     1 A54-A11*A22*A33*A46*A54+A14*A23*  
     1   A32*A41*A56-A13*A24*A32*A41*A56-A14*A22*
     1 A33*A41*A56+A12*A24*A33*A41*
     1   A56+A13*A22*A34*A41*A56-A12*A23*A34*A41*
     1 A56-A14*A23*A31*A42*A56+A13*A24*  
     1   A31*A42*A56+A14*A21*A33*A42*A56-A11*A24*
     1 A33*A42*A56-A13*A21*A34*A42*
     1   A56+A11*A23*A34*A42*A56+A14*A22*A31*A43*
     1 A56-A12*A24*A31*A43*A56-A14*A21*  
     1   A32*A43*A56+A11*A24*A32*A43*A56+A12*A21*
     1 A34*A43*A56-A11*A22*A34*A43*
     1   A56-A13*A22*A31*A44*A56+A12*A23*A31*A44*
     1 A56+A13*A21*A32*A44*A56-A11*A23*  
     1   A32*A44*A56-A12*A21*A33*A44*A56+A11*A22*
     1 A33*A44*A56)*A65+(A15*A24*A33*    
     1   A42*A51-A14*A25*A33*A42*A51-A15*A23*A34*
     1 A42*A51+A13*A25*A34*A42*A51+A14*  
     1   A23*A35*A42*A51-A13*A24*A35*A42*A51-A15*
     1 A24*A32*A43*A51+A14*A25*A32*A43*  
     1   A51+A15*A22*A34*A43*A51-A12*A25*A34*A43*
     1 A51-A14*A22*A35*A43*A51+A12*A24*  
     1   A35*A43*A51+A15*A23*A32*A44*A51-A13*A25*
     1 A32*A44*A51-A15*A22*A33*A44*
     1   A51+A12*A25*A33*A44*A51+A13*A22*A35*A44*
     1 A51-A12*A23*A35*A44*A51-A14*A23*  
     1   A32*A45*A51+A13*A24*A32*A45*A51+A14*A22*
     1 A33*A45*A51-A12*A24*A33*A45*
     1   A51-A13*A22*A34*A45*A51+A12*A23*A34*A45*
     1 A51-A15*A24*A33*A41*A52+A14*A25*  
     1   A33*A41*A52+A15*A23*A34*A41*A52-A13*A25*
     1 A34*A41*A52-A14*A23*A35*A41*
     1   A52+A13*A24*A35*A41*A52+A15*A24*A31*A43*
     1 A52-A14*A25*A31*A43*A52-A15*A21*  
     1   A34*A43*A52+A11*A25*A34*A43*A52+A14*A21*
     1 A35*A43*A52-A11*A24*A35*A43*
     1   A52-A15*A23*A31*A44*A52+A13*A25*A31*A44*
     1 A52+A15*A21*A33*A44*A52-A11*A25*  
     1   A33*A44*A52-A13*A21*A35*A44*A52+A11*A23*
     1 A35*A44*A52+A14*A23*A31*A45*
     1   A52-A13*A24*A31*A45*A52-A14*A21*A33*A45*
     1 A52+A11*A24*A33*A45*A52+A13*A21*  
     1   A34*A45*A52-A11*A23*A34*A45*A52+A15*A24*
     1 A32*A41*A53-A14*A25*A32*A41*
     1   A53-A15*A22*A34*A41*A53+A12*A25*A34*A41*
     1 A53+A14*A22*A35*A41*A53-A12*A24*  
     1   A35*A41*A53-A15*A24*A31*A42*A53+A14*A25*
     1 A31*A42*A53+A15*A21*A34*A42*
     1   A53-A11*A25*A34*A42*A53-A14*A21*A35*A42*
     1 A53+A11*A24*A35*A42*A53+A15*A22*  
     1   A31*A44*A53-A12*A25*A31*A44*A53-A15*A21*
     1 A32*A44*A53+A11*A25*A32*A44*
     1   A53+A12*A21*A35*A44*A53-A11*A22*A35*A44*
     1 A53-A14*A22*A31*A45*A53+A12*A24*  
     1   A31*A45*A53+A14*A21*A32*A45*A53-A11*A24*
     1 A32*A45*A53-A12*A21*A34*A45*
     1   A53+A11*A22*A34*A45*A53-A15*A23*A32*A41*
     1 A54+A13*A25*A32*A41*A54+A15*A22*  
     1   A33*A41*A54-A12*A25*A33*A41*A54-A13*A22*
     1 A35*A41*A54+A12*A23*A35*A41*
     1   A54+A15*A23*A31*A42*A54-A13*A25*A31*A42*
     1 A54-A15*A21*A33*A42*A54+A11*A25*  
     1   A33*A42*A54+A13*A21*A35*A42*A54-A11*A23*
     1 A35*A42*A54-A15*A22*A31*A43*
     1   A54+A12*A25*A31*A43*A54+A15*A21*A32*A43*
     1 A54-A11*A25*A32*A43*A54-A12*A21*  
     1   A35*A43*A54+A11*A22*A35*A43*A54+A13*A22*
     1 A31*A45*A54-A12*A23*A31*A45*
     1   A54-A13*A21*A32*A45*A54+A11*A23*A32*A45*
     1 A54+A12*A21*A33*A45*A54-A11*A22*  
     1   A33*A45*A54+A14*A23*A32*A41*A55-A13*A24*
     1 A32*A41*A55-A14*A22*A33*A41*
     1   A55+A12*A24*A33*A41*A55+A13*A22*A34*A41*
     1 A55-A12*A23*A34*A41*A55-A14*A23*  
     1   A31*A42*A55+A13*A24*A31*A42*A55+A14*A21*
     1 A33*A42*A55-A11*A24*A33*A42*
     1   A55-A13*A21*A34*A42*A55+A11*A23*A34*A42*
     1 A55+A14*A22*A31*A43*A55-A12*A24*  
     1   A31*A43*A55-A14*A21*A32*A43*A55+A11*A24*
     1 A32*A43*A55+A12*A21*A34*A43*
     1   A55-A11*A22*A34*A43*A55-A13*A22*A31*A44*
     1 A55+A12*A23*A31*A44*A55+A13*A21*  
     1   A32*A44*A55-A11*A23*A32*A44*A55-A12*A21*
     1 A33*A44*A55+A11*A22*A33*A44*A55)* 
     1   A66
      COFACTOR(1,1) = A26*A35*A44*A53*A62-A25*A36*
     1 A44*A53*A62-A26*A34*A45*A53*A62+A24*A36*     
     1 A45*A53*A62+A25*A34*A46*A53*A62-A24*A35*A46*
     1 A53*A62-A26*A35*A43*A54*   
     1 A62+A25*A36*A43*A54*A62+A26*A33*A45*A54*A62-
     1 A23*A36*A45*A54*A62-A25*A33*     
     1 A46*A54*A62+A23*A35*A46*A54*A62+A26*A34*A43*
     1 A55*A62-A24*A36*A43*A55*   
     1 A62-A26*A33*A44*A55*A62+A23*A36*A44*A55*A62+
     1 A24*A33*A46*A55*A62-A23*A34*     
     1 A46*A55*A62-A25*A34*A43*A56*A62+A24*A35*A43*
     1 A56*A62+A25*A33*A44*A56*   
     1 A62-A23*A35*A44*A56*A62-A24*A33*A45*A56*A62+
     1 A23*A34*A45*A56*A62-A26*A35*     
     1 A44*A52*A63+A25*A36*A44*A52*A63+A26*A34*A45*
     1 A52*A63-A24*A36*A45*A52*   
     1 A63-A25*A34*A46*A52*A63+A24*A35*A46*A52*A63+
     1 A26*A35*A42*A54*A63-A25*A36*     
     1 A42*A54*A63-A26*A32*A45*A54*A63+A22*A36*A45*
     1 A54*A63+A25*A32*A46*A54*   
     1 A63-A22*A35*A46*A54*A63-A26*A34*A42*A55*A63+
     1 A24*A36*A42*A55*A63+A26*A32*     
     1 A44*A55*A63-A22*A36*A44*A55*A63-A24*A32*A46*
     1 A55*A63+A22*A34*A46*A55*   
     1 A63+A25*A34*A42*A56*A63-A24*A35*A42*A56*A63-
     1 A25*A32*A44*A56*A63+A22*A35*     
     1 A44*A56*A63+A24*A32*A45*A56*A63-A22*A34*A45*
     1 A56*A63+A26*A35*A43*A52*   
     1 A64-A25*A36*A43*A52*A64-A26*A33*A45*A52*A64+
     1 A23*A36*A45*A52*A64+A25*A33*     
     1 A46*A52*A64-A23*A35*A46*A52*A64-A26*A35*A42*
     1 A53*A64+A25*A36*A42*A53*   
     1 A64+A26*A32*A45*A53*A64-A22*A36*A45*A53*A64-
     1 A25*A32*A46*A53*A64+A22*A35*     
     1 A46*A53*A64+A26*A33*A42*A55*A64-A23*A36*A42*
     1 A55*A64-A26*A32*A43*A55*   
     1 A64+A22*A36*A43*A55*A64+A23*A32*A46*A55*A64-
     1 A22*A33*A46*A55*A64-A25*A33*     
     1 A42*A56*A64+A23*A35*A42*A56*A64+A25*A32*A43*
     1 A56*A64-A22*A35*A43*A56*   
     1 A64-A23*A32*A45*A56*A64+A22*A33*A45*A56*A64-
     1 A26*A34*A43*A52*A65+A24*A36*     
     1 A43*A52*A65+A26*A33*A44*A52*A65-A23*A36*A44*
     1 A52*A65-A24*A33*A46*A52*   
     1 A65+A23*A34*A46*A52*A65+A26*A34*A42*A53*A65-
     1 A24*A36*A42*A53*A65-A26*A32*     
     1 A44*A53*A65+A22*A36*A44*A53*A65+A24*A32*A46*
     1 A53*A65-A22*A34*A46*A53*   
     1 A65-A26*A33*A42*A54*A65+A23*A36*A42*A54*A65+
     1 A26*A32*A43*A54*A65-A22*A36*     
     1 A43*A54*A65-A23*A32*A46*A54*A65+A22*A33*A46*
     1 A54*A65+A24*A33*A42*A56*   
     1 A65-A23*A34*A42*A56*A65-A24*A32*A43*A56*A65+
     1 A22*A34*A43*A56*A65+A23*A32*     
     1 A44*A56*A65-A22*A33*A44*A56*A65+A25*A34*A43*
     1 A52*A66-A24*A35*A43*A52*   
     1 A66-A25*A33*A44*A52*A66+A23*A35*A44*A52*A66+
     1 A24*A33*A45*A52*A66-A23*A34*     
     1 A45*A52*A66-A25*A34*A42*A53*A66+A24*A35*A42*
     1 A53*A66+A25*A32*A44*A53*   
     1 A66-A22*A35*A44*A53*A66-A24*A32*A45*A53*A66+
     1 A22*A34*A45*A53*A66+A25*A33*     
     1 A42*A54*A66-A23*A35*A42*A54*A66-A25*A32*A43*
     1 A54*A66+A22*A35*A43*A54*   
     1 A66+A23*A32*A45*A54*A66-A22*A33*A45*A54*A66-
     1 A24*A33*A42*A55*A66+A23*A34*     
     1 A42*A55*A66+A24*A32*A43*A55*A66-A22*A34*A43*
     1 A55*A66-A23*A32*A44*A55*   
     1 A66+A22*A33*A44*A55*A66

      COFACTOR(2,1) = -A16*A35*A44*A53*A62+A15*A36*
     1 A44*A53*A62+A16*A34*
     1 A45*A53*A62-A14*A36*A45*A53*A62-A15*A34*A46*
     1 A53*A62+A14*A35*A46*A53*   
     1 A62+A16*A35*A43*A54*A62-A15*A36*A43*A54*A62-
     1 A16*A33*A45*A54*A62+A13*A36*     
     1 A45*A54*A62+A15*A33*A46*A54*A62-A13*A35*A46*
     1 A54*A62-A16*A34*A43*A55*   
     1 A62+A14*A36*A43*A55*A62+A16*A33*A44*A55*A62-
     1 A13*A36*A44*A55*A62-A14*A33*     
     1 A46*A55*A62+A13*A34*A46*A55*A62+A15*A34*A43*
     1 A56*A62-A14*A35*A43*A56*   
     1 A62-A15*A33*A44*A56*A62+A13*A35*A44*A56*A62+
     1 A14*A33*A45*A56*A62-A13*A34*     
     1 A45*A56*A62+A16*A35*A44*A52*A63-A15*A36*A44*
     1 A52*A63-A16*A34*A45*A52*   
     1 A63+A14*A36*A45*A52*A63+A15*A34*A46*A52*A63-
     1 A14*A35*A46*A52*A63-A16*A35*     
     1 A42*A54*A63+A15*A36*A42*A54*A63+A16*A32*A45*
     1 A54*A63-A12*A36*A45*A54*   
     1 A63-A15*A32*A46*A54*A63+A12*A35*A46*A54*A63+
     1 A16*A34*A42*A55*A63-A14*A36*     
     1 A42*A55*A63-A16*A32*A44*A55*A63+A12*A36*A44*
     1 A55*A63+A14*A32*A46*A55*   
     1 A63-A12*A34*A46*A55*A63-A15*A34*A42*A56*A63+
     1 A14*A35*A42*A56*A63+A15*A32*     
     1 A44*A56*A63-A12*A35*A44*A56*A63-A14*A32*A45*
     1 A56*A63+A12*A34*A45*A56*   
     1 A63-A16*A35*A43*A52*A64+A15*A36*A43*A52*A64+
     1 A16*A33*A45*A52*A64-A13*A36*     
     1 A45*A52*A64-A15*A33*A46*A52*A64+A13*A35*A46*
     1 A52*A64+A16*A35*A42*A53*   
     1 A64-A15*A36*A42*A53*A64-A16*A32*A45*A53*A64+
     1 A12*A36*A45*A53*A64+A15*A32*     
     1 A46*A53*A64-A12*A35*A46*A53*A64-A16*A33*A42*
     1 A55*A64+A13*A36*A42*A55*   
     1 A64+A16*A32*A43*A55*A64-A12*A36*A43*A55*A64-
     1 A13*A32*A46*A55*A64+A12*A33*     
     1 A46*A55*A64+A15*A33*A42*A56*A64-A13*A35*A42*
     1 A56*A64-A15*A32*A43*A56*   
     1 A64+A12*A35*A43*A56*A64+A13*A32*A45*A56*A64-
     1 A12*A33*A45*A56*A64+A16*A34*     
     1 A43*A52*A65-A14*A36*A43*A52*A65-A16*A33*A44*
     1 A52*A65+A13*A36*A44*A52*   
     1 A65+A14*A33*A46*A52*A65-A13*A34*A46*A52*A65-
     1 A16*A34*A42*A53*A65+A14*A36*     
     1 A42*A53*A65+A16*A32*A44*A53*A65-A12*A36*A44*
     1 A53*A65-A14*A32*A46*A53*   
     1 A65+A12*A34*A46*A53*A65+A16*A33*A42*A54*A65-
     1 A13*A36*A42*A54*A65-A16*A32*     
     1 A43*A54*A65+A12*A36*A43*A54*A65+A13*A32*A46*
     1 A54*A65-A12*A33*A46*A54*   
     1 A65-A14*A33*A42*A56*A65+A13*A34*A42*A56*A65+
     1 A14*A32*A43*A56*A65-A12*A34*     
     1 A43*A56*A65-A13*A32*A44*A56*A65+A12*A33*A44*
     1 A56*A65-A15*A34*A43*A52*   
     1 A66+A14*A35*A43*A52*A66+A15*A33*A44*A52*A66-
     1 A13*A35*A44*A52*A66-A14*A33*     
     1 A45*A52*A66+A13*A34*A45*A52*A66+A15*A34*A42*
     1 A53*A66-A14*A35*A42*A53*   
     1 A66-A15*A32*A44*A53*A66+A12*A35*A44*A53*A66+
     1 A14*A32*A45*A53*A66-A12*A34*     
     1 A45*A53*A66-A15*A33*A42*A54*A66+A13*A35*A42*
     1 A54*A66+A15*A32*A43*A54*   
     1 A66-A12*A35*A43*A54*A66-A13*A32*A45*A54*A66+
     1 A12*A33*A45*A54*A66+A14*A33*     
     1 A42*A55*A66-A13*A34*A42*A55*A66-A14*A32*A43*
     1 A55*A66+A12*A34*A43*A55*   
     1 A66+A13*A32*A44*A55*A66-A12*A33*A44*A55*A66

      COFACTOR(3,1) = A16*A25*A44*A53*A62-A15*A26*   
     1 A44*A53*A62-A16*A24*A45*A53*A62+A14*A26*A45*
     1 A53*A62+A15*A24*A46*A53*   
     1 A62-A14*A25*A46*A53*A62-A16*A25*A43*A54*A62+
     1 A15*A26*A43*A54*A62+A16*A23*     
     1 A45*A54*A62-A13*A26*A45*A54*A62-A15*A23*A46*
     1 A54*A62+A13*A25*A46*A54*   
     1 A62+A16*A24*A43*A55*A62-A14*A26*A43*A55*A62-
     1 A16*A23*A44*A55*A62+A13*A26*     
     1 A44*A55*A62+A14*A23*A46*A55*A62-A13*A24*A46*
     1 A55*A62-A15*A24*A43*A56*   
     1 A62+A14*A25*A43*A56*A62+A15*A23*A44*A56*A62-
     1 A13*A25*A44*A56*A62-A14*A23*     
     1 A45*A56*A62+A13*A24*A45*A56*A62-A16*A25*A44*
     1 A52*A63+A15*A26*A44*A52*   
     1 A63+A16*A24*A45*A52*A63-A14*A26*A45*A52*A63-
     1 A15*A24*A46*A52*A63+A14*A25*     
     1 A46*A52*A63+A16*A25*A42*A54*A63-A15*A26*A42*
     1 A54*A63-A16*A22*A45*A54*   
     1 A63+A12*A26*A45*A54*A63+A15*A22*A46*A54*A63-
     1 A12*A25*A46*A54*A63-A16*A24*     
     1 A42*A55*A63+A14*A26*A42*A55*A63+A16*A22*A44*
     1 A55*A63-A12*A26*A44*A55*   
     1 A63-A14*A22*A46*A55*A63+A12*A24*A46*A55*A63+
     1 A15*A24*A42*A56*A63-A14*A25*     
     1 A42*A56*A63-A15*A22*A44*A56*A63+A12*A25*A44*
     1 A56*A63+A14*A22*A45*A56*   
     1 A63-A12*A24*A45*A56*A63+A16*A25*A43*A52*A64-
     1 A15*A26*A43*A52*A64-A16*A23*     
     1 A45*A52*A64+A13*A26*A45*A52*A64+A15*A23*A46*
     1 A52*A64-A13*A25*A46*A52*   
     1 A64-A16*A25*A42*A53*A64+A15*A26*A42*A53*A64+
     1 A16*A22*A45*A53*A64-A12*A26*     
     1 A45*A53*A64-A15*A22*A46*A53*A64+A12*A25*A46*
     1 A53*A64+A16*A23*A42*A55*   
     1 A64-A13*A26*A42*A55*A64-A16*A22*A43*A55*A64+
     1 A12*A26*A43*A55*A64+A13*A22*     
     1 A46*A55*A64-A12*A23*A46*A55*A64-A15*A23*A42*
     1 A56*A64+A13*A25*A42*A56*   
     1 A64+A15*A22*A43*A56*A64-A12*A25*A43*A56*A64-
     1 A13*A22*A45*A56*A64+A12*A23*     
     1 A45*A56*A64-A16*A24*A43*A52*A65+A14*A26*A43*
     1 A52*A65+A16*A23*A44*A52*   
     1 A65-A13*A26*A44*A52*A65-A14*A23*A46*A52*A65+
     1 A13*A24*A46*A52*A65+A16*A24*     
     1 A42*A53*A65-A14*A26*A42*A53*A65-A16*A22*A44*
     1 A53*A65+A12*A26*A44*A53*   
     1 A65+A14*A22*A46*A53*A65-A12*A24*A46*A53*A65-
     1 A16*A23*A42*A54*A65+A13*A26*     
     1 A42*A54*A65+A16*A22*A43*A54*A65-A12*A26*A43*
     1 A54*A65-A13*A22*A46*A54*   
     1 A65+A12*A23*A46*A54*A65+A14*A23*A42*A56*A65-
     1 A13*A24*A42*A56*A65-A14*A22*     
     1 A43*A56*A65+A12*A24*A43*A56*A65+A13*A22*A44*
     1 A56*A65-A12*A23*A44*A56*   
     1 A65+A15*A24*A43*A52*A66-A14*A25*A43*A52*A66-
     1 A15*A23*A44*A52*A66+A13*A25*     
     1 A44*A52*A66+A14*A23*A45*A52*A66-A13*A24*A45*
     1 A52*A66-A15*A24*A42*A53*   
     1 A66+A14*A25*A42*A53*A66+A15*A22*A44*A53*A66-
     1 A12*A25*A44*A53*A66-A14*A22*     
     1 A45*A53*A66+A12*A24*A45*A53*A66+A15*A23*A42*
     1 A54*A66-A13*A25*A42*A54*   
     1 A66-A15*A22*A43*A54*A66+A12*A25*A43*A54*A66+
     1 A13*A22*A45*A54*A66-A12*A23*     
     1 A45*A54*A66-A14*A23*A42*A55*A66+A13*A24*A42*
     1 A55*A66+A14*A22*A43*A55*   
     1 A66-A12*A24*A43*A55*A66-A13*A22*A44*A55*A66+
     1 A12*A23*A44*A55*A66

      COFACTOR(4,1) = -A16*A25*    
     1 A34*A53*A62+A15*A26*A34*A53*A62+A16*A24*A35*
     1 A53*A62-A14*A26*A35*A53*   
     1 A62-A15*A24*A36*A53*A62+A14*A25*A36*A53*A62+
     1 A16*A25*A33*A54*A62-A15*A26*     
     1 A33*A54*A62-A16*A23*A35*A54*A62+A13*A26*A35*
     1 A54*A62+A15*A23*A36*A54*   
     1 A62-A13*A25*A36*A54*A62-A16*A24*A33*A55*A62+
     1 A14*A26*A33*A55*A62+A16*A23*     
     1 A34*A55*A62-A13*A26*A34*A55*A62-A14*A23*A36*
     1 A55*A62+A13*A24*A36*A55*   
     1 A62+A15*A24*A33*A56*A62-A14*A25*A33*A56*A62-
     1 A15*A23*A34*A56*A62+A13*A25*     
     1 A34*A56*A62+A14*A23*A35*A56*A62-A13*A24*A35*
     1 A56*A62+A16*A25*A34*A52*   
     1 A63-A15*A26*A34*A52*A63-A16*A24*A35*A52*A63+
     1 A14*A26*A35*A52*A63+A15*A24*     
     1 A36*A52*A63-A14*A25*A36*A52*A63-A16*A25*A32*
     1 A54*A63+A15*A26*A32*A54*   
     1 A63+A16*A22*A35*A54*A63-A12*A26*A35*A54*A63-
     1 A15*A22*A36*A54*A63+A12*A25*     
     1 A36*A54*A63+A16*A24*A32*A55*A63-A14*A26*A32*
     1 A55*A63-A16*A22*A34*A55*   
     1 A63+A12*A26*A34*A55*A63+A14*A22*A36*A55*A63-
     1 A12*A24*A36*A55*A63-A15*A24*     
     1 A32*A56*A63+A14*A25*A32*A56*A63+A15*A22*A34*
     1 A56*A63-A12*A25*A34*A56*   
     1 A63-A14*A22*A35*A56*A63+A12*A24*A35*A56*A63-
     1 A16*A25*A33*A52*A64+A15*A26*     
     1 A33*A52*A64+A16*A23*A35*A52*A64-A13*A26*A35*
     1 A52*A64-A15*A23*A36*A52*   
     1 A64+A13*A25*A36*A52*A64+A16*A25*A32*A53*A64-
     1 A15*A26*A32*A53*A64-A16*A22*     
     1 A35*A53*A64+A12*A26*A35*A53*A64+A15*A22*A36*
     1 A53*A64-A12*A25*A36*A53*   
     1 A64-A16*A23*A32*A55*A64+A13*A26*A32*A55*A64+
     1 A16*A22*A33*A55*A64-A12*A26*     
     1 A33*A55*A64-A13*A22*A36*A55*A64+A12*A23*A36*
     1 A55*A64+A15*A23*A32*A56*   
     1 A64-A13*A25*A32*A56*A64-A15*A22*A33*A56*A64+
     1 A12*A25*A33*A56*A64+A13*A22*     
     1 A35*A56*A64-A12*A23*A35*A56*A64+A16*A24*A33*
     1 A52*A65-A14*A26*A33*A52*   
     1 A65-A16*A23*A34*A52*A65+A13*A26*A34*A52*A65+
     1 A14*A23*A36*A52*A65-A13*A24*     
     1 A36*A52*A65-A16*A24*A32*A53*A65+A14*A26*A32*
     1 A53*A65+A16*A22*A34*A53*   
     1 A65-A12*A26*A34*A53*A65-A14*A22*A36*A53*A65+
     1 A12*A24*A36*A53*A65+A16*A23*     
     1 A32*A54*A65-A13*A26*A32*A54*A65-A16*A22*A33*
     1 A54*A65+A12*A26*A33*A54*   
     1 A65+A13*A22*A36*A54*A65-A12*A23*A36*A54*A65-
     1 A14*A23*A32*A56*A65+A13*A24*     
     1 A32*A56*A65+A14*A22*A33*A56*A65-A12*A24*A33*
     1 A56*A65-A13*A22*A34*A56*   
     1 A65+A12*A23*A34*A56*A65-A15*A24*A33*A52*A66+
     1 A14*A25*A33*A52*A66+A15*A23*     
     1 A34*A52*A66-A13*A25*A34*A52*A66-A14*A23*A35*
     1 A52*A66+A13*A24*A35*A52*   
     1 A66+A15*A24*A32*A53*A66-A14*A25*A32*A53*A66-
     1 A15*A22*A34*A53*A66+A12*A25*     
     1 A34*A53*A66+A14*A22*A35*A53*A66-A12*A24*A35*
     1 A53*A66-A15*A23*A32*A54*   
     1 A66+A13*A25*A32*A54*A66+A15*A22*A33*A54*A66-
     1 A12*A25*A33*A54*A66-A13*A22*     
     1 A35*A54*A66+A12*A23*A35*A54*A66+A14*A23*A32*
     1 A55*A66-A13*A24*A32*A55*   
     1 A66-A14*A22*A33*A55*A66+A12*A24*A33*A55*A66+
     1 A13*A22*A34*A55*A66-A12*A23*     
     1 A34*A55*A66

      COFACTOR(5,1) = A16*A25*A34*A43*A62-A15*A26*
     1 A34*A43*A62-A16*A24*A35*A43*     
     1 A62+A14*A26*A35*A43*A62+A15*A24*A36*A43*A62-
     1 A14*A25*A36*A43*A62-A16*A25*     
     1 A33*A44*A62+A15*A26*A33*A44*A62+A16*A23*A35*
     1 A44*A62-A13*A26*A35*A44*   
     1 A62-A15*A23*A36*A44*A62+A13*A25*A36*A44*A62+
     1 A16*A24*A33*A45*A62-A14*A26*     
     1 A33*A45*A62-A16*A23*A34*A45*A62+A13*A26*A34*
     1 A45*A62+A14*A23*A36*A45*   
     1 A62-A13*A24*A36*A45*A62-A15*A24*A33*A46*A62+
     1 A14*A25*A33*A46*A62+A15*A23*     
     1 A34*A46*A62-A13*A25*A34*A46*A62-A14*A23*A35*
     1 A46*A62+A13*A24*A35*A46*   
     1 A62-A16*A25*A34*A42*A63+A15*A26*A34*A42*A63+
     1 A16*A24*A35*A42*A63-A14*A26*     
     1 A35*A42*A63-A15*A24*A36*A42*A63+A14*A25*A36*
     1 A42*A63+A16*A25*A32*A44*   
     1 A63-A15*A26*A32*A44*A63-A16*A22*A35*A44*A63+
     1 A12*A26*A35*A44*A63+A15*A22*     
     1 A36*A44*A63-A12*A25*A36*A44*A63-A16*A24*A32*
     1 A45*A63+A14*A26*A32*A45*   
     1 A63+A16*A22*A34*A45*A63-A12*A26*A34*A45*A63-
     1 A14*A22*A36*A45*A63+A12*A24*     
     1 A36*A45*A63+A15*A24*A32*A46*A63-A14*A25*A32*
     1 A46*A63-A15*A22*A34*A46*   
     1 A63+A12*A25*A34*A46*A63+A14*A22*A35*A46*A63-
     1 A12*A24*A35*A46*A63+A16*A25*     
     1 A33*A42*A64-A15*A26*A33*A42*A64-A16*A23*A35*
     1 A42*A64+A13*A26*A35*A42*   
     1 A64+A15*A23*A36*A42*A64-A13*A25*A36*A42*A64-
     1 A16*A25*A32*A43*A64+A15*A26*     
     1 A32*A43*A64+A16*A22*A35*A43*A64-A12*A26*A35*
     1 A43*A64-A15*A22*A36*A43*   
     1 A64+A12*A25*A36*A43*A64+A16*A23*A32*A45*A64-
     1 A13*A26*A32*A45*A64-A16*A22*     
     1 A33*A45*A64+A12*A26*A33*A45*A64+A13*A22*A36*
     1 A45*A64-A12*A23*A36*A45*   
     1 A64-A15*A23*A32*A46*A64+A13*A25*A32*A46*A64+
     1 A15*A22*A33*A46*A64-A12*A25*     
     1 A33*A46*A64-A13*A22*A35*A46*A64+A12*A23*A35*
     1 A46*A64-A16*A24*A33*A42*   
     1 A65+A14*A26*A33*A42*A65+A16*A23*A34*A42*A65-
     1 A13*A26*A34*A42*A65-A14*A23*     
     1 A36*A42*A65+A13*A24*A36*A42*A65+A16*A24*A32*
     1 A43*A65-A14*A26*A32*A43*   
     1 A65-A16*A22*A34*A43*A65+A12*A26*A34*A43*A65+
     1 A14*A22*A36*A43*A65-A12*A24*     
     1 A36*A43*A65-A16*A23*A32*A44*A65+A13*A26*A32*
     1 A44*A65+A16*A22*A33*A44*   
     1 A65-A12*A26*A33*A44*A65-A13*A22*A36*A44*A65+
     1 A12*A23*A36*A44*A65+A14*A23*     
     1 A32*A46*A65-A13*A24*A32*A46*A65-A14*A22*A33*
     1 A46*A65+A12*A24*A33*A46*   
     1 A65+A13*A22*A34*A46*A65-A12*A23*A34*A46*A65+
     1 A15*A24*A33*A42*A66-A14*A25*     
     1 A33*A42*A66-A15*A23*A34*A42*A66+A13*A25*A34*
     1 A42*A66+A14*A23*A35*A42*   
     1 A66-A13*A24*A35*A42*A66-A15*A24*A32*A43*A66+
     1 A14*A25*A32*A43*A66+A15*A22*     
     1 A34*A43*A66-A12*A25*A34*A43*A66-A14*A22*A35*
     1 A43*A66+A12*A24*A35*A43*   
     1 A66+A15*A23*A32*A44*A66-A13*A25*A32*A44*A66-
     1 A15*A22*A33*A44*A66+A12*A25*     
     1 A33*A44*A66+A13*A22*A35*A44*A66-A12*A23*A35*
     1 A44*A66-A14*A23*A32*A45*   
     1 A66+A13*A24*A32*A45*A66+A14*A22*A33*A45*A66-
     1 A12*A24*A33*A45*A66-A13*A22*     
     1 A34*A45*A66+A12*A23*A34*A45*A66

      COFACTOR(6,1) = -A16*A25*A34*A43*A52+A15*A26*A34*A43*
     1 A52+A16*A24*A35*A43*A52-A14*A26*A35*A43*A52-
     1 A15*A24*A36*A43*A52+A14*A25*     
     1 A36*A43*A52+A16*A25*A33*A44*A52-A15*A26*A33*
     1 A44*A52-A16*A23*A35*A44*   
     1 A52+A13*A26*A35*A44*A52+A15*A23*A36*A44*A52-
     1 A13*A25*A36*A44*A52-A16*A24*     
     1 A33*A45*A52+A14*A26*A33*A45*A52+A16*A23*A34*
     1 A45*A52-A13*A26*A34*A45*   
     1 A52-A14*A23*A36*A45*A52+A13*A24*A36*A45*A52+
     1 A15*A24*A33*A46*A52-A14*A25*     
     1 A33*A46*A52-A15*A23*A34*A46*A52+A13*A25*A34*
     1 A46*A52+A14*A23*A35*A46*   
     1 A52-A13*A24*A35*A46*A52+A16*A25*A34*A42*A53-
     1 A15*A26*A34*A42*A53-A16*A24*     
     1 A35*A42*A53+A14*A26*A35*A42*A53+A15*A24*A36*
     1 A42*A53-A14*A25*A36*A42*   
     1 A53-A16*A25*A32*A44*A53+A15*A26*A32*A44*A53+
     1 A16*A22*A35*A44*A53-A12*A26*     
     1 A35*A44*A53-A15*A22*A36*A44*A53+A12*A25*A36*
     1 A44*A53+A16*A24*A32*A45*   
     1 A53-A14*A26*A32*A45*A53-A16*A22*A34*A45*A53+
     1 A12*A26*A34*A45*A53+A14*A22*     
     1 A36*A45*A53-A12*A24*A36*A45*A53-A15*A24*A32*
     1 A46*A53+A14*A25*A32*A46*   
     1 A53+A15*A22*A34*A46*A53-A12*A25*A34*A46*A53-
     1 A14*A22*A35*A46*A53+A12*A24*     
     1 A35*A46*A53-A16*A25*A33*A42*A54+A15*A26*A33*
     1 A42*A54+A16*A23*A35*A42*   
     1 A54-A13*A26*A35*A42*A54-A15*A23*A36*A42*A54+
     1 A13*A25*A36*A42*A54+A16*A25*     
     1 A32*A43*A54-A15*A26*A32*A43*A54-A16*A22*A35*
     1 A43*A54+A12*A26*A35*A43*   
     1 A54+A15*A22*A36*A43*A54-A12*A25*A36*A43*A54-
     1 A16*A23*A32*A45*A54+A13*A26*     
     1 A32*A45*A54+A16*A22*A33*A45*A54-A12*A26*A33*
     1 A45*A54-A13*A22*A36*A45*   
     1 A54+A12*A23*A36*A45*A54+A15*A23*A32*A46*A54-
     1 A13*A25*A32*A46*A54-A15*A22*     
     1 A33*A46*A54+A12*A25*A33*A46*A54+A13*A22*A35*
     1 A46*A54-A12*A23*A35*A46*   
     1 A54+A16*A24*A33*A42*A55-A14*A26*A33*A42*A55-
     1 A16*A23*A34*A42*A55+A13*A26*     
     1 A34*A42*A55+A14*A23*A36*A42*A55-A13*A24*A36*
     1 A42*A55-A16*A24*A32*A43*   
     1 A55+A14*A26*A32*A43*A55+A16*A22*A34*A43*A55-
     1 A12*A26*A34*A43*A55-A14*A22*     
     1 A36*A43*A55+A12*A24*A36*A43*A55+A16*A23*A32*
     1 A44*A55-A13*A26*A32*A44*   
     1 A55-A16*A22*A33*A44*A55+A12*A26*A33*A44*A55+
     1 A13*A22*A36*A44*A55-A12*A23*     
     1 A36*A44*A55-A14*A23*A32*A46*A55+A13*A24*A32*
     1 A46*A55+A14*A22*A33*A46*   
     1 A55-A12*A24*A33*A46*A55-A13*A22*A34*A46*A55+
     1 A12*A23*A34*A46*A55-A15*A24*     
     1 A33*A42*A56+A14*A25*A33*A42*A56+A15*A23*A34*
     1 A42*A56-A13*A25*A34*A42*   
     1 A56-A14*A23*A35*A42*A56+A13*A24*A35*A42*A56+
     1 A15*A24*A32*A43*A56-A14*A25*     
     1 A32*A43*A56-A15*A22*A34*A43*A56+A12*A25*A34*
     1 A43*A56+A14*A22*A35*A43*   
     1 A56-A12*A24*A35*A43*A56-A15*A23*A32*A44*A56+
     1 A13*A25*A32*A44*A56+A15*A22*     
     1 A33*A44*A56-A12*A25*A33*A44*A56-A13*A22*A35*
     1 A44*A56+A12*A23*A35*A44*   
     1 A56+A14*A23*A32*A45*A56-A13*A24*A32*A45*A56-
     1 A14*A22*A33*A45*A56+A12*A24*     
     1 A33*A45*A56+A13*A22*A34*A45*A56-A12*A23*A34*A45*A56

      COFACTOR(1,2) = -A26*A35*A44*A53*  
     1 A61+A25*A36*A44*A53*A61+A26*A34*A45*A53*A61-
     1 A24*A36*A45*A53*A61-A25*A34*     
     1 A46*A53*A61+A24*A35*A46*A53*A61+A26*A35*A43*
     1 A54*A61-A25*A36*A43*A54*   
     1 A61-A26*A33*A45*A54*A61+A23*A36*A45*A54*A61+
     1 A25*A33*A46*A54*A61-A23*A35*     
     1 A46*A54*A61-A26*A34*A43*A55*A61+A24*A36*A43*
     1 A55*A61+A26*A33*A44*A55*   
     1 A61-A23*A36*A44*A55*A61-A24*A33*A46*A55*A61+
     1 A23*A34*A46*A55*A61+A25*A34*     
     1 A43*A56*A61-A24*A35*A43*A56*A61-A25*A33*A44*
     1 A56*A61+A23*A35*A44*A56*   
     1 A61+A24*A33*A45*A56*A61-A23*A34*A45*A56*A61+
     1 A26*A35*A44*A51*A63-A25*A36*     
     1 A44*A51*A63-A26*A34*A45*A51*A63+A24*A36*A45*
     1 A51*A63+A25*A34*A46*A51*   
     1 A63-A24*A35*A46*A51*A63-A26*A35*A41*A54*A63+
     1 A25*A36*A41*A54*A63+A26*A31*     
     1 A45*A54*A63-A21*A36*A45*A54*A63-A25*A31*A46*
     1 A54*A63+A21*A35*A46*A54*   
     1 A63+A26*A34*A41*A55*A63-A24*A36*A41*A55*A63-
     1 A26*A31*A44*A55*A63+A21*A36*     
     1 A44*A55*A63+A24*A31*A46*A55*A63-A21*A34*A46*
     1 A55*A63-A25*A34*A41*A56*   
     1 A63+A24*A35*A41*A56*A63+A25*A31*A44*A56*A63-
     1 A21*A35*A44*A56*A63-A24*A31*     
     1 A45*A56*A63+A21*A34*A45*A56*A63-A26*A35*A43*
     1 A51*A64+A25*A36*A43*A51*   
     1 A64+A26*A33*A45*A51*A64-A23*A36*A45*A51*A64-
     1 A25*A33*A46*A51*A64+A23*A35*     
     1 A46*A51*A64+A26*A35*A41*A53*A64-A25*A36*A41*
     1 A53*A64-A26*A31*A45*A53*   
     1 A64+A21*A36*A45*A53*A64+A25*A31*A46*A53*A64-
     1 A21*A35*A46*A53*A64-A26*A33*     
     1 A41*A55*A64+A23*A36*A41*A55*A64+A26*A31*A43*
     1 A55*A64-A21*A36*A43*A55*   
     1 A64-A23*A31*A46*A55*A64+A21*A33*A46*A55*A64+
     1 A25*A33*A41*A56*A64-A23*A35*     
     1 A41*A56*A64-A25*A31*A43*A56*A64+A21*A35*A43*
     1 A56*A64+A23*A31*A45*A56*   
     1 A64-A21*A33*A45*A56*A64+A26*A34*A43*A51*A65-
     1 A24*A36*A43*A51*A65-A26*A33*     
     1 A44*A51*A65+A23*A36*A44*A51*A65+A24*A33*A46*
     1 A51*A65-A23*A34*A46*A51*   
     1 A65-A26*A34*A41*A53*A65+A24*A36*A41*A53*A65+
     1 A26*A31*A44*A53*A65-A21*A36*     
     1 A44*A53*A65-A24*A31*A46*A53*A65+A21*A34*A46*
     1 A53*A65+A26*A33*A41*A54*   
     1 A65-A23*A36*A41*A54*A65-A26*A31*A43*A54*A65+
     1 A21*A36*A43*A54*A65+A23*A31*     
     1 A46*A54*A65-A21*A33*A46*A54*A65-A24*A33*A41*
     1 A56*A65+A23*A34*A41*A56*   
     1 A65+A24*A31*A43*A56*A65-A21*A34*A43*A56*A65-
     1 A23*A31*A44*A56*A65+A21*A33*     
     1 A44*A56*A65-A25*A34*A43*A51*A66+A24*A35*A43*
     1 A51*A66+A25*A33*A44*A51*   
     1 A66-A23*A35*A44*A51*A66-A24*A33*A45*A51*A66+
     1 A23*A34*A45*A51*A66+A25*A34*     
     1 A41*A53*A66-A24*A35*A41*A53*A66-A25*A31*A44*
     1 A53*A66+A21*A35*A44*A53*   
     1 A66+A24*A31*A45*A53*A66-A21*A34*A45*A53*A66-
     1 A25*A33*A41*A54*A66+A23*A35*     
     1 A41*A54*A66+A25*A31*A43*A54*A66-A21*A35*A43*
     1 A54*A66-A23*A31*A45*A54*   
     1 A66+A21*A33*A45*A54*A66+A24*A33*A41*A55*A66-
     1 A23*A34*A41*A55*A66-A24*A31*     
     1 A43*A55*A66+A21*A34*A43*A55*A66+A23*A31*A44*
     1 A55*A66-A21*A33*A44*A55*   
     1 A66

      COFACTOR(2,2) = A16*A35*A44*A53*A61-A15*A36*
     1 A44*A53*A61-A16*A34*A45*A53*A61+A14*A36*     
     1 A45*A53*A61+A15*A34*A46*A53*A61-A14*A35*A46*
     1 A53*A61-A16*A35*A43*A54*   
     1 A61+A15*A36*A43*A54*A61+A16*A33*A45*A54*A61-
     1 A13*A36*A45*A54*A61-A15*A33*     
     1 A46*A54*A61+A13*A35*A46*A54*A61+A16*A34*A43*
     1 A55*A61-A14*A36*A43*A55*   
     1 A61-A16*A33*A44*A55*A61+A13*A36*A44*A55*A61+
     1 A14*A33*A46*A55*A61-A13*A34*     
     1 A46*A55*A61-A15*A34*A43*A56*A61+A14*A35*A43*
     1 A56*A61+A15*A33*A44*A56*   
     1 A61-A13*A35*A44*A56*A61-A14*A33*A45*A56*A61+
     1 A13*A34*A45*A56*A61-A16*A35*     
     1 A44*A51*A63+A15*A36*A44*A51*A63+A16*A34*A45*
     1 A51*A63-A14*A36*A45*A51*   
     1 A63-A15*A34*A46*A51*A63+A14*A35*A46*A51*A63+
     1 A16*A35*A41*A54*A63-A15*A36*     
     1 A41*A54*A63-A16*A31*A45*A54*A63+A11*A36*A45*
     1 A54*A63+A15*A31*A46*A54*   
     1 A63-A11*A35*A46*A54*A63-A16*A34*A41*A55*A63+
     1 A14*A36*A41*A55*A63+A16*A31*     
     1 A44*A55*A63-A11*A36*A44*A55*A63-A14*A31*A46*
     1 A55*A63+A11*A34*A46*A55*   
     1 A63+A15*A34*A41*A56*A63-A14*A35*A41*A56*A63-
     1 A15*A31*A44*A56*A63+A11*A35*     
     1 A44*A56*A63+A14*A31*A45*A56*A63-A11*A34*A45*
     1 A56*A63+A16*A35*A43*A51*   
     1 A64-A15*A36*A43*A51*A64-A16*A33*A45*A51*A64+
     1 A13*A36*A45*A51*A64+A15*A33*     
     1 A46*A51*A64-A13*A35*A46*A51*A64-A16*A35*A41*
     1 A53*A64+A15*A36*A41*A53*   
     1 A64+A16*A31*A45*A53*A64-A11*A36*A45*A53*A64-
     1 A15*A31*A46*A53*A64+A11*A35*     
     1 A46*A53*A64+A16*A33*A41*A55*A64-A13*A36*A41*
     1 A55*A64-A16*A31*A43*A55*   
     1 A64+A11*A36*A43*A55*A64+A13*A31*A46*A55*A64-
     1 A11*A33*A46*A55*A64-A15*A33*     
     1 A41*A56*A64+A13*A35*A41*A56*A64+A15*A31*A43*
     1 A56*A64-A11*A35*A43*A56*   
     1 A64-A13*A31*A45*A56*A64+A11*A33*A45*A56*A64-
     1 A16*A34*A43*A51*A65+A14*A36*     
     1 A43*A51*A65+A16*A33*A44*A51*A65-A13*A36*A44*
     1 A51*A65-A14*A33*A46*A51*   
     1 A65+A13*A34*A46*A51*A65+A16*A34*A41*A53*A65-
     1 A14*A36*A41*A53*A65-A16*A31*     
     1 A44*A53*A65+A11*A36*A44*A53*A65+A14*A31*A46*
     1 A53*A65-A11*A34*A46*A53*   
     1 A65-A16*A33*A41*A54*A65+A13*A36*A41*A54*A65+
     1 A16*A31*A43*A54*A65-A11*A36*     
     1 A43*A54*A65-A13*A31*A46*A54*A65+A11*A33*A46*
     1 A54*A65+A14*A33*A41*A56*   
     1 A65-A13*A34*A41*A56*A65-A14*A31*A43*A56*A65+
     1 A11*A34*A43*A56*A65+A13*A31*     
     1 A44*A56*A65-A11*A33*A44*A56*A65+A15*A34*A43*
     1 A51*A66-A14*A35*A43*A51*   
     1 A66-A15*A33*A44*A51*A66+A13*A35*A44*A51*A66+
     1 A14*A33*A45*A51*A66-A13*A34*     
     1 A45*A51*A66-A15*A34*A41*A53*A66+A14*A35*A41*
     1 A53*A66+A15*A31*A44*A53*   
     1 A66-A11*A35*A44*A53*A66-A14*A31*A45*A53*A66+
     1 A11*A34*A45*A53*A66+A15*A33*     
     1 A41*A54*A66-A13*A35*A41*A54*A66-A15*A31*A43*
     1 A54*A66+A11*A35*A43*A54*   
     1 A66+A13*A31*A45*A54*A66-A11*A33*A45*A54*A66-
     1 A14*A33*A41*A55*A66+A13*A34*     
     1 A41*A55*A66+A14*A31*A43*A55*A66-A11*A34*A43*
     1 A55*A66-A13*A31*A44*A55*   
     1 A66+A11*A33*A44*A55*A66

      COFACTOR(3,2) = -A16*A25*A44*A53*A61+A15*A26*
     1 A44*A53*A61+A16*A24*
     1 A45*A53*A61-A14*A26*A45*A53*A61-A15*A24*A46*
     1 A53*A61+A14*A25*A46*A53*   
     1 A61+A16*A25*A43*A54*A61-A15*A26*A43*A54*A61-
     1 A16*A23*A45*A54*A61+A13*A26*     
     1 A45*A54*A61+A15*A23*A46*A54*A61-A13*A25*A46*
     1 A54*A61-A16*A24*A43*A55*   
     1 A61+A14*A26*A43*A55*A61+A16*A23*A44*A55*A61-
     1 A13*A26*A44*A55*A61-A14*A23*     
     1 A46*A55*A61+A13*A24*A46*A55*A61+A15*A24*A43*
     1 A56*A61-A14*A25*A43*A56*   
     1 A61-A15*A23*A44*A56*A61+A13*A25*A44*A56*A61+
     1 A14*A23*A45*A56*A61-A13*A24*     
     1 A45*A56*A61+A16*A25*A44*A51*A63-A15*A26*A44*
     1 A51*A63-A16*A24*A45*A51*   
     1 A63+A14*A26*A45*A51*A63+A15*A24*A46*A51*A63-
     1 A14*A25*A46*A51*A63-A16*A25*     
     1 A41*A54*A63+A15*A26*A41*A54*A63+A16*A21*A45*
     1 A54*A63-A11*A26*A45*A54*   
     1 A63-A15*A21*A46*A54*A63+A11*A25*A46*A54*A63+
     1 A16*A24*A41*A55*A63-A14*A26*     
     1 A41*A55*A63-A16*A21*A44*A55*A63+A11*A26*A44*
     1 A55*A63+A14*A21*A46*A55*   
     1 A63-A11*A24*A46*A55*A63-A15*A24*A41*A56*A63+
     1 A14*A25*A41*A56*A63+A15*A21*     
     1 A44*A56*A63-A11*A25*A44*A56*A63-A14*A21*A45*
     1 A56*A63+A11*A24*A45*A56*   
     1 A63-A16*A25*A43*A51*A64+A15*A26*A43*A51*A64+
     1 A16*A23*A45*A51*A64-A13*A26*     
     1 A45*A51*A64-A15*A23*A46*A51*A64+A13*A25*A46*
     1 A51*A64+A16*A25*A41*A53*   
     1 A64-A15*A26*A41*A53*A64-A16*A21*A45*A53*A64+
     1 A11*A26*A45*A53*A64+A15*A21*     
     1 A46*A53*A64-A11*A25*A46*A53*A64-A16*A23*A41*
     1 A55*A64+A13*A26*A41*A55*   
     1 A64+A16*A21*A43*A55*A64-A11*A26*A43*A55*A64-
     1 A13*A21*A46*A55*A64+A11*A23*     
     1 A46*A55*A64+A15*A23*A41*A56*A64-A13*A25*A41*
     1 A56*A64-A15*A21*A43*A56*   
     1 A64+A11*A25*A43*A56*A64+A13*A21*A45*A56*A64-
     1 A11*A23*A45*A56*A64+A16*A24*     
     1 A43*A51*A65-A14*A26*A43*A51*A65-A16*A23*A44*
     1 A51*A65+A13*A26*A44*A51*   
     1 A65+A14*A23*A46*A51*A65-A13*A24*A46*A51*A65-
     1 A16*A24*A41*A53*A65+A14*A26*     
     1 A41*A53*A65+A16*A21*A44*A53*A65-A11*A26*A44*
     1 A53*A65-A14*A21*A46*A53*   
     1 A65+A11*A24*A46*A53*A65+A16*A23*A41*A54*A65-
     1 A13*A26*A41*A54*A65-A16*A21*     
     1 A43*A54*A65+A11*A26*A43*A54*A65+A13*A21*A46*
     1 A54*A65-A11*A23*A46*A54*   
     1 A65-A14*A23*A41*A56*A65+A13*A24*A41*A56*A65+
     1 A14*A21*A43*A56*A65-A11*A24*     
     1 A43*A56*A65-A13*A21*A44*A56*A65+A11*A23*A44*
     1 A56*A65-A15*A24*A43*A51*   
     1 A66+A14*A25*A43*A51*A66+A15*A23*A44*A51*A66-
     1 A13*A25*A44*A51*A66-A14*A23*     
     1 A45*A51*A66+A13*A24*A45*A51*A66+A15*A24*A41*
     1 A53*A66-A14*A25*A41*A53*   
     1 A66-A15*A21*A44*A53*A66+A11*A25*A44*A53*A66+
     1 A14*A21*A45*A53*A66-A11*A24*     
     1 A45*A53*A66-A15*A23*A41*A54*A66+A13*A25*A41*
     1 A54*A66+A15*A21*A43*A54*   
     1 A66-A11*A25*A43*A54*A66-A13*A21*A45*A54*A66+
     1 A11*A23*A45*A54*A66+A14*A23*     
     1 A41*A55*A66-A13*A24*A41*A55*A66-A14*A21*A43*
     1 A55*A66+A11*A24*A43*A55*   
     1 A66+A13*A21*A44*A55*A66-A11*A23*A44*A55*A66

      COFACTOR(4,2) = A16*A25*A34*A53*A61-A15*A26*   
     1 A34*A53*A61-A16*A24*A35*A53*A61+A14*A26*A35*
     1 A53*A61+A15*A24*A36*A53*   
     1 A61-A14*A25*A36*A53*A61-A16*A25*A33*A54*A61+
     1 A15*A26*A33*A54*A61+A16*A23*     
     1 A35*A54*A61-A13*A26*A35*A54*A61-A15*A23*A36*
     1 A54*A61+A13*A25*A36*A54*   
     1 A61+A16*A24*A33*A55*A61-A14*A26*A33*A55*A61-
     1 A16*A23*A34*A55*A61+A13*A26*     
     1 A34*A55*A61+A14*A23*A36*A55*A61-A13*A24*A36*
     1 A55*A61-A15*A24*A33*A56*   
     1 A61+A14*A25*A33*A56*A61+A15*A23*A34*A56*A61-
     1 A13*A25*A34*A56*A61-A14*A23*     
     1 A35*A56*A61+A13*A24*A35*A56*A61-A16*A25*A34*
     1 A51*A63+A15*A26*A34*A51*   
     1 A63+A16*A24*A35*A51*A63-A14*A26*A35*A51*A63-
     1 A15*A24*A36*A51*A63+A14*A25*     
     1 A36*A51*A63+A16*A25*A31*A54*A63-A15*A26*A31*
     1 A54*A63-A16*A21*A35*A54*   
     1 A63+A11*A26*A35*A54*A63+A15*A21*A36*A54*A63-
     1 A11*A25*A36*A54*A63-A16*A24*     
     1 A31*A55*A63+A14*A26*A31*A55*A63+A16*A21*A34*
     1 A55*A63-A11*A26*A34*A55*   
     1 A63-A14*A21*A36*A55*A63+A11*A24*A36*A55*A63+
     1 A15*A24*A31*A56*A63-A14*A25*     
     1 A31*A56*A63-A15*A21*A34*A56*A63+A11*A25*A34*
     1 A56*A63+A14*A21*A35*A56*   
     1 A63-A11*A24*A35*A56*A63+A16*A25*A33*A51*A64-
     1 A15*A26*A33*A51*A64-A16*A23*     
     1 A35*A51*A64+A13*A26*A35*A51*A64+A15*A23*A36*
     1 A51*A64-A13*A25*A36*A51*   
     1 A64-A16*A25*A31*A53*A64+A15*A26*A31*A53*A64+
     1 A16*A21*A35*A53*A64-A11*A26*     
     1 A35*A53*A64-A15*A21*A36*A53*A64+A11*A25*A36*
     1 A53*A64+A16*A23*A31*A55*   
     1 A64-A13*A26*A31*A55*A64-A16*A21*A33*A55*A64+
     1 A11*A26*A33*A55*A64+A13*A21*     
     1 A36*A55*A64-A11*A23*A36*A55*A64-A15*A23*A31*
     1 A56*A64+A13*A25*A31*A56*   
     1 A64+A15*A21*A33*A56*A64-A11*A25*A33*A56*A64-
     1 A13*A21*A35*A56*A64+A11*A23*     
     1 A35*A56*A64-A16*A24*A33*A51*A65+A14*A26*A33*
     1 A51*A65+A16*A23*A34*A51*   
     1 A65-A13*A26*A34*A51*A65-A14*A23*A36*A51*A65+
     1 A13*A24*A36*A51*A65+A16*A24*     
     1 A31*A53*A65-A14*A26*A31*A53*A65-A16*A21*A34*
     1 A53*A65+A11*A26*A34*A53*   
     1 A65+A14*A21*A36*A53*A65-A11*A24*A36*A53*A65-
     1 A16*A23*A31*A54*A65+A13*A26*     
     1 A31*A54*A65+A16*A21*A33*A54*A65-A11*A26*A33*
     1 A54*A65-A13*A21*A36*A54*   
     1 A65+A11*A23*A36*A54*A65+A14*A23*A31*A56*A65-
     1 A13*A24*A31*A56*A65-A14*A21*     
     1 A33*A56*A65+A11*A24*A33*A56*A65+A13*A21*A34*
     1 A56*A65-A11*A23*A34*A56*   
     1 A65+A15*A24*A33*A51*A66-A14*A25*A33*A51*A66-
     1 A15*A23*A34*A51*A66+A13*A25*     
     1 A34*A51*A66+A14*A23*A35*A51*A66-A13*A24*A35*
     1 A51*A66-A15*A24*A31*A53*   
     1 A66+A14*A25*A31*A53*A66+A15*A21*A34*A53*A66-
     1 A11*A25*A34*A53*A66-A14*A21*     
     1 A35*A53*A66+A11*A24*A35*A53*A66+A15*A23*A31*
     1 A54*A66-A13*A25*A31*A54*   
     1 A66-A15*A21*A33*A54*A66+A11*A25*A33*A54*A66+
     1 A13*A21*A35*A54*A66-A11*A23*     
     1 A35*A54*A66-A14*A23*A31*A55*A66+A13*A24*A31*
     1 A55*A66+A14*A21*A33*A55*   
     1 A66-A11*A24*A33*A55*A66-A13*A21*A34*A55*A66+
     1 A11*A23*A34*A55*A66

      COFACTOR(5,2) = -A16*A25*    
     1 A34*A43*A61+A15*A26*A34*A43*A61+A16*A24*A35*
     1 A43*A61-A14*A26*A35*A43*   
     1 A61-A15*A24*A36*A43*A61+A14*A25*A36*A43*A61+
     1 A16*A25*A33*A44*A61-A15*A26*     
     1 A33*A44*A61-A16*A23*A35*A44*A61+A13*A26*A35*
     1 A44*A61+A15*A23*A36*A44*   
     1 A61-A13*A25*A36*A44*A61-A16*A24*A33*A45*A61+
     1 A14*A26*A33*A45*A61+A16*A23*     
     1 A34*A45*A61-A13*A26*A34*A45*A61-A14*A23*A36*
     1 A45*A61+A13*A24*A36*A45*   
     1 A61+A15*A24*A33*A46*A61-A14*A25*A33*A46*A61-
     1 A15*A23*A34*A46*A61+A13*A25*     
     1 A34*A46*A61+A14*A23*A35*A46*A61-A13*A24*A35*
     1 A46*A61+A16*A25*A34*A41*   
     1 A63-A15*A26*A34*A41*A63-A16*A24*A35*A41*A63+
     1 A14*A26*A35*A41*A63+A15*A24*     
     1 A36*A41*A63-A14*A25*A36*A41*A63-A16*A25*A31*
     1 A44*A63+A15*A26*A31*A44*   
     1 A63+A16*A21*A35*A44*A63-A11*A26*A35*A44*A63-
     1 A15*A21*A36*A44*A63+A11*A25*     
     1 A36*A44*A63+A16*A24*A31*A45*A63-A14*A26*A31*
     1 A45*A63-A16*A21*A34*A45*   
     1 A63+A11*A26*A34*A45*A63+A14*A21*A36*A45*A63-
     1 A11*A24*A36*A45*A63-A15*A24*     
     1 A31*A46*A63+A14*A25*A31*A46*A63+A15*A21*A34*
     1 A46*A63-A11*A25*A34*A46*   
     1 A63-A14*A21*A35*A46*A63+A11*A24*A35*A46*A63-
     1 A16*A25*A33*A41*A64+A15*A26*     
     1 A33*A41*A64+A16*A23*A35*A41*A64-A13*A26*A35*
     1 A41*A64-A15*A23*A36*A41*   
     1 A64+A13*A25*A36*A41*A64+A16*A25*A31*A43*A64-
     1 A15*A26*A31*A43*A64-A16*A21*     
     1 A35*A43*A64+A11*A26*A35*A43*A64+A15*A21*A36*
     1 A43*A64-A11*A25*A36*A43*   
     1 A64-A16*A23*A31*A45*A64+A13*A26*A31*A45*A64+
     1 A16*A21*A33*A45*A64-A11*A26*     
     1 A33*A45*A64-A13*A21*A36*A45*A64+A11*A23*A36*
     1 A45*A64+A15*A23*A31*A46*   
     1 A64-A13*A25*A31*A46*A64-A15*A21*A33*A46*A64+
     1 A11*A25*A33*A46*A64+A13*A21*     
     1 A35*A46*A64-A11*A23*A35*A46*A64+A16*A24*A33*
     1 A41*A65-A14*A26*A33*A41*   
     1 A65-A16*A23*A34*A41*A65+A13*A26*A34*A41*A65+
     1 A14*A23*A36*A41*A65-A13*A24*     
     1 A36*A41*A65-A16*A24*A31*A43*A65+A14*A26*A31*
     1 A43*A65+A16*A21*A34*A43*   
     1 A65-A11*A26*A34*A43*A65-A14*A21*A36*A43*A65+
     1 A11*A24*A36*A43*A65+A16*A23*     
     1 A31*A44*A65-A13*A26*A31*A44*A65-A16*A21*A33*
     1 A44*A65+A11*A26*A33*A44*   
     1 A65+A13*A21*A36*A44*A65-A11*A23*A36*A44*A65-
     1 A14*A23*A31*A46*A65+A13*A24*     
     1 A31*A46*A65+A14*A21*A33*A46*A65-A11*A24*A33*
     1 A46*A65-A13*A21*A34*A46*   
     1 A65+A11*A23*A34*A46*A65-A15*A24*A33*A41*A66+
     1 A14*A25*A33*A41*A66+A15*A23*     
     1 A34*A41*A66-A13*A25*A34*A41*A66-A14*A23*A35*
     1 A41*A66+A13*A24*A35*A41*   
     1 A66+A15*A24*A31*A43*A66-A14*A25*A31*A43*A66-
     1 A15*A21*A34*A43*A66+A11*A25*     
     1 A34*A43*A66+A14*A21*A35*A43*A66-A11*A24*A35*
     1 A43*A66-A15*A23*A31*A44*   
     1 A66+A13*A25*A31*A44*A66+A15*A21*A33*A44*A66-
     1 A11*A25*A33*A44*A66-A13*A21*     
     1 A35*A44*A66+A11*A23*A35*A44*A66+A14*A23*A31*
     1 A45*A66-A13*A24*A31*A45*   
     1 A66-A14*A21*A33*A45*A66+A11*A24*A33*A45*A66+
     1 A13*A21*A34*A45*A66-A11*A23*     
     1 A34*A45*A66

      COFACTOR(6,2) = A16*A25*A34*A43*A51-A15*A26*
     1 A34*A43*A51-A16*A24*A35*A43*     
     1 A51+A14*A26*A35*A43*A51+A15*A24*A36*A43*A51-
     1 A14*A25*A36*A43*A51-A16*A25*     
     1 A33*A44*A51+A15*A26*A33*A44*A51+A16*A23*A35*
     1 A44*A51-A13*A26*A35*A44*   
     1 A51-A15*A23*A36*A44*A51+A13*A25*A36*A44*A51+
     1 A16*A24*A33*A45*A51-A14*A26*     
     1 A33*A45*A51-A16*A23*A34*A45*A51+A13*A26*A34*
     1 A45*A51+A14*A23*A36*A45*   
     1 A51-A13*A24*A36*A45*A51-A15*A24*A33*A46*A51+
     1 A14*A25*A33*A46*A51+A15*A23*     
     1 A34*A46*A51-A13*A25*A34*A46*A51-A14*A23*A35*
     1 A46*A51+A13*A24*A35*A46*   
     1 A51-A16*A25*A34*A41*A53+A15*A26*A34*A41*A53+
     1 A16*A24*A35*A41*A53-A14*A26*     
     1 A35*A41*A53-A15*A24*A36*A41*A53+A14*A25*A36*
     1 A41*A53+A16*A25*A31*A44*   
     1 A53-A15*A26*A31*A44*A53-A16*A21*A35*A44*A53+
     1 A11*A26*A35*A44*A53+A15*A21*     
     1 A36*A44*A53-A11*A25*A36*A44*A53-A16*A24*A31*
     1 A45*A53+A14*A26*A31*A45*   
     1 A53+A16*A21*A34*A45*A53-A11*A26*A34*A45*A53-
     1 A14*A21*A36*A45*A53+A11*A24*     
     1 A36*A45*A53+A15*A24*A31*A46*A53-A14*A25*A31*
     1 A46*A53-A15*A21*A34*A46*   
     1 A53+A11*A25*A34*A46*A53+A14*A21*A35*A46*A53-
     1 A11*A24*A35*A46*A53+A16*A25*     
     1 A33*A41*A54-A15*A26*A33*A41*A54-A16*A23*A35*
     1 A41*A54+A13*A26*A35*A41*   
     1 A54+A15*A23*A36*A41*A54-A13*A25*A36*A41*A54-
     1 A16*A25*A31*A43*A54+A15*A26*     
     1 A31*A43*A54+A16*A21*A35*A43*A54-A11*A26*A35*
     1 A43*A54-A15*A21*A36*A43*   
     1 A54+A11*A25*A36*A43*A54+A16*A23*A31*A45*A54-
     1 A13*A26*A31*A45*A54-A16*A21*     
     1 A33*A45*A54+A11*A26*A33*A45*A54+A13*A21*A36*
     1 A45*A54-A11*A23*A36*A45*   
     1 A54-A15*A23*A31*A46*A54+A13*A25*A31*A46*A54+
     1 A15*A21*A33*A46*A54-A11*A25*     
     1 A33*A46*A54-A13*A21*A35*A46*A54+A11*A23*A35*
     1 A46*A54-A16*A24*A33*A41*   
     1 A55+A14*A26*A33*A41*A55+A16*A23*A34*A41*A55-
     1 A13*A26*A34*A41*A55-A14*A23*     
     1 A36*A41*A55+A13*A24*A36*A41*A55+A16*A24*A31*
     1 A43*A55-A14*A26*A31*A43*   
     1 A55-A16*A21*A34*A43*A55+A11*A26*A34*A43*A55+
     1 A14*A21*A36*A43*A55-A11*A24*     
     1 A36*A43*A55-A16*A23*A31*A44*A55+A13*A26*A31*
     1 A44*A55+A16*A21*A33*A44*   
     1 A55-A11*A26*A33*A44*A55-A13*A21*A36*A44*A55+
     1 A11*A23*A36*A44*A55+A14*A23*     
     1 A31*A46*A55-A13*A24*A31*A46*A55-A14*A21*A33*
     1 A46*A55+A11*A24*A33*A46*   
     1 A55+A13*A21*A34*A46*A55-A11*A23*A34*A46*A55+
     1 A15*A24*A33*A41*A56-A14*A25*     
     1 A33*A41*A56-A15*A23*A34*A41*A56+A13*A25*A34*
     1 A41*A56+A14*A23*A35*A41*   
     1 A56-A13*A24*A35*A41*A56-A15*A24*A31*A43*A56+
     1 A14*A25*A31*A43*A56+A15*A21*     
     1 A34*A43*A56-A11*A25*A34*A43*A56-A14*A21*A35*
     1 A43*A56+A11*A24*A35*A43*   
     1 A56+A15*A23*A31*A44*A56-A13*A25*A31*A44*A56-
     1 A15*A21*A33*A44*A56+A11*A25*     
     1 A33*A44*A56+A13*A21*A35*A44*A56-A11*A23*A35*
     1 A44*A56-A14*A23*A31*A45*   
     1 A56+A13*A24*A31*A45*A56+A14*A21*A33*A45*A56-
     1 A11*A24*A33*A45*A56-A13*A21*     
     1 A34*A45*A56+A11*A23*A34*A45*A56

      COFACTOR(1,3) = A26*A35*A44*A52*A61-A25*A36*A44*A52* 
     1 A61-A26*A34*A45*A52*A61+A24*A36*A45*A52*A61+
     1 A25*A34*A46*A52*A61-A24*A35*     
     1 A46*A52*A61-A26*A35*A42*A54*A61+A25*A36*A42*
     1 A54*A61+A26*A32*A45*A54*   
     1 A61-A22*A36*A45*A54*A61-A25*A32*A46*A54*A61+
     1 A22*A35*A46*A54*A61+A26*A34*     
     1 A42*A55*A61-A24*A36*A42*A55*A61-A26*A32*A44*
     1 A55*A61+A22*A36*A44*A55*   
     1 A61+A24*A32*A46*A55*A61-A22*A34*A46*A55*A61-
     1 A25*A34*A42*A56*A61+A24*A35*     
     1 A42*A56*A61+A25*A32*A44*A56*A61-A22*A35*A44*
     1 A56*A61-A24*A32*A45*A56*   
     1 A61+A22*A34*A45*A56*A61-A26*A35*A44*A51*A62+
     1 A25*A36*A44*A51*A62+A26*A34*     
     1 A45*A51*A62-A24*A36*A45*A51*A62-A25*A34*A46*
     1 A51*A62+A24*A35*A46*A51*   
     1 A62+A26*A35*A41*A54*A62-A25*A36*A41*A54*A62-
     1 A26*A31*A45*A54*A62+A21*A36*     
     1 A45*A54*A62+A25*A31*A46*A54*A62-A21*A35*A46*
     1 A54*A62-A26*A34*A41*A55*   
     1 A62+A24*A36*A41*A55*A62+A26*A31*A44*A55*A62-
     1 A21*A36*A44*A55*A62-A24*A31*     
     1 A46*A55*A62+A21*A34*A46*A55*A62+A25*A34*A41*
     1 A56*A62-A24*A35*A41*A56*   
     1 A62-A25*A31*A44*A56*A62+A21*A35*A44*A56*A62+
     1 A24*A31*A45*A56*A62-A21*A34*     
     1 A45*A56*A62+A26*A35*A42*A51*A64-A25*A36*A42*
     1 A51*A64-A26*A32*A45*A51*   
     1 A64+A22*A36*A45*A51*A64+A25*A32*A46*A51*A64-
     1 A22*A35*A46*A51*A64-A26*A35*     
     1 A41*A52*A64+A25*A36*A41*A52*A64+A26*A31*A45*
     1 A52*A64-A21*A36*A45*A52*   
     1 A64-A25*A31*A46*A52*A64+A21*A35*A46*A52*A64+
     1 A26*A32*A41*A55*A64-A22*A36*     
     1 A41*A55*A64-A26*A31*A42*A55*A64+A21*A36*A42*
     1 A55*A64+A22*A31*A46*A55*   
     1 A64-A21*A32*A46*A55*A64-A25*A32*A41*A56*A64+
     1 A22*A35*A41*A56*A64+A25*A31*     
     1 A42*A56*A64-A21*A35*A42*A56*A64-A22*A31*A45*
     1 A56*A64+A21*A32*A45*A56*   
     1 A64-A26*A34*A42*A51*A65+A24*A36*A42*A51*A65+
     1 A26*A32*A44*A51*A65-A22*A36*     
     1 A44*A51*A65-A24*A32*A46*A51*A65+A22*A34*A46*
     1 A51*A65+A26*A34*A41*A52*   
     1 A65-A24*A36*A41*A52*A65-A26*A31*A44*A52*A65+
     1 A21*A36*A44*A52*A65+A24*A31*     
     1 A46*A52*A65-A21*A34*A46*A52*A65-A26*A32*A41*
     1 A54*A65+A22*A36*A41*A54*   
     1 A65+A26*A31*A42*A54*A65-A21*A36*A42*A54*A65-
     1 A22*A31*A46*A54*A65+A21*A32*     
     1 A46*A54*A65+A24*A32*A41*A56*A65-A22*A34*A41*
     1 A56*A65-A24*A31*A42*A56*   
     1 A65+A21*A34*A42*A56*A65+A22*A31*A44*A56*A65-
     1 A21*A32*A44*A56*A65+A25*A34*     
     1 A42*A51*A66-A24*A35*A42*A51*A66-A25*A32*A44*
     1 A51*A66+A22*A35*A44*A51*   
     1 A66+A24*A32*A45*A51*A66-A22*A34*A45*A51*A66-
     1 A25*A34*A41*A52*A66+A24*A35*     
     1 A41*A52*A66+A25*A31*A44*A52*A66-A21*A35*A44*
     1 A52*A66-A24*A31*A45*A52*   
     1 A66+A21*A34*A45*A52*A66+A25*A32*A41*A54*A66-
     1 A22*A35*A41*A54*A66-A25*A31*     
     1 A42*A54*A66+A21*A35*A42*A54*A66+A22*A31*A45*
     1 A54*A66-A21*A32*A45*A54*   
     1 A66-A24*A32*A41*A55*A66+A22*A34*A41*A55*A66+
     1 A24*A31*A42*A55*A66-A21*A34*     
     1 A42*A55*A66-A22*A31*A44*A55*A66+A21*A32*A44*A55*A66

      COFACTOR(2,3) = -A16*A35*A44*A52*  
     1 A61+A15*A36*A44*A52*A61+A16*A34*A45*A52*A61-
     1 A14*A36*A45*A52*A61-A15*A34*     
     1 A46*A52*A61+A14*A35*A46*A52*A61+A16*A35*A42*
     1 A54*A61-A15*A36*A42*A54*   
     1 A61-A16*A32*A45*A54*A61+A12*A36*A45*A54*A61+
     1 A15*A32*A46*A54*A61-A12*A35*     
     1 A46*A54*A61-A16*A34*A42*A55*A61+A14*A36*A42*
     1 A55*A61+A16*A32*A44*A55*   
     1 A61-A12*A36*A44*A55*A61-A14*A32*A46*A55*A61+
     1 A12*A34*A46*A55*A61+A15*A34*     
     1 A42*A56*A61-A14*A35*A42*A56*A61-A15*A32*A44*
     1 A56*A61+A12*A35*A44*A56*   
     1 A61+A14*A32*A45*A56*A61-A12*A34*A45*A56*A61+
     1 A16*A35*A44*A51*A62-A15*A36*     
     1 A44*A51*A62-A16*A34*A45*A51*A62+A14*A36*A45*
     1 A51*A62+A15*A34*A46*A51*   
     1 A62-A14*A35*A46*A51*A62-A16*A35*A41*A54*A62+
     1 A15*A36*A41*A54*A62+A16*A31*     
     1 A45*A54*A62-A11*A36*A45*A54*A62-A15*A31*A46*
     1 A54*A62+A11*A35*A46*A54*   
     1 A62+A16*A34*A41*A55*A62-A14*A36*A41*A55*A62-
     1 A16*A31*A44*A55*A62+A11*A36*     
     1 A44*A55*A62+A14*A31*A46*A55*A62-A11*A34*A46*
     1 A55*A62-A15*A34*A41*A56*   
     1 A62+A14*A35*A41*A56*A62+A15*A31*A44*A56*A62-
     1 A11*A35*A44*A56*A62-A14*A31*     
     1 A45*A56*A62+A11*A34*A45*A56*A62-A16*A35*A42*
     1 A51*A64+A15*A36*A42*A51*   
     1 A64+A16*A32*A45*A51*A64-A12*A36*A45*A51*A64-
     1 A15*A32*A46*A51*A64+A12*A35*     
     1 A46*A51*A64+A16*A35*A41*A52*A64-A15*A36*A41*
     1 A52*A64-A16*A31*A45*A52*   
     1 A64+A11*A36*A45*A52*A64+A15*A31*A46*A52*A64-
     1 A11*A35*A46*A52*A64-A16*A32*     
     1 A41*A55*A64+A12*A36*A41*A55*A64+A16*A31*A42*
     1 A55*A64-A11*A36*A42*A55*   
     1 A64-A12*A31*A46*A55*A64+A11*A32*A46*A55*A64+
     1 A15*A32*A41*A56*A64-A12*A35*     
     1 A41*A56*A64-A15*A31*A42*A56*A64+A11*A35*A42*
     1 A56*A64+A12*A31*A45*A56*   
     1 A64-A11*A32*A45*A56*A64+A16*A34*A42*A51*A65-
     1 A14*A36*A42*A51*A65-A16*A32*     
     1 A44*A51*A65+A12*A36*A44*A51*A65+A14*A32*A46*
     1 A51*A65-A12*A34*A46*A51*   
     1 A65-A16*A34*A41*A52*A65+A14*A36*A41*A52*A65+
     1 A16*A31*A44*A52*A65-A11*A36*     
     1 A44*A52*A65-A14*A31*A46*A52*A65+A11*A34*A46*
     1 A52*A65+A16*A32*A41*A54*   
     1 A65-A12*A36*A41*A54*A65-A16*A31*A42*A54*A65+
     1 A11*A36*A42*A54*A65+A12*A31*     
     1 A46*A54*A65-A11*A32*A46*A54*A65-A14*A32*A41*
     1 A56*A65+A12*A34*A41*A56*   
     1 A65+A14*A31*A42*A56*A65-A11*A34*A42*A56*A65-
     1 A12*A31*A44*A56*A65+A11*A32*     
     1 A44*A56*A65-A15*A34*A42*A51*A66+A14*A35*A42*
     1 A51*A66+A15*A32*A44*A51*   
     1 A66-A12*A35*A44*A51*A66-A14*A32*A45*A51*A66+
     1 A12*A34*A45*A51*A66+A15*A34*     
     1 A41*A52*A66-A14*A35*A41*A52*A66-A15*A31*A44*
     1 A52*A66+A11*A35*A44*A52*   
     1 A66+A14*A31*A45*A52*A66-A11*A34*A45*A52*A66-
     1 A15*A32*A41*A54*A66+A12*A35*     
     1 A41*A54*A66+A15*A31*A42*A54*A66-A11*A35*A42*
     1 A54*A66-A12*A31*A45*A54*   
     1 A66+A11*A32*A45*A54*A66+A14*A32*A41*A55*A66-
     1 A12*A34*A41*A55*A66-A14*A31*     
     1 A42*A55*A66+A11*A34*A42*A55*A66+A12*A31*A44*
     1 A55*A66-A11*A32*A44*A55*   
     1 A66

      COFACTOR(3,3) = A16*A25*A44*A52*A61-A15*A26*
     1 A44*A52*A61-A16*A24*A45*A52*A61+A14*A26*     
     1 A45*A52*A61+A15*A24*A46*A52*A61-A14*A25*A46*
     1 A52*A61-A16*A25*A42*A54*   
     1 A61+A15*A26*A42*A54*A61+A16*A22*A45*A54*A61-
     1 A12*A26*A45*A54*A61-A15*A22*     
     1 A46*A54*A61+A12*A25*A46*A54*A61+A16*A24*A42*
     1 A55*A61-A14*A26*A42*A55*   
     1 A61-A16*A22*A44*A55*A61+A12*A26*A44*A55*A61+
     1 A14*A22*A46*A55*A61-A12*A24*     
     1 A46*A55*A61-A15*A24*A42*A56*A61+A14*A25*A42*
     1 A56*A61+A15*A22*A44*A56*   
     1 A61-A12*A25*A44*A56*A61-A14*A22*A45*A56*A61+
     1 A12*A24*A45*A56*A61-A16*A25*     
     1 A44*A51*A62+A15*A26*A44*A51*A62+A16*A24*A45*
     1 A51*A62-A14*A26*A45*A51*   
     1 A62-A15*A24*A46*A51*A62+A14*A25*A46*A51*A62+
     1 A16*A25*A41*A54*A62-A15*A26*     
     1 A41*A54*A62-A16*A21*A45*A54*A62+A11*A26*A45*
     1 A54*A62+A15*A21*A46*A54*   
     1 A62-A11*A25*A46*A54*A62-A16*A24*A41*A55*A62+
     1 A14*A26*A41*A55*A62+A16*A21*     
     1 A44*A55*A62-A11*A26*A44*A55*A62-A14*A21*A46*
     1 A55*A62+A11*A24*A46*A55*   
     1 A62+A15*A24*A41*A56*A62-A14*A25*A41*A56*A62-
     1 A15*A21*A44*A56*A62+A11*A25*     
     1 A44*A56*A62+A14*A21*A45*A56*A62-A11*A24*A45*
     1 A56*A62+A16*A25*A42*A51*   
     1 A64-A15*A26*A42*A51*A64-A16*A22*A45*A51*A64+
     1 A12*A26*A45*A51*A64+A15*A22*     
     1 A46*A51*A64-A12*A25*A46*A51*A64-A16*A25*A41*
     1 A52*A64+A15*A26*A41*A52*   
     1 A64+A16*A21*A45*A52*A64-A11*A26*A45*A52*A64-
     1 A15*A21*A46*A52*A64+A11*A25*     
     1 A46*A52*A64+A16*A22*A41*A55*A64-A12*A26*A41*
     1 A55*A64-A16*A21*A42*A55*   
     1 A64+A11*A26*A42*A55*A64+A12*A21*A46*A55*A64-
     1 A11*A22*A46*A55*A64-A15*A22*     
     1 A41*A56*A64+A12*A25*A41*A56*A64+A15*A21*A42*
     1 A56*A64-A11*A25*A42*A56*   
     1 A64-A12*A21*A45*A56*A64+A11*A22*A45*A56*A64-
     1 A16*A24*A42*A51*A65+A14*A26*     
     1 A42*A51*A65+A16*A22*A44*A51*A65-A12*A26*A44*
     1 A51*A65-A14*A22*A46*A51*   
     1 A65+A12*A24*A46*A51*A65+A16*A24*A41*A52*A65-
     1 A14*A26*A41*A52*A65-A16*A21*     
     1 A44*A52*A65+A11*A26*A44*A52*A65+A14*A21*A46*
     1 A52*A65-A11*A24*A46*A52*   
     1 A65-A16*A22*A41*A54*A65+A12*A26*A41*A54*A65+
     1 A16*A21*A42*A54*A65-A11*A26*     
     1 A42*A54*A65-A12*A21*A46*A54*A65+A11*A22*A46*
     1 A54*A65+A14*A22*A41*A56*   
     1 A65-A12*A24*A41*A56*A65-A14*A21*A42*A56*A65+
     1 A11*A24*A42*A56*A65+A12*A21*     
     1 A44*A56*A65-A11*A22*A44*A56*A65+A15*A24*A42*
     1 A51*A66-A14*A25*A42*A51*   
     1 A66-A15*A22*A44*A51*A66+A12*A25*A44*A51*A66+
     1 A14*A22*A45*A51*A66-A12*A24*     
     1 A45*A51*A66-A15*A24*A41*A52*A66+A14*A25*A41*
     1 A52*A66+A15*A21*A44*A52*   
     1 A66-A11*A25*A44*A52*A66-A14*A21*A45*A52*A66+
     1 A11*A24*A45*A52*A66+A15*A22*     
     1 A41*A54*A66-A12*A25*A41*A54*A66-A15*A21*A42*
     1 A54*A66+A11*A25*A42*A54*   
     1 A66+A12*A21*A45*A54*A66-A11*A22*A45*A54*A66-
     1 A14*A22*A41*A55*A66+A12*A24*     
     1 A41*A55*A66+A14*A21*A42*A55*A66-A11*A24*A42*
     1 A55*A66-A12*A21*A44*A55*   
     1 A66+A11*A22*A44*A55*A66

      COFACTOR(4,3) = -A16*A25*A34*A52*A61+A15*A26*
     1 A34*A52*A61+A16*A24*
     1 A35*A52*A61-A14*A26*A35*A52*A61-A15*A24*A36*
     1 A52*A61+A14*A25*A36*A52*   
     1 A61+A16*A25*A32*A54*A61-A15*A26*A32*A54*A61-
     1 A16*A22*A35*A54*A61+A12*A26*     
     1 A35*A54*A61+A15*A22*A36*A54*A61-A12*A25*A36*
     1 A54*A61-A16*A24*A32*A55*   
     1 A61+A14*A26*A32*A55*A61+A16*A22*A34*A55*A61-
     1 A12*A26*A34*A55*A61-A14*A22*     
     1 A36*A55*A61+A12*A24*A36*A55*A61+A15*A24*A32*
     1 A56*A61-A14*A25*A32*A56*   
     1 A61-A15*A22*A34*A56*A61+A12*A25*A34*A56*A61+
     1 A14*A22*A35*A56*A61-A12*A24*     
     1 A35*A56*A61+A16*A25*A34*A51*A62-A15*A26*A34*
     1 A51*A62-A16*A24*A35*A51*   
     1 A62+A14*A26*A35*A51*A62+A15*A24*A36*A51*A62-
     1 A14*A25*A36*A51*A62-A16*A25*     
     1 A31*A54*A62+A15*A26*A31*A54*A62+A16*A21*A35*
     1 A54*A62-A11*A26*A35*A54*   
     1 A62-A15*A21*A36*A54*A62+A11*A25*A36*A54*A62+
     1 A16*A24*A31*A55*A62-A14*A26*     
     1 A31*A55*A62-A16*A21*A34*A55*A62+A11*A26*A34*
     1 A55*A62+A14*A21*A36*A55*   
     1 A62-A11*A24*A36*A55*A62-A15*A24*A31*A56*A62+
     1 A14*A25*A31*A56*A62+A15*A21*     
     1 A34*A56*A62-A11*A25*A34*A56*A62-A14*A21*A35*
     1 A56*A62+A11*A24*A35*A56*   
     1 A62-A16*A25*A32*A51*A64+A15*A26*A32*A51*A64+
     1 A16*A22*A35*A51*A64-A12*A26*     
     1 A35*A51*A64-A15*A22*A36*A51*A64+A12*A25*A36*
     1 A51*A64+A16*A25*A31*A52*   
     1 A64-A15*A26*A31*A52*A64-A16*A21*A35*A52*A64+
     1 A11*A26*A35*A52*A64+A15*A21*     
     1 A36*A52*A64-A11*A25*A36*A52*A64-A16*A22*A31*
     1 A55*A64+A12*A26*A31*A55*   
     1 A64+A16*A21*A32*A55*A64-A11*A26*A32*A55*A64-
     1 A12*A21*A36*A55*A64+A11*A22*     
     1 A36*A55*A64+A15*A22*A31*A56*A64-A12*A25*A31*
     1 A56*A64-A15*A21*A32*A56*   
     1 A64+A11*A25*A32*A56*A64+A12*A21*A35*A56*A64-
     1 A11*A22*A35*A56*A64+A16*A24*     
     1 A32*A51*A65-A14*A26*A32*A51*A65-A16*A22*A34*
     1 A51*A65+A12*A26*A34*A51*   
     1 A65+A14*A22*A36*A51*A65-A12*A24*A36*A51*A65-
     1 A16*A24*A31*A52*A65+A14*A26*     
     1 A31*A52*A65+A16*A21*A34*A52*A65-A11*A26*A34*
     1 A52*A65-A14*A21*A36*A52*   
     1 A65+A11*A24*A36*A52*A65+A16*A22*A31*A54*A65-
     1 A12*A26*A31*A54*A65-A16*A21*     
     1 A32*A54*A65+A11*A26*A32*A54*A65+A12*A21*A36*
     1 A54*A65-A11*A22*A36*A54*   
     1 A65-A14*A22*A31*A56*A65+A12*A24*A31*A56*A65+
     1 A14*A21*A32*A56*A65-A11*A24*     
     1 A32*A56*A65-A12*A21*A34*A56*A65+A11*A22*A34*
     1 A56*A65-A15*A24*A32*A51*   
     1 A66+A14*A25*A32*A51*A66+A15*A22*A34*A51*A66-
     1 A12*A25*A34*A51*A66-A14*A22*     
     1 A35*A51*A66+A12*A24*A35*A51*A66+A15*A24*A31*
     1 A52*A66-A14*A25*A31*A52*   
     1 A66-A15*A21*A34*A52*A66+A11*A25*A34*A52*A66+
     1 A14*A21*A35*A52*A66-A11*A24*     
     1 A35*A52*A66-A15*A22*A31*A54*A66+A12*A25*A31*
     1 A54*A66+A15*A21*A32*A54*   
     1 A66-A11*A25*A32*A54*A66-A12*A21*A35*A54*A66+
     1 A11*A22*A35*A54*A66+A14*A22*     
     1 A31*A55*A66-A12*A24*A31*A55*A66-A14*A21*A32*
     1 A55*A66+A11*A24*A32*A55*   
     1 A66+A12*A21*A34*A55*A66-A11*A22*A34*A55*A66

      COFACTOR(5,3) = A16*A25*A34*A42*A61-A15*A26*   
     1 A34*A42*A61-A16*A24*A35*A42*A61+A14*A26*A35*
     1 A42*A61+A15*A24*A36*A42*   
     1 A61-A14*A25*A36*A42*A61-A16*A25*A32*A44*A61+
     1 A15*A26*A32*A44*A61+A16*A22*     
     1 A35*A44*A61-A12*A26*A35*A44*A61-A15*A22*A36*
     1 A44*A61+A12*A25*A36*A44*   
     1 A61+A16*A24*A32*A45*A61-A14*A26*A32*A45*A61-
     1 A16*A22*A34*A45*A61+A12*A26*     
     1 A34*A45*A61+A14*A22*A36*A45*A61-A12*A24*A36*
     1 A45*A61-A15*A24*A32*A46*   
     1 A61+A14*A25*A32*A46*A61+A15*A22*A34*A46*A61-
     1 A12*A25*A34*A46*A61-A14*A22*     
     1 A35*A46*A61+A12*A24*A35*A46*A61-A16*A25*A34*
     1 A41*A62+A15*A26*A34*A41*   
     1 A62+A16*A24*A35*A41*A62-A14*A26*A35*A41*A62-
     1 A15*A24*A36*A41*A62+A14*A25*     
     1 A36*A41*A62+A16*A25*A31*A44*A62-A15*A26*A31*
     1 A44*A62-A16*A21*A35*A44*   
     1 A62+A11*A26*A35*A44*A62+A15*A21*A36*A44*A62-
     1 A11*A25*A36*A44*A62-A16*A24*     
     1 A31*A45*A62+A14*A26*A31*A45*A62+A16*A21*A34*
     1 A45*A62-A11*A26*A34*A45*   
     1 A62-A14*A21*A36*A45*A62+A11*A24*A36*A45*A62+
     1 A15*A24*A31*A46*A62-A14*A25*     
     1 A31*A46*A62-A15*A21*A34*A46*A62+A11*A25*A34*
     1 A46*A62+A14*A21*A35*A46*   
     1 A62-A11*A24*A35*A46*A62+A16*A25*A32*A41*A64-
     1 A15*A26*A32*A41*A64-A16*A22*     
     1 A35*A41*A64+A12*A26*A35*A41*A64+A15*A22*A36*
     1 A41*A64-A12*A25*A36*A41*   
     1 A64-A16*A25*A31*A42*A64+A15*A26*A31*A42*A64+
     1 A16*A21*A35*A42*A64-A11*A26*     
     1 A35*A42*A64-A15*A21*A36*A42*A64+A11*A25*A36*
     1 A42*A64+A16*A22*A31*A45*   
     1 A64-A12*A26*A31*A45*A64-A16*A21*A32*A45*A64+
     1 A11*A26*A32*A45*A64+A12*A21*     
     1 A36*A45*A64-A11*A22*A36*A45*A64-A15*A22*A31*
     1 A46*A64+A12*A25*A31*A46*   
     1 A64+A15*A21*A32*A46*A64-A11*A25*A32*A46*A64-
     1 A12*A21*A35*A46*A64+A11*A22*     
     1 A35*A46*A64-A16*A24*A32*A41*A65+A14*A26*A32*
     1 A41*A65+A16*A22*A34*A41*   
     1 A65-A12*A26*A34*A41*A65-A14*A22*A36*A41*A65+
     1 A12*A24*A36*A41*A65+A16*A24*     
     1 A31*A42*A65-A14*A26*A31*A42*A65-A16*A21*A34*
     1 A42*A65+A11*A26*A34*A42*   
     1 A65+A14*A21*A36*A42*A65-A11*A24*A36*A42*A65-
     1 A16*A22*A31*A44*A65+A12*A26*     
     1 A31*A44*A65+A16*A21*A32*A44*A65-A11*A26*A32*
     1 A44*A65-A12*A21*A36*A44*   
     1 A65+A11*A22*A36*A44*A65+A14*A22*A31*A46*A65-
     1 A12*A24*A31*A46*A65-A14*A21*     
     1 A32*A46*A65+A11*A24*A32*A46*A65+A12*A21*A34*
     1 A46*A65-A11*A22*A34*A46*   
     1 A65+A15*A24*A32*A41*A66-A14*A25*A32*A41*A66-
     1 A15*A22*A34*A41*A66+A12*A25*     
     1 A34*A41*A66+A14*A22*A35*A41*A66-A12*A24*A35*
     1 A41*A66-A15*A24*A31*A42*   
     1 A66+A14*A25*A31*A42*A66+A15*A21*A34*A42*A66-
     1 A11*A25*A34*A42*A66-A14*A21*     
     1 A35*A42*A66+A11*A24*A35*A42*A66+A15*A22*A31*
     1 A44*A66-A12*A25*A31*A44*   
     1 A66-A15*A21*A32*A44*A66+A11*A25*A32*A44*A66+
     1 A12*A21*A35*A44*A66-A11*A22*     
     1 A35*A44*A66-A14*A22*A31*A45*A66+A12*A24*A31*
     1 A45*A66+A14*A21*A32*A45*   
     1 A66-A11*A24*A32*A45*A66-A12*A21*A34*A45*A66+
     1 A11*A22*A34*A45*A66

      COFACTOR(6,3) = -A16*A25*    
     1 A34*A42*A51+A15*A26*A34*A42*A51+A16*A24*A35*
     1 A42*A51-A14*A26*A35*A42*   
     1 A51-A15*A24*A36*A42*A51+A14*A25*A36*A42*A51+
     1 A16*A25*A32*A44*A51-A15*A26*     
     1 A32*A44*A51-A16*A22*A35*A44*A51+A12*A26*A35*
     1 A44*A51+A15*A22*A36*A44*   
     1 A51-A12*A25*A36*A44*A51-A16*A24*A32*A45*A51+
     1 A14*A26*A32*A45*A51+A16*A22*     
     1 A34*A45*A51-A12*A26*A34*A45*A51-A14*A22*A36*
     1 A45*A51+A12*A24*A36*A45*   
     1 A51+A15*A24*A32*A46*A51-A14*A25*A32*A46*A51-
     1 A15*A22*A34*A46*A51+A12*A25*     
     1 A34*A46*A51+A14*A22*A35*A46*A51-A12*A24*A35*
     1 A46*A51+A16*A25*A34*A41*   
     1 A52-A15*A26*A34*A41*A52-A16*A24*A35*A41*A52+
     1 A14*A26*A35*A41*A52+A15*A24*     
     1 A36*A41*A52-A14*A25*A36*A41*A52-A16*A25*A31*
     1 A44*A52+A15*A26*A31*A44*   
     1 A52+A16*A21*A35*A44*A52-A11*A26*A35*A44*A52-
     1 A15*A21*A36*A44*A52+A11*A25*     
     1 A36*A44*A52+A16*A24*A31*A45*A52-A14*A26*A31*
     1 A45*A52-A16*A21*A34*A45*   
     1 A52+A11*A26*A34*A45*A52+A14*A21*A36*A45*A52-
     1 A11*A24*A36*A45*A52-A15*A24*     
     1 A31*A46*A52+A14*A25*A31*A46*A52+A15*A21*A34*
     1 A46*A52-A11*A25*A34*A46*   
     1 A52-A14*A21*A35*A46*A52+A11*A24*A35*A46*A52-
     1 A16*A25*A32*A41*A54+A15*A26*     
     1 A32*A41*A54+A16*A22*A35*A41*A54-A12*A26*A35*
     1 A41*A54-A15*A22*A36*A41*   
     1 A54+A12*A25*A36*A41*A54+A16*A25*A31*A42*A54-
     1 A15*A26*A31*A42*A54-A16*A21*     
     1 A35*A42*A54+A11*A26*A35*A42*A54+A15*A21*A36*
     1 A42*A54-A11*A25*A36*A42*   
     1 A54-A16*A22*A31*A45*A54+A12*A26*A31*A45*A54+
     1 A16*A21*A32*A45*A54-A11*A26*     
     1 A32*A45*A54-A12*A21*A36*A45*A54+A11*A22*A36*
     1 A45*A54+A15*A22*A31*A46*   
     1 A54-A12*A25*A31*A46*A54-A15*A21*A32*A46*A54+
     1 A11*A25*A32*A46*A54+A12*A21*     
     1 A35*A46*A54-A11*A22*A35*A46*A54+A16*A24*A32*
     1 A41*A55-A14*A26*A32*A41*   
     1 A55-A16*A22*A34*A41*A55+A12*A26*A34*A41*A55+
     1 A14*A22*A36*A41*A55-A12*A24*     
     1 A36*A41*A55-A16*A24*A31*A42*A55+A14*A26*A31*
     1 A42*A55+A16*A21*A34*A42*   
     1 A55-A11*A26*A34*A42*A55-A14*A21*A36*A42*A55+
     1 A11*A24*A36*A42*A55+A16*A22*     
     1 A31*A44*A55-A12*A26*A31*A44*A55-A16*A21*A32*
     1 A44*A55+A11*A26*A32*A44*   
     1 A55+A12*A21*A36*A44*A55-A11*A22*A36*A44*A55-
     1 A14*A22*A31*A46*A55+A12*A24*     
     1 A31*A46*A55+A14*A21*A32*A46*A55-A11*A24*A32*
     1 A46*A55-A12*A21*A34*A46*   
     1 A55+A11*A22*A34*A46*A55-A15*A24*A32*A41*A56+
     1 A14*A25*A32*A41*A56+A15*A22*     
     1 A34*A41*A56-A12*A25*A34*A41*A56-A14*A22*A35*
     1 A41*A56+A12*A24*A35*A41*   
     1 A56+A15*A24*A31*A42*A56-A14*A25*A31*A42*A56-
     1 A15*A21*A34*A42*A56+A11*A25*     
     1 A34*A42*A56+A14*A21*A35*A42*A56-A11*A24*A35*
     1 A42*A56-A15*A22*A31*A44*   
     1 A56+A12*A25*A31*A44*A56+A15*A21*A32*A44*A56-
     1 A11*A25*A32*A44*A56-A12*A21*     
     1 A35*A44*A56+A11*A22*A35*A44*A56+A14*A22*A31*
     1 A45*A56-A12*A24*A31*A45*   
     1 A56-A14*A21*A32*A45*A56+A11*A24*A32*A45*A56+
     1 A12*A21*A34*A45*A56-A11*A22*     
     1 A34*A45*A56

      COFACTOR(1,4) = -A26*A35*A43*A52*A61+A25*A36*
     1 A43*A52*A61+A26*A33*A45*A52*    
     1 A61-A23*A36*A45*A52*A61-A25*A33*A46*A52*A61+
     1 A23*A35*A46*A52*A61+A26*A35*     
     1 A42*A53*A61-A25*A36*A42*A53*A61-A26*A32*A45*
     1 A53*A61+A22*A36*A45*A53*   
     1 A61+A25*A32*A46*A53*A61-A22*A35*A46*A53*A61-
     1 A26*A33*A42*A55*A61+A23*A36*     
     1 A42*A55*A61+A26*A32*A43*A55*A61-A22*A36*A43*
     1 A55*A61-A23*A32*A46*A55*   
     1 A61+A22*A33*A46*A55*A61+A25*A33*A42*A56*A61-
     1 A23*A35*A42*A56*A61-A25*A32*     
     1 A43*A56*A61+A22*A35*A43*A56*A61+A23*A32*A45*
     1 A56*A61-A22*A33*A45*A56*   
     1 A61+A26*A35*A43*A51*A62-A25*A36*A43*A51*A62-
     1 A26*A33*A45*A51*A62+A23*A36*     
     1 A45*A51*A62+A25*A33*A46*A51*A62-A23*A35*A46*
     1 A51*A62-A26*A35*A41*A53*   
     1 A62+A25*A36*A41*A53*A62+A26*A31*A45*A53*A62-
     1 A21*A36*A45*A53*A62-A25*A31*     
     1 A46*A53*A62+A21*A35*A46*A53*A62+A26*A33*A41*
     1 A55*A62-A23*A36*A41*A55*   
     1 A62-A26*A31*A43*A55*A62+A21*A36*A43*A55*A62+
     1 A23*A31*A46*A55*A62-A21*A33*     
     1 A46*A55*A62-A25*A33*A41*A56*A62+A23*A35*A41*
     1 A56*A62+A25*A31*A43*A56*   
     1 A62-A21*A35*A43*A56*A62-A23*A31*A45*A56*A62+
     1 A21*A33*A45*A56*A62-A26*A35*     
     1 A42*A51*A63+A25*A36*A42*A51*A63+A26*A32*A45*
     1 A51*A63-A22*A36*A45*A51*   
     1 A63-A25*A32*A46*A51*A63+A22*A35*A46*A51*A63+
     1 A26*A35*A41*A52*A63-A25*A36*     
     1 A41*A52*A63-A26*A31*A45*A52*A63+A21*A36*A45*
     1 A52*A63+A25*A31*A46*A52*   
     1 A63-A21*A35*A46*A52*A63-A26*A32*A41*A55*A63+
     1 A22*A36*A41*A55*A63+A26*A31*     
     1 A42*A55*A63-A21*A36*A42*A55*A63-A22*A31*A46*
     1 A55*A63+A21*A32*A46*A55*   
     1 A63+A25*A32*A41*A56*A63-A22*A35*A41*A56*A63-
     1 A25*A31*A42*A56*A63+A21*A35*     
     1 A42*A56*A63+A22*A31*A45*A56*A63-A21*A32*A45*
     1 A56*A63+A26*A33*A42*A51*   
     1 A65-A23*A36*A42*A51*A65-A26*A32*A43*A51*A65+
     1 A22*A36*A43*A51*A65+A23*A32*     
     1 A46*A51*A65-A22*A33*A46*A51*A65-A26*A33*A41*
     1 A52*A65+A23*A36*A41*A52*   
     1 A65+A26*A31*A43*A52*A65-A21*A36*A43*A52*A65-
     1 A23*A31*A46*A52*A65+A21*A33*     
     1 A46*A52*A65+A26*A32*A41*A53*A65-A22*A36*A41*
     1 A53*A65-A26*A31*A42*A53*   
     1 A65+A21*A36*A42*A53*A65+A22*A31*A46*A53*A65-
     1 A21*A32*A46*A53*A65-A23*A32*     
     1 A41*A56*A65+A22*A33*A41*A56*A65+A23*A31*A42*
     1 A56*A65-A21*A33*A42*A56*   
     1 A65-A22*A31*A43*A56*A65+A21*A32*A43*A56*A65-
     1 A25*A33*A42*A51*A66+A23*A35*     
     1 A42*A51*A66+A25*A32*A43*A51*A66-A22*A35*A43*
     1 A51*A66-A23*A32*A45*A51*   
     1 A66+A22*A33*A45*A51*A66+A25*A33*A41*A52*A66-
     1 A23*A35*A41*A52*A66-A25*A31*     
     1 A43*A52*A66+A21*A35*A43*A52*A66+A23*A31*A45*
     1 A52*A66-A21*A33*A45*A52*   
     1 A66-A25*A32*A41*A53*A66+A22*A35*A41*A53*A66+
     1 A25*A31*A42*A53*A66-A21*A35*     
     1 A42*A53*A66-A22*A31*A45*A53*A66+A21*A32*A45*
     1 A53*A66+A23*A32*A41*A55*   
     1 A66-A22*A33*A41*A55*A66-A23*A31*A42*A55*A66+
     1 A21*A33*A42*A55*A66+A22*A31*     
     1 A43*A55*A66-A21*A32*A43*A55*A66

      COFACTOR(2,4) = A16*A35*A43*A52*A61-A15*A36*A43*A52* 
     1 A61-A16*A33*A45*A52*A61+A13*A36*A45*A52*A61+
     1 A15*A33*A46*A52*A61-A13*A35*     
     1 A46*A52*A61-A16*A35*A42*A53*A61+A15*A36*A42*
     1 A53*A61+A16*A32*A45*A53*   
     1 A61-A12*A36*A45*A53*A61-A15*A32*A46*A53*A61+
     1 A12*A35*A46*A53*A61+A16*A33*     
     1 A42*A55*A61-A13*A36*A42*A55*A61-A16*A32*A43*
     1 A55*A61+A12*A36*A43*A55*   
     1 A61+A13*A32*A46*A55*A61-A12*A33*A46*A55*A61-
     1 A15*A33*A42*A56*A61+A13*A35*     
     1 A42*A56*A61+A15*A32*A43*A56*A61-A12*A35*A43*
     1 A56*A61-A13*A32*A45*A56*   
     1 A61+A12*A33*A45*A56*A61-A16*A35*A43*A51*A62+
     1 A15*A36*A43*A51*A62+A16*A33*     
     1 A45*A51*A62-A13*A36*A45*A51*A62-A15*A33*A46*
     1 A51*A62+A13*A35*A46*A51*   
     1 A62+A16*A35*A41*A53*A62-A15*A36*A41*A53*A62-
     1 A16*A31*A45*A53*A62+A11*A36*     
     1 A45*A53*A62+A15*A31*A46*A53*A62-A11*A35*A46*
     1 A53*A62-A16*A33*A41*A55*   
     1 A62+A13*A36*A41*A55*A62+A16*A31*A43*A55*A62-
     1 A11*A36*A43*A55*A62-A13*A31*     
     1 A46*A55*A62+A11*A33*A46*A55*A62+A15*A33*A41*
     1 A56*A62-A13*A35*A41*A56*   
     1 A62-A15*A31*A43*A56*A62+A11*A35*A43*A56*A62+
     1 A13*A31*A45*A56*A62-A11*A33*     
     1 A45*A56*A62+A16*A35*A42*A51*A63-A15*A36*A42*
     1 A51*A63-A16*A32*A45*A51*   
     1 A63+A12*A36*A45*A51*A63+A15*A32*A46*A51*A63-
     1 A12*A35*A46*A51*A63-A16*A35*     
     1 A41*A52*A63+A15*A36*A41*A52*A63+A16*A31*A45*
     1 A52*A63-A11*A36*A45*A52*   
     1 A63-A15*A31*A46*A52*A63+A11*A35*A46*A52*A63+
     1 A16*A32*A41*A55*A63-A12*A36*     
     1 A41*A55*A63-A16*A31*A42*A55*A63+A11*A36*A42*
     1 A55*A63+A12*A31*A46*A55*   
     1 A63-A11*A32*A46*A55*A63-A15*A32*A41*A56*A63+
     1 A12*A35*A41*A56*A63+A15*A31*     
     1 A42*A56*A63-A11*A35*A42*A56*A63-A12*A31*A45*
     1 A56*A63+A11*A32*A45*A56*   
     1 A63-A16*A33*A42*A51*A65+A13*A36*A42*A51*A65+
     1 A16*A32*A43*A51*A65-A12*A36*     
     1 A43*A51*A65-A13*A32*A46*A51*A65+A12*A33*A46*
     1 A51*A65+A16*A33*A41*A52*   
     1 A65-A13*A36*A41*A52*A65-A16*A31*A43*A52*A65+
     1 A11*A36*A43*A52*A65+A13*A31*     
     1 A46*A52*A65-A11*A33*A46*A52*A65-A16*A32*A41*
     1 A53*A65+A12*A36*A41*A53*   
     1 A65+A16*A31*A42*A53*A65-A11*A36*A42*A53*A65-
     1 A12*A31*A46*A53*A65+A11*A32*     
     1 A46*A53*A65+A13*A32*A41*A56*A65-A12*A33*A41*
     1 A56*A65-A13*A31*A42*A56*   
     1 A65+A11*A33*A42*A56*A65+A12*A31*A43*A56*A65-
     1 A11*A32*A43*A56*A65+A15*A33*     
     1 A42*A51*A66-A13*A35*A42*A51*A66-A15*A32*A43*
     1 A51*A66+A12*A35*A43*A51*   
     1 A66+A13*A32*A45*A51*A66-A12*A33*A45*A51*A66-
     1 A15*A33*A41*A52*A66+A13*A35*     
     1 A41*A52*A66+A15*A31*A43*A52*A66-A11*A35*A43*
     1 A52*A66-A13*A31*A45*A52*   
     1 A66+A11*A33*A45*A52*A66+A15*A32*A41*A53*A66-
     1 A12*A35*A41*A53*A66-A15*A31*     
     1 A42*A53*A66+A11*A35*A42*A53*A66+A12*A31*A45*
     1 A53*A66-A11*A32*A45*A53*   
     1 A66-A13*A32*A41*A55*A66+A12*A33*A41*A55*A66+
     1 A13*A31*A42*A55*A66-A11*A33*     
     1 A42*A55*A66-A12*A31*A43*A55*A66+A11*A32*A43*A55*A66

      COFACTOR(3,4) = -A16*A25*A43*A52*  
     1 A61+A15*A26*A43*A52*A61+A16*A23*A45*A52*A61-
     1 A13*A26*A45*A52*A61-A15*A23*     
     1 A46*A52*A61+A13*A25*A46*A52*A61+A16*A25*A42*
     1 A53*A61-A15*A26*A42*A53*   
     1 A61-A16*A22*A45*A53*A61+A12*A26*A45*A53*A61+
     1 A15*A22*A46*A53*A61-A12*A25*     
     1 A46*A53*A61-A16*A23*A42*A55*A61+A13*A26*A42*
     1 A55*A61+A16*A22*A43*A55*   
     1 A61-A12*A26*A43*A55*A61-A13*A22*A46*A55*A61+
     1 A12*A23*A46*A55*A61+A15*A23*     
     1 A42*A56*A61-A13*A25*A42*A56*A61-A15*A22*A43*
     1 A56*A61+A12*A25*A43*A56*   
     1 A61+A13*A22*A45*A56*A61-A12*A23*A45*A56*A61+
     1 A16*A25*A43*A51*A62-A15*A26*     
     1 A43*A51*A62-A16*A23*A45*A51*A62+A13*A26*A45*
     1 A51*A62+A15*A23*A46*A51*   
     1 A62-A13*A25*A46*A51*A62-A16*A25*A41*A53*A62+
     1 A15*A26*A41*A53*A62+A16*A21*     
     1 A45*A53*A62-A11*A26*A45*A53*A62-A15*A21*A46*
     1 A53*A62+A11*A25*A46*A53*   
     1 A62+A16*A23*A41*A55*A62-A13*A26*A41*A55*A62-
     1 A16*A21*A43*A55*A62+A11*A26*     
     1 A43*A55*A62+A13*A21*A46*A55*A62-A11*A23*A46*
     1 A55*A62-A15*A23*A41*A56*   
     1 A62+A13*A25*A41*A56*A62+A15*A21*A43*A56*A62-
     1 A11*A25*A43*A56*A62-A13*A21*     
     1 A45*A56*A62+A11*A23*A45*A56*A62-A16*A25*A42*
     1 A51*A63+A15*A26*A42*A51*   
     1 A63+A16*A22*A45*A51*A63-A12*A26*A45*A51*A63-
     1 A15*A22*A46*A51*A63+A12*A25*     
     1 A46*A51*A63+A16*A25*A41*A52*A63-A15*A26*A41*
     1 A52*A63-A16*A21*A45*A52*   
     1 A63+A11*A26*A45*A52*A63+A15*A21*A46*A52*A63-
     1 A11*A25*A46*A52*A63-A16*A22*     
     1 A41*A55*A63+A12*A26*A41*A55*A63+A16*A21*A42*
     1 A55*A63-A11*A26*A42*A55*   
     1 A63-A12*A21*A46*A55*A63+A11*A22*A46*A55*A63+
     1 A15*A22*A41*A56*A63-A12*A25*     
     1 A41*A56*A63-A15*A21*A42*A56*A63+A11*A25*A42*
     1 A56*A63+A12*A21*A45*A56*   
     1 A63-A11*A22*A45*A56*A63+A16*A23*A42*A51*A65-
     1 A13*A26*A42*A51*A65-A16*A22*     
     1 A43*A51*A65+A12*A26*A43*A51*A65+A13*A22*A46*
     1 A51*A65-A12*A23*A46*A51*   
     1 A65-A16*A23*A41*A52*A65+A13*A26*A41*A52*A65+
     1 A16*A21*A43*A52*A65-A11*A26*     
     1 A43*A52*A65-A13*A21*A46*A52*A65+A11*A23*A46*
     1 A52*A65+A16*A22*A41*A53*   
     1 A65-A12*A26*A41*A53*A65-A16*A21*A42*A53*A65+
     1 A11*A26*A42*A53*A65+A12*A21*     
     1 A46*A53*A65-A11*A22*A46*A53*A65-A13*A22*A41*
     1 A56*A65+A12*A23*A41*A56*   
     1 A65+A13*A21*A42*A56*A65-A11*A23*A42*A56*A65-
     1 A12*A21*A43*A56*A65+A11*A22*     
     1 A43*A56*A65-A15*A23*A42*A51*A66+A13*A25*A42*
     1 A51*A66+A15*A22*A43*A51*   
     1 A66-A12*A25*A43*A51*A66-A13*A22*A45*A51*A66+
     1 A12*A23*A45*A51*A66+A15*A23*     
     1 A41*A52*A66-A13*A25*A41*A52*A66-A15*A21*A43*
     1 A52*A66+A11*A25*A43*A52*   
     1 A66+A13*A21*A45*A52*A66-A11*A23*A45*A52*A66-
     1 A15*A22*A41*A53*A66+A12*A25*     
     1 A41*A53*A66+A15*A21*A42*A53*A66-A11*A25*A42*
     1 A53*A66-A12*A21*A45*A53*   
     1 A66+A11*A22*A45*A53*A66+A13*A22*A41*A55*A66-
     1 A12*A23*A41*A55*A66-A13*A21*     
     1 A42*A55*A66+A11*A23*A42*A55*A66+A12*A21*A43*
     1 A55*A66-A11*A22*A43*A55*   
     1 A66

      COFACTOR(4,4) = A16*A25*A33*A52*A61-A15*A26*
     1 A33*A52*A61-A16*A23*A35*A52*A61+A13*A26*     
     1 A35*A52*A61+A15*A23*A36*A52*A61-A13*A25*A36*
     1 A52*A61-A16*A25*A32*A53*   
     1 A61+A15*A26*A32*A53*A61+A16*A22*A35*A53*A61-
     1 A12*A26*A35*A53*A61-A15*A22*     
     1 A36*A53*A61+A12*A25*A36*A53*A61+A16*A23*A32*
     1 A55*A61-A13*A26*A32*A55*   
     1 A61-A16*A22*A33*A55*A61+A12*A26*A33*A55*A61+
     1 A13*A22*A36*A55*A61-A12*A23*     
     1 A36*A55*A61-A15*A23*A32*A56*A61+A13*A25*A32*
     1 A56*A61+A15*A22*A33*A56*   
     1 A61-A12*A25*A33*A56*A61-A13*A22*A35*A56*A61+
     1 A12*A23*A35*A56*A61-A16*A25*     
     1 A33*A51*A62+A15*A26*A33*A51*A62+A16*A23*A35*
     1 A51*A62-A13*A26*A35*A51*   
     1 A62-A15*A23*A36*A51*A62+A13*A25*A36*A51*A62+
     1 A16*A25*A31*A53*A62-A15*A26*     
     1 A31*A53*A62-A16*A21*A35*A53*A62+A11*A26*A35*
     1 A53*A62+A15*A21*A36*A53*   
     1 A62-A11*A25*A36*A53*A62-A16*A23*A31*A55*A62+
     1 A13*A26*A31*A55*A62+A16*A21*     
     1 A33*A55*A62-A11*A26*A33*A55*A62-A13*A21*A36*
     1 A55*A62+A11*A23*A36*A55*   
     1 A62+A15*A23*A31*A56*A62-A13*A25*A31*A56*A62-
     1 A15*A21*A33*A56*A62+A11*A25*     
     1 A33*A56*A62+A13*A21*A35*A56*A62-A11*A23*A35*
     1 A56*A62+A16*A25*A32*A51*   
     1 A63-A15*A26*A32*A51*A63-A16*A22*A35*A51*A63+
     1 A12*A26*A35*A51*A63+A15*A22*     
     1 A36*A51*A63-A12*A25*A36*A51*A63-A16*A25*A31*
     1 A52*A63+A15*A26*A31*A52*   
     1 A63+A16*A21*A35*A52*A63-A11*A26*A35*A52*A63-
     1 A15*A21*A36*A52*A63+A11*A25*     
     1 A36*A52*A63+A16*A22*A31*A55*A63-A12*A26*A31*
     1 A55*A63-A16*A21*A32*A55*   
     1 A63+A11*A26*A32*A55*A63+A12*A21*A36*A55*A63-
     1 A11*A22*A36*A55*A63-A15*A22*     
     1 A31*A56*A63+A12*A25*A31*A56*A63+A15*A21*A32*
     1 A56*A63-A11*A25*A32*A56*   
     1 A63-A12*A21*A35*A56*A63+A11*A22*A35*A56*A63-
     1 A16*A23*A32*A51*A65+A13*A26*     
     1 A32*A51*A65+A16*A22*A33*A51*A65-A12*A26*A33*
     1 A51*A65-A13*A22*A36*A51*   
     1 A65+A12*A23*A36*A51*A65+A16*A23*A31*A52*A65-
     1 A13*A26*A31*A52*A65-A16*A21*     
     1 A33*A52*A65+A11*A26*A33*A52*A65+A13*A21*A36*
     1 A52*A65-A11*A23*A36*A52*   
     1 A65-A16*A22*A31*A53*A65+A12*A26*A31*A53*A65+
     1 A16*A21*A32*A53*A65-A11*A26*     
     1 A32*A53*A65-A12*A21*A36*A53*A65+A11*A22*A36*
     1 A53*A65+A13*A22*A31*A56*   
     1 A65-A12*A23*A31*A56*A65-A13*A21*A32*A56*A65+
     1 A11*A23*A32*A56*A65+A12*A21*     
     1 A33*A56*A65-A11*A22*A33*A56*A65+A15*A23*A32*
     1 A51*A66-A13*A25*A32*A51*   
     1 A66-A15*A22*A33*A51*A66+A12*A25*A33*A51*A66+
     1 A13*A22*A35*A51*A66-A12*A23*     
     1 A35*A51*A66-A15*A23*A31*A52*A66+A13*A25*A31*
     1 A52*A66+A15*A21*A33*A52*   
     1 A66-A11*A25*A33*A52*A66-A13*A21*A35*A52*A66+
     1 A11*A23*A35*A52*A66+A15*A22*     
     1 A31*A53*A66-A12*A25*A31*A53*A66-A15*A21*A32*
     1 A53*A66+A11*A25*A32*A53*   
     1 A66+A12*A21*A35*A53*A66-A11*A22*A35*A53*A66-
     1 A13*A22*A31*A55*A66+A12*A23*     
     1 A31*A55*A66+A13*A21*A32*A55*A66-A11*A23*A32*
     1 A55*A66-A12*A21*A33*A55*   
     1 A66+A11*A22*A33*A55*A66

      COFACTOR(5,4) = -A16*A25*A33*A42*A61+A15*A26*
     1 A33*A42*A61+A16*A23*
     1 A35*A42*A61-A13*A26*A35*A42*A61-A15*A23*A36*
     1 A42*A61+A13*A25*A36*A42*   
     1 A61+A16*A25*A32*A43*A61-A15*A26*A32*A43*A61-
     1 A16*A22*A35*A43*A61+A12*A26*     
     1 A35*A43*A61+A15*A22*A36*A43*A61-A12*A25*A36*
     1 A43*A61-A16*A23*A32*A45*   
     1 A61+A13*A26*A32*A45*A61+A16*A22*A33*A45*A61-
     1 A12*A26*A33*A45*A61-A13*A22*     
     1 A36*A45*A61+A12*A23*A36*A45*A61+A15*A23*A32*
     1 A46*A61-A13*A25*A32*A46*   
     1 A61-A15*A22*A33*A46*A61+A12*A25*A33*A46*A61+
     1 A13*A22*A35*A46*A61-A12*A23*     
     1 A35*A46*A61+A16*A25*A33*A41*A62-A15*A26*A33*
     1 A41*A62-A16*A23*A35*A41*   
     1 A62+A13*A26*A35*A41*A62+A15*A23*A36*A41*A62-
     1 A13*A25*A36*A41*A62-A16*A25*     
     1 A31*A43*A62+A15*A26*A31*A43*A62+A16*A21*A35*
     1 A43*A62-A11*A26*A35*A43*   
     1 A62-A15*A21*A36*A43*A62+A11*A25*A36*A43*A62+
     1 A16*A23*A31*A45*A62-A13*A26*     
     1 A31*A45*A62-A16*A21*A33*A45*A62+A11*A26*A33*
     1 A45*A62+A13*A21*A36*A45*   
     1 A62-A11*A23*A36*A45*A62-A15*A23*A31*A46*A62+
     1 A13*A25*A31*A46*A62+A15*A21*     
     1 A33*A46*A62-A11*A25*A33*A46*A62-A13*A21*A35*
     1 A46*A62+A11*A23*A35*A46*   
     1 A62-A16*A25*A32*A41*A63+A15*A26*A32*A41*A63+
     1 A16*A22*A35*A41*A63-A12*A26*     
     1 A35*A41*A63-A15*A22*A36*A41*A63+A12*A25*A36*
     1 A41*A63+A16*A25*A31*A42*   
     1 A63-A15*A26*A31*A42*A63-A16*A21*A35*A42*A63+
     1 A11*A26*A35*A42*A63+A15*A21*     
     1 A36*A42*A63-A11*A25*A36*A42*A63-A16*A22*A31*
     1 A45*A63+A12*A26*A31*A45*   
     1 A63+A16*A21*A32*A45*A63-A11*A26*A32*A45*A63-
     1 A12*A21*A36*A45*A63+A11*A22*     
     1 A36*A45*A63+A15*A22*A31*A46*A63-A12*A25*A31*
     1 A46*A63-A15*A21*A32*A46*   
     1 A63+A11*A25*A32*A46*A63+A12*A21*A35*A46*A63-
     1 A11*A22*A35*A46*A63+A16*A23*     
     1 A32*A41*A65-A13*A26*A32*A41*A65-A16*A22*A33*
     1 A41*A65+A12*A26*A33*A41*   
     1 A65+A13*A22*A36*A41*A65-A12*A23*A36*A41*A65-
     1 A16*A23*A31*A42*A65+A13*A26*     
     1 A31*A42*A65+A16*A21*A33*A42*A65-A11*A26*A33*
     1 A42*A65-A13*A21*A36*A42*   
     1 A65+A11*A23*A36*A42*A65+A16*A22*A31*A43*A65-
     1 A12*A26*A31*A43*A65-A16*A21*     
     1 A32*A43*A65+A11*A26*A32*A43*A65+A12*A21*A36*
     1 A43*A65-A11*A22*A36*A43*   
     1 A65-A13*A22*A31*A46*A65+A12*A23*A31*A46*A65+
     1 A13*A21*A32*A46*A65-A11*A23*     
     1 A32*A46*A65-A12*A21*A33*A46*A65+A11*A22*A33*
     1 A46*A65-A15*A23*A32*A41*   
     1 A66+A13*A25*A32*A41*A66+A15*A22*A33*A41*A66-
     1 A12*A25*A33*A41*A66-A13*A22*     
     1 A35*A41*A66+A12*A23*A35*A41*A66+A15*A23*A31*
     1 A42*A66-A13*A25*A31*A42*   
     1 A66-A15*A21*A33*A42*A66+A11*A25*A33*A42*A66+
     1 A13*A21*A35*A42*A66-A11*A23*     
     1 A35*A42*A66-A15*A22*A31*A43*A66+A12*A25*A31*
     1 A43*A66+A15*A21*A32*A43*   
     1 A66-A11*A25*A32*A43*A66-A12*A21*A35*A43*A66+
     1 A11*A22*A35*A43*A66+A13*A22*     
     1 A31*A45*A66-A12*A23*A31*A45*A66-A13*A21*A32*
     1 A45*A66+A11*A23*A32*A45*   
     1 A66+A12*A21*A33*A45*A66-A11*A22*A33*A45*A66

      COFACTOR(6,4) = A16*A25*A33*A42*A51-A15*A26*   
     1 A33*A42*A51-A16*A23*A35*A42*A51+A13*A26*A35*
     1 A42*A51+A15*A23*A36*A42*   
     1 A51-A13*A25*A36*A42*A51-A16*A25*A32*A43*A51+
     1 A15*A26*A32*A43*A51+A16*A22*     
     1 A35*A43*A51-A12*A26*A35*A43*A51-A15*A22*A36*
     1 A43*A51+A12*A25*A36*A43*   
     1 A51+A16*A23*A32*A45*A51-A13*A26*A32*A45*A51-
     1 A16*A22*A33*A45*A51+A12*A26*     
     1 A33*A45*A51+A13*A22*A36*A45*A51-A12*A23*A36*
     1 A45*A51-A15*A23*A32*A46*   
     1 A51+A13*A25*A32*A46*A51+A15*A22*A33*A46*A51-
     1 A12*A25*A33*A46*A51-A13*A22*     
     1 A35*A46*A51+A12*A23*A35*A46*A51-A16*A25*A33*
     1 A41*A52+A15*A26*A33*A41*   
     1 A52+A16*A23*A35*A41*A52-A13*A26*A35*A41*A52-
     1 A15*A23*A36*A41*A52+A13*A25*     
     1 A36*A41*A52+A16*A25*A31*A43*A52-A15*A26*A31*
     1 A43*A52-A16*A21*A35*A43*   
     1 A52+A11*A26*A35*A43*A52+A15*A21*A36*A43*A52-
     1 A11*A25*A36*A43*A52-A16*A23*     
     1 A31*A45*A52+A13*A26*A31*A45*A52+A16*A21*A33*
     1 A45*A52-A11*A26*A33*A45*   
     1 A52-A13*A21*A36*A45*A52+A11*A23*A36*A45*A52+
     1 A15*A23*A31*A46*A52-A13*A25*     
     1 A31*A46*A52-A15*A21*A33*A46*A52+A11*A25*A33*
     1 A46*A52+A13*A21*A35*A46*   
     1 A52-A11*A23*A35*A46*A52+A16*A25*A32*A41*A53-
     1 A15*A26*A32*A41*A53-A16*A22*     
     1 A35*A41*A53+A12*A26*A35*A41*A53+A15*A22*A36*
     1 A41*A53-A12*A25*A36*A41*   
     1 A53-A16*A25*A31*A42*A53+A15*A26*A31*A42*A53+
     1 A16*A21*A35*A42*A53-A11*A26*     
     1 A35*A42*A53-A15*A21*A36*A42*A53+A11*A25*A36*
     1 A42*A53+A16*A22*A31*A45*   
     1 A53-A12*A26*A31*A45*A53-A16*A21*A32*A45*A53+
     1 A11*A26*A32*A45*A53+A12*A21*     
     1 A36*A45*A53-A11*A22*A36*A45*A53-A15*A22*A31*
     1 A46*A53+A12*A25*A31*A46*   
     1 A53+A15*A21*A32*A46*A53-A11*A25*A32*A46*A53-
     1 A12*A21*A35*A46*A53+A11*A22*     
     1 A35*A46*A53-A16*A23*A32*A41*A55+A13*A26*A32*
     1 A41*A55+A16*A22*A33*A41*   
     1 A55-A12*A26*A33*A41*A55-A13*A22*A36*A41*A55+
     1 A12*A23*A36*A41*A55+A16*A23*     
     1 A31*A42*A55-A13*A26*A31*A42*A55-A16*A21*A33*
     1 A42*A55+A11*A26*A33*A42*   
     1 A55+A13*A21*A36*A42*A55-A11*A23*A36*A42*A55-
     1 A16*A22*A31*A43*A55+A12*A26*     
     1 A31*A43*A55+A16*A21*A32*A43*A55-A11*A26*A32*
     1 A43*A55-A12*A21*A36*A43*   
     1 A55+A11*A22*A36*A43*A55+A13*A22*A31*A46*A55-
     1 A12*A23*A31*A46*A55-A13*A21*     
     1 A32*A46*A55+A11*A23*A32*A46*A55+A12*A21*A33*
     1 A46*A55-A11*A22*A33*A46*   
     1 A55+A15*A23*A32*A41*A56-A13*A25*A32*A41*A56-
     1 A15*A22*A33*A41*A56+A12*A25*     
     1 A33*A41*A56+A13*A22*A35*A41*A56-A12*A23*A35*
     1 A41*A56-A15*A23*A31*A42*   
     1 A56+A13*A25*A31*A42*A56+A15*A21*A33*A42*A56-
     1 A11*A25*A33*A42*A56-A13*A21*     
     1 A35*A42*A56+A11*A23*A35*A42*A56+A15*A22*A31*
     1 A43*A56-A12*A25*A31*A43*   
     1 A56-A15*A21*A32*A43*A56+A11*A25*A32*A43*A56+
     1 A12*A21*A35*A43*A56-A11*A22*     
     1 A35*A43*A56-A13*A22*A31*A45*A56+A12*A23*A31*
     1 A45*A56+A13*A21*A32*A45*   
     1 A56-A11*A23*A32*A45*A56-A12*A21*A33*A45*A56+
     1 A11*A22*A33*A45*A56

      COFACTOR(1,5) = A26*   
     1 A34*A43*A52*A61-A24*A36*A43*A52*A61-A26*A33*
     1 A44*A52*A61+A23*A36*A44*A52*     
     1 A61+A24*A33*A46*A52*A61-A23*A34*A46*A52*A61-
     1 A26*A34*A42*A53*A61+A24*A36*     
     1 A42*A53*A61+A26*A32*A44*A53*A61-A22*A36*A44*
     1 A53*A61-A24*A32*A46*A53*   
     1 A61+A22*A34*A46*A53*A61+A26*A33*A42*A54*A61-
     1 A23*A36*A42*A54*A61-A26*A32*     
     1 A43*A54*A61+A22*A36*A43*A54*A61+A23*A32*A46*
     1 A54*A61-A22*A33*A46*A54*   
     1 A61-A24*A33*A42*A56*A61+A23*A34*A42*A56*A61+
     1 A24*A32*A43*A56*A61-A22*A34*     
     1 A43*A56*A61-A23*A32*A44*A56*A61+A22*A33*A44*
     1 A56*A61-A26*A34*A43*A51*   
     1 A62+A24*A36*A43*A51*A62+A26*A33*A44*A51*A62-
     1 A23*A36*A44*A51*A62-A24*A33*     
     1 A46*A51*A62+A23*A34*A46*A51*A62+A26*A34*A41*
     1 A53*A62-A24*A36*A41*A53*   
     1 A62-A26*A31*A44*A53*A62+A21*A36*A44*A53*A62+
     1 A24*A31*A46*A53*A62-A21*A34*     
     1 A46*A53*A62-A26*A33*A41*A54*A62+A23*A36*A41*
     1 A54*A62+A26*A31*A43*A54*   
     1 A62-A21*A36*A43*A54*A62-A23*A31*A46*A54*A62+
     1 A21*A33*A46*A54*A62+A24*A33*     
     1 A41*A56*A62-A23*A34*A41*A56*A62-A24*A31*A43*
     1 A56*A62+A21*A34*A43*A56*   
     1 A62+A23*A31*A44*A56*A62-A21*A33*A44*A56*A62+
     1 A26*A34*A42*A51*A63-A24*A36*     
     1 A42*A51*A63-A26*A32*A44*A51*A63+A22*A36*A44*
     1 A51*A63+A24*A32*A46*A51*   
     1 A63-A22*A34*A46*A51*A63-A26*A34*A41*A52*A63+
     1 A24*A36*A41*A52*A63+A26*A31*     
     1 A44*A52*A63-A21*A36*A44*A52*A63-A24*A31*A46*
     1 A52*A63+A21*A34*A46*A52*   
     1 A63+A26*A32*A41*A54*A63-A22*A36*A41*A54*A63-
     1 A26*A31*A42*A54*A63+A21*A36*     
     1 A42*A54*A63+A22*A31*A46*A54*A63-A21*A32*A46*
     1 A54*A63-A24*A32*A41*A56*   
     1 A63+A22*A34*A41*A56*A63+A24*A31*A42*A56*A63-
     1 A21*A34*A42*A56*A63-A22*A31*     
     1 A44*A56*A63+A21*A32*A44*A56*A63-A26*A33*A42*
     1 A51*A64+A23*A36*A42*A51*   
     1 A64+A26*A32*A43*A51*A64-A22*A36*A43*A51*A64-
     1 A23*A32*A46*A51*A64+A22*A33*     
     1 A46*A51*A64+A26*A33*A41*A52*A64-A23*A36*A41*
     1 A52*A64-A26*A31*A43*A52*   
     1 A64+A21*A36*A43*A52*A64+A23*A31*A46*A52*A64-
     1 A21*A33*A46*A52*A64-A26*A32*     
     1 A41*A53*A64+A22*A36*A41*A53*A64+A26*A31*A42*
     1 A53*A64-A21*A36*A42*A53*   
     1 A64-A22*A31*A46*A53*A64+A21*A32*A46*A53*A64+
     1 A23*A32*A41*A56*A64-A22*A33*     
     1 A41*A56*A64-A23*A31*A42*A56*A64+A21*A33*A42*
     1 A56*A64+A22*A31*A43*A56*   
     1 A64-A21*A32*A43*A56*A64+A24*A33*A42*A51*A66-
     1 A23*A34*A42*A51*A66-A24*A32*     
     1 A43*A51*A66+A22*A34*A43*A51*A66+A23*A32*A44*
     1 A51*A66-A22*A33*A44*A51*   
     1 A66-A24*A33*A41*A52*A66+A23*A34*A41*A52*A66+
     1 A24*A31*A43*A52*A66-A21*A34*     
     1 A43*A52*A66-A23*A31*A44*A52*A66+A21*A33*A44*
     1 A52*A66+A24*A32*A41*A53*   
     1 A66-A22*A34*A41*A53*A66-A24*A31*A42*A53*A66+
     1 A21*A34*A42*A53*A66+A22*A31*     
     1 A44*A53*A66-A21*A32*A44*A53*A66-A23*A32*A41*
     1 A54*A66+A22*A33*A41*A54*   
     1 A66+A23*A31*A42*A54*A66-A21*A33*A42*A54*A66-
     1 A22*A31*A43*A54*A66+A21*A32*     
     1 A43*A54*A66

      COFACTOR(2,5) = -A16*A34*A43*A52*A61+A14*A36*
     1 A43*A52*A61+A16*A33*A44*A52*    
     1 A61-A13*A36*A44*A52*A61-A14*A33*A46*A52*A61+
     1 A13*A34*A46*A52*A61+A16*A34*     
     1 A42*A53*A61-A14*A36*A42*A53*A61-A16*A32*A44*
     1 A53*A61+A12*A36*A44*A53*   
     1 A61+A14*A32*A46*A53*A61-A12*A34*A46*A53*A61-
     1 A16*A33*A42*A54*A61+A13*A36*     
     1 A42*A54*A61+A16*A32*A43*A54*A61-A12*A36*A43*
     1 A54*A61-A13*A32*A46*A54*   
     1 A61+A12*A33*A46*A54*A61+A14*A33*A42*A56*A61-
     1 A13*A34*A42*A56*A61-A14*A32*     
     1 A43*A56*A61+A12*A34*A43*A56*A61+A13*A32*A44*
     1 A56*A61-A12*A33*A44*A56*   
     1 A61+A16*A34*A43*A51*A62-A14*A36*A43*A51*A62-
     1 A16*A33*A44*A51*A62+A13*A36*     
     1 A44*A51*A62+A14*A33*A46*A51*A62-A13*A34*A46*
     1 A51*A62-A16*A34*A41*A53*   
     1 A62+A14*A36*A41*A53*A62+A16*A31*A44*A53*A62-
     1 A11*A36*A44*A53*A62-A14*A31*     
     1 A46*A53*A62+A11*A34*A46*A53*A62+A16*A33*A41*
     1 A54*A62-A13*A36*A41*A54*   
     1 A62-A16*A31*A43*A54*A62+A11*A36*A43*A54*A62+
     1 A13*A31*A46*A54*A62-A11*A33*     
     1 A46*A54*A62-A14*A33*A41*A56*A62+A13*A34*A41*
     1 A56*A62+A14*A31*A43*A56*   
     1 A62-A11*A34*A43*A56*A62-A13*A31*A44*A56*A62+
     1 A11*A33*A44*A56*A62-A16*A34*     
     1 A42*A51*A63+A14*A36*A42*A51*A63+A16*A32*A44*
     1 A51*A63-A12*A36*A44*A51*   
     1 A63-A14*A32*A46*A51*A63+A12*A34*A46*A51*A63+
     1 A16*A34*A41*A52*A63-A14*A36*     
     1 A41*A52*A63-A16*A31*A44*A52*A63+A11*A36*A44*
     1 A52*A63+A14*A31*A46*A52*   
     1 A63-A11*A34*A46*A52*A63-A16*A32*A41*A54*A63+
     1 A12*A36*A41*A54*A63+A16*A31*     
     1 A42*A54*A63-A11*A36*A42*A54*A63-A12*A31*A46*
     1 A54*A63+A11*A32*A46*A54*   
     1 A63+A14*A32*A41*A56*A63-A12*A34*A41*A56*A63-
     1 A14*A31*A42*A56*A63+A11*A34*     
     1 A42*A56*A63+A12*A31*A44*A56*A63-A11*A32*A44*
     1 A56*A63+A16*A33*A42*A51*   
     1 A64-A13*A36*A42*A51*A64-A16*A32*A43*A51*A64+
     1 A12*A36*A43*A51*A64+A13*A32*     
     1 A46*A51*A64-A12*A33*A46*A51*A64-A16*A33*A41*
     1 A52*A64+A13*A36*A41*A52*   
     1 A64+A16*A31*A43*A52*A64-A11*A36*A43*A52*A64-
     1 A13*A31*A46*A52*A64+A11*A33*     
     1 A46*A52*A64+A16*A32*A41*A53*A64-A12*A36*A41*
     1 A53*A64-A16*A31*A42*A53*   
     1 A64+A11*A36*A42*A53*A64+A12*A31*A46*A53*A64-
     1 A11*A32*A46*A53*A64-A13*A32*     
     1 A41*A56*A64+A12*A33*A41*A56*A64+A13*A31*A42*
     1 A56*A64-A11*A33*A42*A56*   
     1 A64-A12*A31*A43*A56*A64+A11*A32*A43*A56*A64-
     1 A14*A33*A42*A51*A66+A13*A34*     
     1 A42*A51*A66+A14*A32*A43*A51*A66-A12*A34*A43*
     1 A51*A66-A13*A32*A44*A51*   
     1 A66+A12*A33*A44*A51*A66+A14*A33*A41*A52*A66-
     1 A13*A34*A41*A52*A66-A14*A31*     
     1 A43*A52*A66+A11*A34*A43*A52*A66+A13*A31*A44*
     1 A52*A66-A11*A33*A44*A52*   
     1 A66-A14*A32*A41*A53*A66+A12*A34*A41*A53*A66+
     1 A14*A31*A42*A53*A66-A11*A34*     
     1 A42*A53*A66-A12*A31*A44*A53*A66+A11*A32*A44*
     1 A53*A66+A13*A32*A41*A54*   
     1 A66-A12*A33*A41*A54*A66-A13*A31*A42*A54*A66+
     1 A11*A33*A42*A54*A66+A12*A31*     
     1 A43*A54*A66-A11*A32*A43*A54*A66

      COFACTOR(3,5) = A16*A24*A43*A52*A61-A14*A26*A43*A52* 
     1 A61-A16*A23*A44*A52*A61+A13*A26*A44*A52*A61+
     1 A14*A23*A46*A52*A61-A13*A24*     
     1 A46*A52*A61-A16*A24*A42*A53*A61+A14*A26*A42*
     1 A53*A61+A16*A22*A44*A53*   
     1 A61-A12*A26*A44*A53*A61-A14*A22*A46*A53*A61+
     1 A12*A24*A46*A53*A61+A16*A23*     
     1 A42*A54*A61-A13*A26*A42*A54*A61-A16*A22*A43*
     1 A54*A61+A12*A26*A43*A54*   
     1 A61+A13*A22*A46*A54*A61-A12*A23*A46*A54*A61-
     1 A14*A23*A42*A56*A61+A13*A24*     
     1 A42*A56*A61+A14*A22*A43*A56*A61-A12*A24*A43*
     1 A56*A61-A13*A22*A44*A56*   
     1 A61+A12*A23*A44*A56*A61-A16*A24*A43*A51*A62+
     1 A14*A26*A43*A51*A62+A16*A23*     
     1 A44*A51*A62-A13*A26*A44*A51*A62-A14*A23*A46*
     1 A51*A62+A13*A24*A46*A51*   
     1 A62+A16*A24*A41*A53*A62-A14*A26*A41*A53*A62-
     1 A16*A21*A44*A53*A62+A11*A26*     
     1 A44*A53*A62+A14*A21*A46*A53*A62-A11*A24*A46*
     1 A53*A62-A16*A23*A41*A54*   
     1 A62+A13*A26*A41*A54*A62+A16*A21*A43*A54*A62-
     1 A11*A26*A43*A54*A62-A13*A21*     
     1 A46*A54*A62+A11*A23*A46*A54*A62+A14*A23*A41*
     1 A56*A62-A13*A24*A41*A56*   
     1 A62-A14*A21*A43*A56*A62+A11*A24*A43*A56*A62+
     1 A13*A21*A44*A56*A62-A11*A23*     
     1 A44*A56*A62+A16*A24*A42*A51*A63-A14*A26*A42*
     1 A51*A63-A16*A22*A44*A51*   
     1 A63+A12*A26*A44*A51*A63+A14*A22*A46*A51*A63-
     1 A12*A24*A46*A51*A63-A16*A24*     
     1 A41*A52*A63+A14*A26*A41*A52*A63+A16*A21*A44*
     1 A52*A63-A11*A26*A44*A52*   
     1 A63-A14*A21*A46*A52*A63+A11*A24*A46*A52*A63+
     1 A16*A22*A41*A54*A63-A12*A26*     
     1 A41*A54*A63-A16*A21*A42*A54*A63+A11*A26*A42*
     1 A54*A63+A12*A21*A46*A54*   
     1 A63-A11*A22*A46*A54*A63-A14*A22*A41*A56*A63+
     1 A12*A24*A41*A56*A63+A14*A21*     
     1 A42*A56*A63-A11*A24*A42*A56*A63-A12*A21*A44*
     1 A56*A63+A11*A22*A44*A56*   
     1 A63-A16*A23*A42*A51*A64+A13*A26*A42*A51*A64+
     1 A16*A22*A43*A51*A64-A12*A26*     
     1 A43*A51*A64-A13*A22*A46*A51*A64+A12*A23*A46*
     1 A51*A64+A16*A23*A41*A52*   
     1 A64-A13*A26*A41*A52*A64-A16*A21*A43*A52*A64+
     1 A11*A26*A43*A52*A64+A13*A21*     
     1 A46*A52*A64-A11*A23*A46*A52*A64-A16*A22*A41*
     1 A53*A64+A12*A26*A41*A53*   
     1 A64+A16*A21*A42*A53*A64-A11*A26*A42*A53*A64-
     1 A12*A21*A46*A53*A64+A11*A22*     
     1 A46*A53*A64+A13*A22*A41*A56*A64-A12*A23*A41*
     1 A56*A64-A13*A21*A42*A56*   
     1 A64+A11*A23*A42*A56*A64+A12*A21*A43*A56*A64-
     1 A11*A22*A43*A56*A64+A14*A23*     
     1 A42*A51*A66-A13*A24*A42*A51*A66-A14*A22*A43*
     1 A51*A66+A12*A24*A43*A51*   
     1 A66+A13*A22*A44*A51*A66-A12*A23*A44*A51*A66-
     1 A14*A23*A41*A52*A66+A13*A24*     
     1 A41*A52*A66+A14*A21*A43*A52*A66-A11*A24*A43*
     1 A52*A66-A13*A21*A44*A52*   
     1 A66+A11*A23*A44*A52*A66+A14*A22*A41*A53*A66-
     1 A12*A24*A41*A53*A66-A14*A21*     
     1 A42*A53*A66+A11*A24*A42*A53*A66+A12*A21*A44*
     1 A53*A66-A11*A22*A44*A53*   
     1 A66-A13*A22*A41*A54*A66+A12*A23*A41*A54*A66+
     1 A13*A21*A42*A54*A66-A11*A23*     
     1 A42*A54*A66-A12*A21*A43*A54*A66+A11*A22*A43*A54*A66

      COFACTOR(4,5) = -A16*A24*A33*A52*  
     1 A61+A14*A26*A33*A52*A61+A16*A23*A34*A52*A61-
     1 A13*A26*A34*A52*A61-A14*A23*     
     1 A36*A52*A61+A13*A24*A36*A52*A61+A16*A24*A32*
     1 A53*A61-A14*A26*A32*A53*   
     1 A61-A16*A22*A34*A53*A61+A12*A26*A34*A53*A61+
     1 A14*A22*A36*A53*A61-A12*A24*     
     1 A36*A53*A61-A16*A23*A32*A54*A61+A13*A26*A32*
     1 A54*A61+A16*A22*A33*A54*   
     1 A61-A12*A26*A33*A54*A61-A13*A22*A36*A54*A61+
     1 A12*A23*A36*A54*A61+A14*A23*     
     1 A32*A56*A61-A13*A24*A32*A56*A61-A14*A22*A33*
     1 A56*A61+A12*A24*A33*A56*   
     1 A61+A13*A22*A34*A56*A61-A12*A23*A34*A56*A61+
     1 A16*A24*A33*A51*A62-A14*A26*     
     1 A33*A51*A62-A16*A23*A34*A51*A62+A13*A26*A34*
     1 A51*A62+A14*A23*A36*A51*   
     1 A62-A13*A24*A36*A51*A62-A16*A24*A31*A53*A62+
     1 A14*A26*A31*A53*A62+A16*A21*     
     1 A34*A53*A62-A11*A26*A34*A53*A62-A14*A21*A36*
     1 A53*A62+A11*A24*A36*A53*   
     1 A62+A16*A23*A31*A54*A62-A13*A26*A31*A54*A62-
     1 A16*A21*A33*A54*A62+A11*A26*     
     1 A33*A54*A62+A13*A21*A36*A54*A62-A11*A23*A36*
     1 A54*A62-A14*A23*A31*A56*   
     1 A62+A13*A24*A31*A56*A62+A14*A21*A33*A56*A62-
     1 A11*A24*A33*A56*A62-A13*A21*     
     1 A34*A56*A62+A11*A23*A34*A56*A62-A16*A24*A32*
     1 A51*A63+A14*A26*A32*A51*   
     1 A63+A16*A22*A34*A51*A63-A12*A26*A34*A51*A63-
     1 A14*A22*A36*A51*A63+A12*A24*     
     1 A36*A51*A63+A16*A24*A31*A52*A63-A14*A26*A31*
     1 A52*A63-A16*A21*A34*A52*   
     1 A63+A11*A26*A34*A52*A63+A14*A21*A36*A52*A63-
     1 A11*A24*A36*A52*A63-A16*A22*     
     1 A31*A54*A63+A12*A26*A31*A54*A63+A16*A21*A32*
     1 A54*A63-A11*A26*A32*A54*   
     1 A63-A12*A21*A36*A54*A63+A11*A22*A36*A54*A63+
     1 A14*A22*A31*A56*A63-A12*A24*     
     1 A31*A56*A63-A14*A21*A32*A56*A63+A11*A24*A32*
     1 A56*A63+A12*A21*A34*A56*   
     1 A63-A11*A22*A34*A56*A63+A16*A23*A32*A51*A64-
     1 A13*A26*A32*A51*A64-A16*A22*     
     1 A33*A51*A64+A12*A26*A33*A51*A64+A13*A22*A36*
     1 A51*A64-A12*A23*A36*A51*   
     1 A64-A16*A23*A31*A52*A64+A13*A26*A31*A52*A64+
     1 A16*A21*A33*A52*A64-A11*A26*     
     1 A33*A52*A64-A13*A21*A36*A52*A64+A11*A23*A36*
     1 A52*A64+A16*A22*A31*A53*   
     1 A64-A12*A26*A31*A53*A64-A16*A21*A32*A53*A64+
     1 A11*A26*A32*A53*A64+A12*A21*     
     1 A36*A53*A64-A11*A22*A36*A53*A64-A13*A22*A31*
     1 A56*A64+A12*A23*A31*A56*   
     1 A64+A13*A21*A32*A56*A64-A11*A23*A32*A56*A64-
     1 A12*A21*A33*A56*A64+A11*A22*     
     1 A33*A56*A64-A14*A23*A32*A51*A66+A13*A24*A32*
     1 A51*A66+A14*A22*A33*A51*   
     1 A66-A12*A24*A33*A51*A66-A13*A22*A34*A51*A66+
     1 A12*A23*A34*A51*A66+A14*A23*     
     1 A31*A52*A66-A13*A24*A31*A52*A66-A14*A21*A33*
     1 A52*A66+A11*A24*A33*A52*   
     1 A66+A13*A21*A34*A52*A66-A11*A23*A34*A52*A66-
     1 A14*A22*A31*A53*A66+A12*A24*     
     1 A31*A53*A66+A14*A21*A32*A53*A66-A11*A24*A32*
     1 A53*A66-A12*A21*A34*A53*   
     1 A66+A11*A22*A34*A53*A66+A13*A22*A31*A54*A66-
     1 A12*A23*A31*A54*A66-A13*A21*     
     1 A32*A54*A66+A11*A23*A32*A54*A66+A12*A21*A33*
     1 A54*A66-A11*A22*A33*A54*   
     1 A66

      COFACTOR(5,5) = A16*A24*A33*A42*A61-A14*A26*
     1 A33*A42*A61-A16*A23*A34*A42*A61+A13*A26*     
     1 A34*A42*A61+A14*A23*A36*A42*A61-A13*A24*A36*
     1 A42*A61-A16*A24*A32*A43*   
     1 A61+A14*A26*A32*A43*A61+A16*A22*A34*A43*A61-
     1 A12*A26*A34*A43*A61-A14*A22*     
     1 A36*A43*A61+A12*A24*A36*A43*A61+A16*A23*A32*
     1 A44*A61-A13*A26*A32*A44*   
     1 A61-A16*A22*A33*A44*A61+A12*A26*A33*A44*A61+
     1 A13*A22*A36*A44*A61-A12*A23*     
     1 A36*A44*A61-A14*A23*A32*A46*A61+A13*A24*A32*
     1 A46*A61+A14*A22*A33*A46*   
     1 A61-A12*A24*A33*A46*A61-A13*A22*A34*A46*A61+
     1 A12*A23*A34*A46*A61-A16*A24*     
     1 A33*A41*A62+A14*A26*A33*A41*A62+A16*A23*A34*
     1 A41*A62-A13*A26*A34*A41*   
     1 A62-A14*A23*A36*A41*A62+A13*A24*A36*A41*A62+
     1 A16*A24*A31*A43*A62-A14*A26*     
     1 A31*A43*A62-A16*A21*A34*A43*A62+A11*A26*A34*
     1 A43*A62+A14*A21*A36*A43*   
     1 A62-A11*A24*A36*A43*A62-A16*A23*A31*A44*A62+
     1 A13*A26*A31*A44*A62+A16*A21*     
     1 A33*A44*A62-A11*A26*A33*A44*A62-A13*A21*A36*
     1 A44*A62+A11*A23*A36*A44*   
     1 A62+A14*A23*A31*A46*A62-A13*A24*A31*A46*A62-
     1 A14*A21*A33*A46*A62+A11*A24*     
     1 A33*A46*A62+A13*A21*A34*A46*A62-A11*A23*A34*
     1 A46*A62+A16*A24*A32*A41*   
     1 A63-A14*A26*A32*A41*A63-A16*A22*A34*A41*A63+
     1 A12*A26*A34*A41*A63+A14*A22*     
     1 A36*A41*A63-A12*A24*A36*A41*A63-A16*A24*A31*
     1 A42*A63+A14*A26*A31*A42*   
     1 A63+A16*A21*A34*A42*A63-A11*A26*A34*A42*A63-
     1 A14*A21*A36*A42*A63+A11*A24*     
     1 A36*A42*A63+A16*A22*A31*A44*A63-A12*A26*A31*
     1 A44*A63-A16*A21*A32*A44*   
     1 A63+A11*A26*A32*A44*A63+A12*A21*A36*A44*A63-
     1 A11*A22*A36*A44*A63-A14*A22*     
     1 A31*A46*A63+A12*A24*A31*A46*A63+A14*A21*A32*
     1 A46*A63-A11*A24*A32*A46*   
     1 A63-A12*A21*A34*A46*A63+A11*A22*A34*A46*A63-
     1 A16*A23*A32*A41*A64+A13*A26*     
     1 A32*A41*A64+A16*A22*A33*A41*A64-A12*A26*A33*
     1 A41*A64-A13*A22*A36*A41*   
     1 A64+A12*A23*A36*A41*A64+A16*A23*A31*A42*A64-
     1 A13*A26*A31*A42*A64-A16*A21*     
     1 A33*A42*A64+A11*A26*A33*A42*A64+A13*A21*A36*
     1 A42*A64-A11*A23*A36*A42*   
     1 A64-A16*A22*A31*A43*A64+A12*A26*A31*A43*A64+
     1 A16*A21*A32*A43*A64-A11*A26*     
     1 A32*A43*A64-A12*A21*A36*A43*A64+A11*A22*A36*
     1 A43*A64+A13*A22*A31*A46*   
     1 A64-A12*A23*A31*A46*A64-A13*A21*A32*A46*A64+
     1 A11*A23*A32*A46*A64+A12*A21*     
     1 A33*A46*A64-A11*A22*A33*A46*A64+A14*A23*A32*
     1 A41*A66-A13*A24*A32*A41*   
     1 A66-A14*A22*A33*A41*A66+A12*A24*A33*A41*A66+
     1 A13*A22*A34*A41*A66-A12*A23*     
     1 A34*A41*A66-A14*A23*A31*A42*A66+A13*A24*A31*
     1 A42*A66+A14*A21*A33*A42*   
     1 A66-A11*A24*A33*A42*A66-A13*A21*A34*A42*A66+
     1 A11*A23*A34*A42*A66+A14*A22*     
     1 A31*A43*A66-A12*A24*A31*A43*A66-A14*A21*A32*
     1 A43*A66+A11*A24*A32*A43*   
     1 A66+A12*A21*A34*A43*A66-A11*A22*A34*A43*A66-
     1 A13*A22*A31*A44*A66+A12*A23*     
     1 A31*A44*A66+A13*A21*A32*A44*A66-A11*A23*A32*
     1 A44*A66-A12*A21*A33*A44*   
     1 A66+A11*A22*A33*A44*A66

      COFACTOR(6,5) = -A16*A24*A33*A42*A51+A14*A26*
     1 A33*A42*A51+A16*A23*
     1 A34*A42*A51-A13*A26*A34*A42*A51-A14*A23*A36*
     1 A42*A51+A13*A24*A36*A42*   
     1 A51+A16*A24*A32*A43*A51-A14*A26*A32*A43*A51-
     1 A16*A22*A34*A43*A51+A12*A26*     
     1 A34*A43*A51+A14*A22*A36*A43*A51-A12*A24*A36*
     1 A43*A51-A16*A23*A32*A44*   
     1 A51+A13*A26*A32*A44*A51+A16*A22*A33*A44*A51-
     1 A12*A26*A33*A44*A51-A13*A22*     
     1 A36*A44*A51+A12*A23*A36*A44*A51+A14*A23*A32*
     1 A46*A51-A13*A24*A32*A46*   
     1 A51-A14*A22*A33*A46*A51+A12*A24*A33*A46*A51+
     1 A13*A22*A34*A46*A51-A12*A23*     
     1 A34*A46*A51+A16*A24*A33*A41*A52-A14*A26*A33*
     1 A41*A52-A16*A23*A34*A41*   
     1 A52+A13*A26*A34*A41*A52+A14*A23*A36*A41*A52-
     1 A13*A24*A36*A41*A52-A16*A24*     
     1 A31*A43*A52+A14*A26*A31*A43*A52+A16*A21*A34*
     1 A43*A52-A11*A26*A34*A43*   
     1 A52-A14*A21*A36*A43*A52+A11*A24*A36*A43*A52+
     1 A16*A23*A31*A44*A52-A13*A26*     
     1 A31*A44*A52-A16*A21*A33*A44*A52+A11*A26*A33*
     1 A44*A52+A13*A21*A36*A44*   
     1 A52-A11*A23*A36*A44*A52-A14*A23*A31*A46*A52+
     1 A13*A24*A31*A46*A52+A14*A21*     
     1 A33*A46*A52-A11*A24*A33*A46*A52-A13*A21*A34*
     1 A46*A52+A11*A23*A34*A46*   
     1 A52-A16*A24*A32*A41*A53+A14*A26*A32*A41*A53+
     1 A16*A22*A34*A41*A53-A12*A26*     
     1 A34*A41*A53-A14*A22*A36*A41*A53+A12*A24*A36*
     1 A41*A53+A16*A24*A31*A42*   
     1 A53-A14*A26*A31*A42*A53-A16*A21*A34*A42*A53+
     1 A11*A26*A34*A42*A53+A14*A21*     
     1 A36*A42*A53-A11*A24*A36*A42*A53-A16*A22*A31*
     1 A44*A53+A12*A26*A31*A44*   
     1 A53+A16*A21*A32*A44*A53-A11*A26*A32*A44*A53-
     1 A12*A21*A36*A44*A53+A11*A22*     
     1 A36*A44*A53+A14*A22*A31*A46*A53-A12*A24*A31*
     1 A46*A53-A14*A21*A32*A46*   
     1 A53+A11*A24*A32*A46*A53+A12*A21*A34*A46*A53-
     1 A11*A22*A34*A46*A53+A16*A23*     
     1 A32*A41*A54-A13*A26*A32*A41*A54-A16*A22*A33*
     1 A41*A54+A12*A26*A33*A41*   
     1 A54+A13*A22*A36*A41*A54-A12*A23*A36*A41*A54-
     1 A16*A23*A31*A42*A54+A13*A26*     
     1 A31*A42*A54+A16*A21*A33*A42*A54-A11*A26*A33*
     1 A42*A54-A13*A21*A36*A42*   
     1 A54+A11*A23*A36*A42*A54+A16*A22*A31*A43*A54-
     1 A12*A26*A31*A43*A54-A16*A21*     
     1 A32*A43*A54+A11*A26*A32*A43*A54+A12*A21*A36*
     1 A43*A54-A11*A22*A36*A43*   
     1 A54-A13*A22*A31*A46*A54+A12*A23*A31*A46*A54+
     1 A13*A21*A32*A46*A54-A11*A23*     
     1 A32*A46*A54-A12*A21*A33*A46*A54+A11*A22*A33*
     1 A46*A54-A14*A23*A32*A41*   
     1 A56+A13*A24*A32*A41*A56+A14*A22*A33*A41*A56-
     1 A12*A24*A33*A41*A56-A13*A22*     
     1 A34*A41*A56+A12*A23*A34*A41*A56+A14*A23*A31*
     1 A42*A56-A13*A24*A31*A42*   
     1 A56-A14*A21*A33*A42*A56+A11*A24*A33*A42*A56+
     1 A13*A21*A34*A42*A56-A11*A23*     
     1 A34*A42*A56-A14*A22*A31*A43*A56+A12*A24*A31*
     1 A43*A56+A14*A21*A32*A43*   
     1 A56-A11*A24*A32*A43*A56-A12*A21*A34*A43*A56+
     1 A11*A22*A34*A43*A56+A13*A22*     
     1 A31*A44*A56-A12*A23*A31*A44*A56-A13*A21*A32*
     1 A44*A56+A11*A23*A32*A44*   
     1 A56+A12*A21*A33*A44*A56-A11*A22*A33*A44*A56

      COFACTOR(1,6) = -A25*A34*A43*A52*A61+A24*
     1 A35*A43*A52*A61+A25*A33*A44*A52*A61-A23*A35*
     1 A44*A52*A61-A24*A33*A45*A52*     
     1 A61+A23*A34*A45*A52*A61+A25*A34*A42*A53*A61-
     1 A24*A35*A42*A53*A61-A25*A32*     
     1 A44*A53*A61+A22*A35*A44*A53*A61+A24*A32*A45*
     1 A53*A61-A22*A34*A45*A53*   
     1 A61-A25*A33*A42*A54*A61+A23*A35*A42*A54*A61+
     1 A25*A32*A43*A54*A61-A22*A35*     
     1 A43*A54*A61-A23*A32*A45*A54*A61+A22*A33*A45*
     1 A54*A61+A24*A33*A42*A55*   
     1 A61-A23*A34*A42*A55*A61-A24*A32*A43*A55*A61+
     1 A22*A34*A43*A55*A61+A23*A32*     
     1 A44*A55*A61-A22*A33*A44*A55*A61+A25*A34*A43*
     1 A51*A62-A24*A35*A43*A51*   
     1 A62-A25*A33*A44*A51*A62+A23*A35*A44*A51*A62+
     1 A24*A33*A45*A51*A62-A23*A34*     
     1 A45*A51*A62-A25*A34*A41*A53*A62+A24*A35*A41*
     1 A53*A62+A25*A31*A44*A53*   
     1 A62-A21*A35*A44*A53*A62-A24*A31*A45*A53*A62+
     1 A21*A34*A45*A53*A62+A25*A33*     
     1 A41*A54*A62-A23*A35*A41*A54*A62-A25*A31*A43*
     1 A54*A62+A21*A35*A43*A54*   
     1 A62+A23*A31*A45*A54*A62-A21*A33*A45*A54*A62-
     1 A24*A33*A41*A55*A62+A23*A34*     
     1 A41*A55*A62+A24*A31*A43*A55*A62-A21*A34*A43*
     1 A55*A62-A23*A31*A44*A55*   
     1 A62+A21*A33*A44*A55*A62-A25*A34*A42*A51*A63+
     1 A24*A35*A42*A51*A63+A25*A32*     
     1 A44*A51*A63-A22*A35*A44*A51*A63-A24*A32*A45*
     1 A51*A63+A22*A34*A45*A51*   
     1 A63+A25*A34*A41*A52*A63-A24*A35*A41*A52*A63-
     1 A25*A31*A44*A52*A63+A21*A35*     
     1 A44*A52*A63+A24*A31*A45*A52*A63-A21*A34*A45*
     1 A52*A63-A25*A32*A41*A54*   
     1 A63+A22*A35*A41*A54*A63+A25*A31*A42*A54*A63-
     1 A21*A35*A42*A54*A63-A22*A31*     
     1 A45*A54*A63+A21*A32*A45*A54*A63+A24*A32*A41*
     1 A55*A63-A22*A34*A41*A55*   
     1 A63-A24*A31*A42*A55*A63+A21*A34*A42*A55*A63+
     1 A22*A31*A44*A55*A63-A21*A32*     
     1 A44*A55*A63+A25*A33*A42*A51*A64-A23*A35*A42*
     1 A51*A64-A25*A32*A43*A51*   
     1 A64+A22*A35*A43*A51*A64+A23*A32*A45*A51*A64-
     1 A22*A33*A45*A51*A64-A25*A33*     
     1 A41*A52*A64+A23*A35*A41*A52*A64+A25*A31*A43*
     1 A52*A64-A21*A35*A43*A52*   
     1 A64-A23*A31*A45*A52*A64+A21*A33*A45*A52*A64+
     1 A25*A32*A41*A53*A64-A22*A35*     
     1 A41*A53*A64-A25*A31*A42*A53*A64+A21*A35*A42*
     1 A53*A64+A22*A31*A45*A53*   
     1 A64-A21*A32*A45*A53*A64-A23*A32*A41*A55*A64+
     1 A22*A33*A41*A55*A64+A23*A31*     
     1 A42*A55*A64-A21*A33*A42*A55*A64-A22*A31*A43*
     1 A55*A64+A21*A32*A43*A55*   
     1 A64-A24*A33*A42*A51*A65+A23*A34*A42*A51*A65+
     1 A24*A32*A43*A51*A65-A22*A34*     
     1 A43*A51*A65-A23*A32*A44*A51*A65+A22*A33*A44*
     1 A51*A65+A24*A33*A41*A52*   
     1 A65-A23*A34*A41*A52*A65-A24*A31*A43*A52*A65+
     1 A21*A34*A43*A52*A65+A23*A31*     
     1 A44*A52*A65-A21*A33*A44*A52*A65-A24*A32*A41*
     1 A53*A65+A22*A34*A41*A53*   
     1 A65+A24*A31*A42*A53*A65-A21*A34*A42*A53*A65-
     1 A22*A31*A44*A53*A65+A21*A32*     
     1 A44*A53*A65+A23*A32*A41*A54*A65-A22*A33*A41*
     1 A54*A65-A23*A31*A42*A54*   
     1 A65+A21*A33*A42*A54*A65+A22*A31*A43*A54*A65-
     1 A21*A32*A43*A54*A65

      COFACTOR(2,6) = A15*A34*     
     1 A43*A52*A61-A14*A35*A43*A52*A61-A15*A33*A44*
     1 A52*A61+A13*A35*A44*A52*   
     1 A61+A14*A33*A45*A52*A61-A13*A34*A45*A52*A61-
     1 A15*A34*A42*A53*A61+A14*A35*     
     1 A42*A53*A61+A15*A32*A44*A53*A61-A12*A35*A44*
     1 A53*A61-A14*A32*A45*A53*   
     1 A61+A12*A34*A45*A53*A61+A15*A33*A42*A54*A61-
     1 A13*A35*A42*A54*A61-A15*A32*     
     1 A43*A54*A61+A12*A35*A43*A54*A61+A13*A32*A45*
     1 A54*A61-A12*A33*A45*A54*   
     1 A61-A14*A33*A42*A55*A61+A13*A34*A42*A55*A61+
     1 A14*A32*A43*A55*A61-A12*A34*     
     1 A43*A55*A61-A13*A32*A44*A55*A61+A12*A33*A44*
     1 A55*A61-A15*A34*A43*A51*   
     1 A62+A14*A35*A43*A51*A62+A15*A33*A44*A51*A62-
     1 A13*A35*A44*A51*A62-A14*A33*     
     1 A45*A51*A62+A13*A34*A45*A51*A62+A15*A34*A41*
     1 A53*A62-A14*A35*A41*A53*   
     1 A62-A15*A31*A44*A53*A62+A11*A35*A44*A53*A62+
     1 A14*A31*A45*A53*A62-A11*A34*     
     1 A45*A53*A62-A15*A33*A41*A54*A62+A13*A35*A41*
     1 A54*A62+A15*A31*A43*A54*   
     1 A62-A11*A35*A43*A54*A62-A13*A31*A45*A54*A62+
     1 A11*A33*A45*A54*A62+A14*A33*     
     1 A41*A55*A62-A13*A34*A41*A55*A62-A14*A31*A43*
     1 A55*A62+A11*A34*A43*A55*   
     1 A62+A13*A31*A44*A55*A62-A11*A33*A44*A55*A62+
     1 A15*A34*A42*A51*A63-A14*A35*     
     1 A42*A51*A63-A15*A32*A44*A51*A63+A12*A35*A44*
     1 A51*A63+A14*A32*A45*A51*   
     1 A63-A12*A34*A45*A51*A63-A15*A34*A41*A52*A63+
     1 A14*A35*A41*A52*A63+A15*A31*     
     1 A44*A52*A63-A11*A35*A44*A52*A63-A14*A31*A45*
     1 A52*A63+A11*A34*A45*A52*   
     1 A63+A15*A32*A41*A54*A63-A12*A35*A41*A54*A63-
     1 A15*A31*A42*A54*A63+A11*A35*     
     1 A42*A54*A63+A12*A31*A45*A54*A63-A11*A32*A45*
     1 A54*A63-A14*A32*A41*A55*   
     1 A63+A12*A34*A41*A55*A63+A14*A31*A42*A55*A63-
     1 A11*A34*A42*A55*A63-A12*A31*     
     1 A44*A55*A63+A11*A32*A44*A55*A63-A15*A33*A42*
     1 A51*A64+A13*A35*A42*A51*   
     1 A64+A15*A32*A43*A51*A64-A12*A35*A43*A51*A64-
     1 A13*A32*A45*A51*A64+A12*A33*     
     1 A45*A51*A64+A15*A33*A41*A52*A64-A13*A35*A41*
     1 A52*A64-A15*A31*A43*A52*   
     1 A64+A11*A35*A43*A52*A64+A13*A31*A45*A52*A64-
     1 A11*A33*A45*A52*A64-A15*A32*     
     1 A41*A53*A64+A12*A35*A41*A53*A64+A15*A31*A42*
     1 A53*A64-A11*A35*A42*A53*   
     1 A64-A12*A31*A45*A53*A64+A11*A32*A45*A53*A64+
     1 A13*A32*A41*A55*A64-A12*A33*     
     1 A41*A55*A64-A13*A31*A42*A55*A64+A11*A33*A42*
     1 A55*A64+A12*A31*A43*A55*   
     1 A64-A11*A32*A43*A55*A64+A14*A33*A42*A51*A65-
     1 A13*A34*A42*A51*A65-A14*A32*     
     1 A43*A51*A65+A12*A34*A43*A51*A65+A13*A32*A44*
     1 A51*A65-A12*A33*A44*A51*   
     1 A65-A14*A33*A41*A52*A65+A13*A34*A41*A52*A65+
     1 A14*A31*A43*A52*A65-A11*A34*     
     1 A43*A52*A65-A13*A31*A44*A52*A65+A11*A33*A44*
     1 A52*A65+A14*A32*A41*A53*   
     1 A65-A12*A34*A41*A53*A65-A14*A31*A42*A53*A65+
     1 A11*A34*A42*A53*A65+A12*A31*     
     1 A44*A53*A65-A11*A32*A44*A53*A65-A13*A32*A41*
     1 A54*A65+A12*A33*A41*A54*   
     1 A65+A13*A31*A42*A54*A65-A11*A33*A42*A54*A65-
     1 A12*A31*A43*A54*A65+A11*A32*     
     1 A43*A54*A65

      COFACTOR(3,6) = -A15*A24*A43*A52*A61+A14*A25*
     1 A43*A52*A61+A15*A23*A44*A52*    
     1 A61-A13*A25*A44*A52*A61-A14*A23*A45*A52*A61+
     1 A13*A24*A45*A52*A61+A15*A24*     
     1 A42*A53*A61-A14*A25*A42*A53*A61-A15*A22*A44*
     1 A53*A61+A12*A25*A44*A53*   
     1 A61+A14*A22*A45*A53*A61-A12*A24*A45*A53*A61-
     1 A15*A23*A42*A54*A61+A13*A25*     
     1 A42*A54*A61+A15*A22*A43*A54*A61-A12*A25*A43*
     1 A54*A61-A13*A22*A45*A54*   
     1 A61+A12*A23*A45*A54*A61+A14*A23*A42*A55*A61-
     1 A13*A24*A42*A55*A61-A14*A22*     
     1 A43*A55*A61+A12*A24*A43*A55*A61+A13*A22*A44*
     1 A55*A61-A12*A23*A44*A55*   
     1 A61+A15*A24*A43*A51*A62-A14*A25*A43*A51*A62-
     1 A15*A23*A44*A51*A62+A13*A25*     
     1 A44*A51*A62+A14*A23*A45*A51*A62-A13*A24*A45*
     1 A51*A62-A15*A24*A41*A53*   
     1 A62+A14*A25*A41*A53*A62+A15*A21*A44*A53*A62-
     1 A11*A25*A44*A53*A62-A14*A21*     
     1 A45*A53*A62+A11*A24*A45*A53*A62+A15*A23*A41*
     1 A54*A62-A13*A25*A41*A54*   
     1 A62-A15*A21*A43*A54*A62+A11*A25*A43*A54*A62+
     1 A13*A21*A45*A54*A62-A11*A23*     
     1 A45*A54*A62-A14*A23*A41*A55*A62+A13*A24*A41*
     1 A55*A62+A14*A21*A43*A55*   
     1 A62-A11*A24*A43*A55*A62-A13*A21*A44*A55*A62+
     1 A11*A23*A44*A55*A62-A15*A24*     
     1 A42*A51*A63+A14*A25*A42*A51*A63+A15*A22*A44*
     1 A51*A63-A12*A25*A44*A51*   
     1 A63-A14*A22*A45*A51*A63+A12*A24*A45*A51*A63+
     1 A15*A24*A41*A52*A63-A14*A25*     
     1 A41*A52*A63-A15*A21*A44*A52*A63+A11*A25*A44*
     1 A52*A63+A14*A21*A45*A52*   
     1 A63-A11*A24*A45*A52*A63-A15*A22*A41*A54*A63+
     1 A12*A25*A41*A54*A63+A15*A21*     
     1 A42*A54*A63-A11*A25*A42*A54*A63-A12*A21*A45*
     1 A54*A63+A11*A22*A45*A54*   
     1 A63+A14*A22*A41*A55*A63-A12*A24*A41*A55*A63-
     1 A14*A21*A42*A55*A63+A11*A24*     
     1 A42*A55*A63+A12*A21*A44*A55*A63-A11*A22*A44*
     1 A55*A63+A15*A23*A42*A51*   
     1 A64-A13*A25*A42*A51*A64-A15*A22*A43*A51*A64+
     1 A12*A25*A43*A51*A64+A13*A22*     
     1 A45*A51*A64-A12*A23*A45*A51*A64-A15*A23*A41*
     1 A52*A64+A13*A25*A41*A52*   
     1 A64+A15*A21*A43*A52*A64-A11*A25*A43*A52*A64-
     1 A13*A21*A45*A52*A64+A11*A23*     
     1 A45*A52*A64+A15*A22*A41*A53*A64-A12*A25*A41*
     1 A53*A64-A15*A21*A42*A53*   
     1 A64+A11*A25*A42*A53*A64+A12*A21*A45*A53*A64-
     1 A11*A22*A45*A53*A64-A13*A22*     
     1 A41*A55*A64+A12*A23*A41*A55*A64+A13*A21*A42*
     1 A55*A64-A11*A23*A42*A55*   
     1 A64-A12*A21*A43*A55*A64+A11*A22*A43*A55*A64-
     1 A14*A23*A42*A51*A65+A13*A24*     
     1 A42*A51*A65+A14*A22*A43*A51*A65-A12*A24*A43*
     1 A51*A65-A13*A22*A44*A51*   
     1 A65+A12*A23*A44*A51*A65+A14*A23*A41*A52*A65-
     1 A13*A24*A41*A52*A65-A14*A21*     
     1 A43*A52*A65+A11*A24*A43*A52*A65+A13*A21*A44*
     1 A52*A65-A11*A23*A44*A52*   
     1 A65-A14*A22*A41*A53*A65+A12*A24*A41*A53*A65+
     1 A14*A21*A42*A53*A65-A11*A24*     
     1 A42*A53*A65-A12*A21*A44*A53*A65+A11*A22*A44*
     1 A53*A65+A13*A22*A41*A54*   
     1 A65-A12*A23*A41*A54*A65-A13*A21*A42*A54*A65+
     1 A11*A23*A42*A54*A65+A12*A21*     
     1 A43*A54*A65-A11*A22*A43*A54*A65

      COFACTOR(4,6) = A15*A24*A33*A52*A61-A14*A25*A33*A52* 
     1 A61-A15*A23*A34*A52*A61+A13*A25*A34*A52*A61+
     1 A14*A23*A35*A52*A61-A13*A24*     
     1 A35*A52*A61-A15*A24*A32*A53*A61+A14*A25*A32*
     1 A53*A61+A15*A22*A34*A53*   
     1 A61-A12*A25*A34*A53*A61-A14*A22*A35*A53*A61+
     1 A12*A24*A35*A53*A61+A15*A23*     
     1 A32*A54*A61-A13*A25*A32*A54*A61-A15*A22*A33*
     1 A54*A61+A12*A25*A33*A54*   
     1 A61+A13*A22*A35*A54*A61-A12*A23*A35*A54*A61-
     1 A14*A23*A32*A55*A61+A13*A24*     
     1 A32*A55*A61+A14*A22*A33*A55*A61-A12*A24*A33*
     1 A55*A61-A13*A22*A34*A55*   
     1 A61+A12*A23*A34*A55*A61-A15*A24*A33*A51*A62+
     1 A14*A25*A33*A51*A62+A15*A23*     
     1 A34*A51*A62-A13*A25*A34*A51*A62-A14*A23*A35*
     1 A51*A62+A13*A24*A35*A51*   
     1 A62+A15*A24*A31*A53*A62-A14*A25*A31*A53*A62-
     1 A15*A21*A34*A53*A62+A11*A25*     
     1 A34*A53*A62+A14*A21*A35*A53*A62-A11*A24*A35*
     1 A53*A62-A15*A23*A31*A54*   
     1 A62+A13*A25*A31*A54*A62+A15*A21*A33*A54*A62-
     1 A11*A25*A33*A54*A62-A13*A21*     
     1 A35*A54*A62+A11*A23*A35*A54*A62+A14*A23*A31*
     1 A55*A62-A13*A24*A31*A55*   
     1 A62-A14*A21*A33*A55*A62+A11*A24*A33*A55*A62+
     1 A13*A21*A34*A55*A62-A11*A23*     
     1 A34*A55*A62+A15*A24*A32*A51*A63-A14*A25*A32*
     1 A51*A63-A15*A22*A34*A51*   
     1 A63+A12*A25*A34*A51*A63+A14*A22*A35*A51*A63-
     1 A12*A24*A35*A51*A63-A15*A24*     
     1 A31*A52*A63+A14*A25*A31*A52*A63+A15*A21*A34*
     1 A52*A63-A11*A25*A34*A52*   
     1 A63-A14*A21*A35*A52*A63+A11*A24*A35*A52*A63+
     1 A15*A22*A31*A54*A63-A12*A25*     
     1 A31*A54*A63-A15*A21*A32*A54*A63+A11*A25*A32*
     1 A54*A63+A12*A21*A35*A54*   
     1 A63-A11*A22*A35*A54*A63-A14*A22*A31*A55*A63+
     1 A12*A24*A31*A55*A63+A14*A21*     
     1 A32*A55*A63-A11*A24*A32*A55*A63-A12*A21*A34*
     1 A55*A63+A11*A22*A34*A55*   
     1 A63-A15*A23*A32*A51*A64+A13*A25*A32*A51*A64+
     1 A15*A22*A33*A51*A64-A12*A25*     
     1 A33*A51*A64-A13*A22*A35*A51*A64+A12*A23*A35*
     1 A51*A64+A15*A23*A31*A52*   
     1 A64-A13*A25*A31*A52*A64-A15*A21*A33*A52*A64+
     1 A11*A25*A33*A52*A64+A13*A21*     
     1 A35*A52*A64-A11*A23*A35*A52*A64-A15*A22*A31*
     1 A53*A64+A12*A25*A31*A53*   
     1 A64+A15*A21*A32*A53*A64-A11*A25*A32*A53*A64-
     1 A12*A21*A35*A53*A64+A11*A22*     
     1 A35*A53*A64+A13*A22*A31*A55*A64-A12*A23*A31*
     1 A55*A64-A13*A21*A32*A55*   
     1 A64+A11*A23*A32*A55*A64+A12*A21*A33*A55*A64-
     1 A11*A22*A33*A55*A64+A14*A23*     
     1 A32*A51*A65-A13*A24*A32*A51*A65-A14*A22*A33*
     1 A51*A65+A12*A24*A33*A51*   
     1 A65+A13*A22*A34*A51*A65-A12*A23*A34*A51*A65-
     1 A14*A23*A31*A52*A65+A13*A24*     
     1 A31*A52*A65+A14*A21*A33*A52*A65-A11*A24*A33*
     1 A52*A65-A13*A21*A34*A52*   
     1 A65+A11*A23*A34*A52*A65+A14*A22*A31*A53*A65-
     1 A12*A24*A31*A53*A65-A14*A21*     
     1 A32*A53*A65+A11*A24*A32*A53*A65+A12*A21*A34*
     1 A53*A65-A11*A22*A34*A53*   
     1 A65-A13*A22*A31*A54*A65+A12*A23*A31*A54*A65+
     1 A13*A21*A32*A54*A65-A11*A23*     
     1 A32*A54*A65-A12*A21*A33*A54*A65+A11*A22*A33*A54*A65

      COFACTOR(5,6) = -A15*A24*A33*A42*  
     1 A61+A14*A25*A33*A42*A61+A15*A23*A34*A42*A61-
     1 A13*A25*A34*A42*A61-A14*A23*     
     1 A35*A42*A61+A13*A24*A35*A42*A61+A15*A24*A32*
     1 A43*A61-A14*A25*A32*A43*   
     1 A61-A15*A22*A34*A43*A61+A12*A25*A34*A43*A61+
     1 A14*A22*A35*A43*A61-A12*A24*     
     1 A35*A43*A61-A15*A23*A32*A44*A61+A13*A25*A32*
     1 A44*A61+A15*A22*A33*A44*   
     1 A61-A12*A25*A33*A44*A61-A13*A22*A35*A44*A61+
     1 A12*A23*A35*A44*A61+A14*A23*     
     1 A32*A45*A61-A13*A24*A32*A45*A61-A14*A22*A33*
     1 A45*A61+A12*A24*A33*A45*   
     1 A61+A13*A22*A34*A45*A61-A12*A23*A34*A45*A61+
     1 A15*A24*A33*A41*A62-A14*A25*     
     1 A33*A41*A62-A15*A23*A34*A41*A62+A13*A25*A34*
     1 A41*A62+A14*A23*A35*A41*   
     1 A62-A13*A24*A35*A41*A62-A15*A24*A31*A43*A62+
     1 A14*A25*A31*A43*A62+A15*A21*     
     1 A34*A43*A62-A11*A25*A34*A43*A62-A14*A21*A35*
     1 A43*A62+A11*A24*A35*A43*   
     1 A62+A15*A23*A31*A44*A62-A13*A25*A31*A44*A62-
     1 A15*A21*A33*A44*A62+A11*A25*     
     1 A33*A44*A62+A13*A21*A35*A44*A62-A11*A23*A35*
     1 A44*A62-A14*A23*A31*A45*   
     1 A62+A13*A24*A31*A45*A62+A14*A21*A33*A45*A62-
     1 A11*A24*A33*A45*A62-A13*A21*     
     1 A34*A45*A62+A11*A23*A34*A45*A62-A15*A24*A32*
     1 A41*A63+A14*A25*A32*A41*   
     1 A63+A15*A22*A34*A41*A63-A12*A25*A34*A41*A63-
     1 A14*A22*A35*A41*A63+A12*A24*     
     1 A35*A41*A63+A15*A24*A31*A42*A63-A14*A25*A31*
     1 A42*A63-A15*A21*A34*A42*   
     1 A63+A11*A25*A34*A42*A63+A14*A21*A35*A42*A63-
     1 A11*A24*A35*A42*A63-A15*A22*     
     1 A31*A44*A63+A12*A25*A31*A44*A63+A15*A21*A32*
     1 A44*A63-A11*A25*A32*A44*   
     1 A63-A12*A21*A35*A44*A63+A11*A22*A35*A44*A63+
     1 A14*A22*A31*A45*A63-A12*A24*     
     1 A31*A45*A63-A14*A21*A32*A45*A63+A11*A24*A32*
     1 A45*A63+A12*A21*A34*A45*   
     1 A63-A11*A22*A34*A45*A63+A15*A23*A32*A41*A64-
     1 A13*A25*A32*A41*A64-A15*A22*     
     1 A33*A41*A64+A12*A25*A33*A41*A64+A13*A22*A35*
     1 A41*A64-A12*A23*A35*A41*   
     1 A64-A15*A23*A31*A42*A64+A13*A25*A31*A42*A64+
     1 A15*A21*A33*A42*A64-A11*A25*     
     1 A33*A42*A64-A13*A21*A35*A42*A64+A11*A23*A35*
     1 A42*A64+A15*A22*A31*A43*   
     1 A64-A12*A25*A31*A43*A64-A15*A21*A32*A43*A64+
     1 A11*A25*A32*A43*A64+A12*A21*     
     1 A35*A43*A64-A11*A22*A35*A43*A64-A13*A22*A31*
     1 A45*A64+A12*A23*A31*A45*   
     1 A64+A13*A21*A32*A45*A64-A11*A23*A32*A45*A64-
     1 A12*A21*A33*A45*A64+A11*A22*     
     1 A33*A45*A64-A14*A23*A32*A41*A65+A13*A24*A32*
     1 A41*A65+A14*A22*A33*A41*   
     1 A65-A12*A24*A33*A41*A65-A13*A22*A34*A41*A65+
     1 A12*A23*A34*A41*A65+A14*A23*     
     1 A31*A42*A65-A13*A24*A31*A42*A65-A14*A21*A33*
     1 A42*A65+A11*A24*A33*A42*   
     1 A65+A13*A21*A34*A42*A65-A11*A23*A34*A42*A65-
     1 A14*A22*A31*A43*A65+A12*A24*     
     1 A31*A43*A65+A14*A21*A32*A43*A65-A11*A24*A32*
     1 A43*A65-A12*A21*A34*A43*   
     1 A65+A11*A22*A34*A43*A65+A13*A22*A31*A44*A65-
     1 A12*A23*A31*A44*A65-A13*A21*     
     1 A32*A44*A65+A11*A23*A32*A44*A65+A12*A21*A33*
     1 A44*A65-A11*A22*A33*A44*   
     1 A65

      COFACTOR(6,6) = A15*A24*A33*A42*A51-A14*A25*
     1 A33*A42*A51-A15*A23*A34*A42*A51+A13*A25*     
     1 A34*A42*A51+A14*A23*A35*A42*A51-A13*A24*A35*
     1 A42*A51-A15*A24*A32*A43*   
     1 A51+A14*A25*A32*A43*A51+A15*A22*A34*A43*A51-
     1 A12*A25*A34*A43*A51-A14*A22*     
     1 A35*A43*A51+A12*A24*A35*A43*A51+A15*A23*A32*
     1 A44*A51-A13*A25*A32*A44*   
     1 A51-A15*A22*A33*A44*A51+A12*A25*A33*A44*A51+
     1 A13*A22*A35*A44*A51-A12*A23*     
     1 A35*A44*A51-A14*A23*A32*A45*A51+A13*A24*A32*
     1 A45*A51+A14*A22*A33*A45*   
     1 A51-A12*A24*A33*A45*A51-A13*A22*A34*A45*A51+
     1 A12*A23*A34*A45*A51-A15*A24*     
     1 A33*A41*A52+A14*A25*A33*A41*A52+A15*A23*A34*
     1 A41*A52-A13*A25*A34*A41*   
     1 A52-A14*A23*A35*A41*A52+A13*A24*A35*A41*A52+
     1 A15*A24*A31*A43*A52-A14*A25*     
     1 A31*A43*A52-A15*A21*A34*A43*A52+A11*A25*A34*
     1 A43*A52+A14*A21*A35*A43*   
     1 A52-A11*A24*A35*A43*A52-A15*A23*A31*A44*A52+
     1 A13*A25*A31*A44*A52+A15*A21*     
     1 A33*A44*A52-A11*A25*A33*A44*A52-A13*A21*A35*
     1 A44*A52+A11*A23*A35*A44*   
     1 A52+A14*A23*A31*A45*A52-A13*A24*A31*A45*A52-
     1 A14*A21*A33*A45*A52+A11*A24*     
     1 A33*A45*A52+A13*A21*A34*A45*A52-A11*A23*A34*
     1 A45*A52+A15*A24*A32*A41*   
     1 A53-A14*A25*A32*A41*A53-A15*A22*A34*A41*A53+
     1 A12*A25*A34*A41*A53+A14*A22*     
     1 A35*A41*A53-A12*A24*A35*A41*A53-A15*A24*A31*
     1 A42*A53+A14*A25*A31*A42*   
     1 A53+A15*A21*A34*A42*A53-A11*A25*A34*A42*A53-
     1 A14*A21*A35*A42*A53+A11*A24*     
     1 A35*A42*A53+A15*A22*A31*A44*A53-A12*A25*A31*
     1 A44*A53-A15*A21*A32*A44*   
     1 A53+A11*A25*A32*A44*A53+A12*A21*A35*A44*A53-
     1 A11*A22*A35*A44*A53-A14*A22*     
     1 A31*A45*A53+A12*A24*A31*A45*A53+A14*A21*A32*
     1 A45*A53-A11*A24*A32*A45*   
     1 A53-A12*A21*A34*A45*A53+A11*A22*A34*A45*A53-
     1 A15*A23*A32*A41*A54+A13*A25*     
     1 A32*A41*A54+A15*A22*A33*A41*A54-A12*A25*A33*
     1 A41*A54-A13*A22*A35*A41*   
     1 A54+A12*A23*A35*A41*A54+A15*A23*A31*A42*A54-
     1 A13*A25*A31*A42*A54-A15*A21*     
     1 A33*A42*A54+A11*A25*A33*A42*A54+A13*A21*A35*
     1 A42*A54-A11*A23*A35*A42*   
     1 A54-A15*A22*A31*A43*A54+A12*A25*A31*A43*A54+
     1 A15*A21*A32*A43*A54-A11*A25*     
     1 A32*A43*A54-A12*A21*A35*A43*A54+A11*A22*A35*
     1 A43*A54+A13*A22*A31*A45*   
     1 A54-A12*A23*A31*A45*A54-A13*A21*A32*A45*A54+
     1 A11*A23*A32*A45*A54+A12*A21*     
     1 A33*A45*A54-A11*A22*A33*A45*A54+A14*A23*A32*
     1 A41*A55-A13*A24*A32*A41*   
     1 A55-A14*A22*A33*A41*A55+A12*A24*A33*A41*A55+
     1 A13*A22*A34*A41*A55-A12*A23*     
     1 A34*A41*A55-A14*A23*A31*A42*A55+A13*A24*A31*
     1 A42*A55+A14*A21*A33*A42*   
     1 A55-A11*A24*A33*A42*A55-A13*A21*A34*A42*A55+
     1 A11*A23*A34*A42*A55+A14*A22*     
     1 A31*A43*A55-A12*A24*A31*A43*A55-A14*A21*A32*
     1 A43*A55+A11*A24*A32*A43*   
     1 A55+A12*A21*A34*A43*A55-A11*A22*A34*A43*A55-
     1 A13*A22*A31*A44*A55+A12*A23*     
     1 A31*A44*A55+A13*A21*A32*A44*A55-A11*A23*A32*
     1 A44*A55-A12*A21*A33*A44*   
     1 A55+A11*A22*A33*A44*A55

      AINV=TRANSPOSE(COFACTOR)/DET
      RETURN
      END SUBROUTINE M66INV

